//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Actors\CActor.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CACTOR_H
#define CACTOR_H

#include "..\Application\CDX11Object.h"
#include "..\Utilities\CDebugError.h"
#include "..\Math\CMathVector2.h"
#include "..\Math\CMathVector3.h"
#include "..\Math\CMathQuaternion.h"
#include "..\Math\CMathTransform.h"

namespace Actor
{
	// Actor class
	// Primarily used as a base class for creating dynamic characters for the game.
	class CActor : public App::CDX11Object
	{
	public:
		CActor();
		CActor(const CActor& other);
		virtual ~CActor();

		virtual int							Initialize() { return Util::Err::NONE; }
		virtual int							Update() { return Util::Err::NONE; }
		virtual int							Render() { return Util::Err::NONE; }

		virtual int							PreResize() { return Util::Err::NONE; }
		virtual int							PostResize() { return Util::Err::NONE; }

		virtual void						Release() { return; }
		
		inline Math::CVector3				Position() const;
		inline void							Position(const Math::CVector3& position);
		inline Math::CVector3				EulerAngles() const;
		inline void							EulerAngles(const Math::CVector3& eulerAngles);

		inline Math::CVector3				Velocity() const;
		inline void							Velocity(const Math::CVector3& velocity);
		inline Math::CVector3				TargetVelocity() const;
		inline void							TargetVelocity(const Math::CVector3& targetVelocity);

		inline float						Acceleration() const;
		inline void							Acceleration(float acceleration);
		inline float						Speed() const;
		inline void							Speed(float speed);

		inline Math::CTransformSharedPtr	Transform() const;
		inline void							Transform(Math::CTransformSharedPtr pTransform);

	protected:
		Math::CVector3						m_position;
		Math::CVector3						m_eulerAngles;

		Math::CVector3						m_targetVelocity;
		Math::CVector3						m_velocity;

		float								m_acceleration;
		float								m_speed;
		
		Math::CTransformSharedPtr			m_pTransform;

	private:
	};
	
	// Position getter and setter.

	inline Math::CVector3 CActor::Position() const
	{
		return m_position;
	}

	inline void CActor::Position(const Math::CVector3& position)
	{
		m_position = position;
	}
	
	// Euler angles getter and setter.

	inline Math::CVector3 CActor::EulerAngles() const
	{
		return m_eulerAngles;
	}

	inline void CActor::EulerAngles(const Math::CVector3& eulerAngles)
	{
		m_eulerAngles = eulerAngles;
	}
	
	// Velocity getter and setter.

	inline Math::CVector3 CActor::Velocity() const
	{
		return m_velocity;
	}

	inline void CActor::Velocity(const Math::CVector3& velocity)
	{
		m_velocity = velocity;
	}
	
	// Target velocity getter and setter.

	inline Math::CVector3 CActor::TargetVelocity() const
	{
		return m_targetVelocity;
	}

	inline void CActor::TargetVelocity(const Math::CVector3& targetVelocity)
	{
		m_targetVelocity = targetVelocity;
	}
	
	// Acceleration getter and setter.

	inline float CActor::Acceleration() const
	{
		return m_acceleration;
	}

	inline void CActor::Acceleration(float acceleration)
	{
		m_acceleration = acceleration;
	}
	
	// Speed getter and setter.

	inline float CActor::Speed() const
	{
		return m_speed;
	}

	inline void CActor::Speed(float speed)
	{
		m_speed = speed;
	}
	
	// Transform getter and setter.

	inline Math::CTransformSharedPtr CActor::Transform() const
	{
		return m_pTransform;
	}

	inline void CActor::Transform(Math::CTransformSharedPtr pTransform)
	{
		m_pTransform = pTransform;
	}
};

#endif