//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Application\CDX11Data.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CDX11DATA_H
#define CDX11DATA_H

#include <d3d11.h>
#include "..\Utilities\CMemoryFree.h"
#include "..\Math\CMathColor.h"

// Enumerator for render target clear modes.
enum DX11_CLEAR_MODE
{
	DX11_CLEAR_NONE		= 0,
	DX11_CLEAR_DEPTH	= 1,
	DX11_CLEAR_COLOR	= 3,
};

namespace App
{
	struct DX11DATA
	{
		Math::CColor						backgroundColor;

		IDXGISwapChain*						pSwapChain;
		ID3D11Device*						pDev;
		ID3D11DeviceContext*				pDevCon;
		ID3D11RenderTargetView*				pRenderTargetView;
		ID3D11Texture2D*					pDepthStencilBuffer;
		ID3D11DepthStencilState*			pDepthStencilState;
		ID3D11DepthStencilView*				pDepthStencilView;
		ID3D11ShaderResourceView*			pDepthStencilResView;
		D3D11_VIEWPORT						viewport;

		DXGI_SAMPLE_DESC					samplerDesc;

		DXGI_ADAPTER_DESC					adapterDesc;
		u32									videoCardMemory;
		char								videoCardDesc[128];

		DX11DATA()
		{
			backgroundColor					= Math::CColor(1.0f, 1.0f, 1.0f, 1.0f);
			pSwapChain						= NULL;
			pDev							= NULL;
			pDevCon							= NULL;
			pRenderTargetView				= NULL;
			pDepthStencilBuffer				= NULL;
			pDepthStencilState				= NULL;
			pDepthStencilView				= NULL;
			pDepthStencilResView			= NULL;

			samplerDesc.Count				= 1;
			samplerDesc.Quality				= 0;

			videoCardMemory					= 0;
		}

		~DX11DATA()
		{
			Release();
		}

		// Method for removing all directx 11 data from memory.
		void Release()
		{
			if(pSwapChain) pSwapChain->SetFullscreenState(false, NULL);

			SAFE_RELEASE(pDepthStencilResView);
			SAFE_RELEASE(pDepthStencilView);
			SAFE_RELEASE(pDepthStencilState);
			SAFE_RELEASE(pDepthStencilBuffer);
			SAFE_RELEASE(pRenderTargetView);
			SAFE_RELEASE(pDevCon);
			SAFE_RELEASE(pDev);
			SAFE_RELEASE(pSwapChain);
		}
	};
};

#endif
