//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Application\CDX11States.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CDXSTATES_H
#define CDXSTATES_H

#include <d3d11.h>
#include <d3dx11.h>
#include "CDX11Object.h"
#include "..\Utilities\CDebugError.h"

namespace App
{
	// Built-in blend state enumerator.
	enum class BlendState
	{
		None,
		Default,
		UI,
		Particle,
	};
	
	// Built-in reserizer state enumerator.
	enum class RasterizerState
	{
		Default,
		CullNone,
		Wireframe,
	};
	
	// Built-in sampler state enumerator.
	enum class SamplerState
	{
		Anisotropic,
		AnisotropicClamped,
		Linear,
		LinearClamped,
		Point,
		PointClamped,
	};

	// CDXStates class.
	// Class that stores built-in DirectX state objects.
	class CDXStates : public CDX11Object
	{
	public:
		CDXStates();
		CDXStates(const CDXStates& other);
		virtual ~CDXStates();

		int									Initialize();

		int									SetBlend(const BlendState& state);
		int									SetRasterizer(const RasterizerState& state);
		int									BindSamplers();

		ID3D11BlendState*					GetBlend(const BlendState& state) const;
		ID3D11RasterizerState*				GetRasterizer(const RasterizerState& state) const;
		ID3D11SamplerState*					GetSampler(const SamplerState& state) const;

		void								Release();

	private:
		int									InitializeBlendStates();
		int									InitializeRasterizerStates();
		int									InitializeSamplerStates();

		ID3D11BlendState*					m_pBlendNone;
		ID3D11BlendState*					m_pBlendDefault;
		ID3D11BlendState*					m_pBlendUI;
		ID3D11BlendState*					m_pBlendParticle;

		ID3D11RasterizerState*				m_pRasterizerStateDefault;
		ID3D11RasterizerState*				m_pRasterizerStateCullNone;
		ID3D11RasterizerState*				m_pRasterizerStateWireframe;
		
		ID3D11SamplerState*					m_pSamplerStateAnisotropic;
		ID3D11SamplerState*					m_pSamplerStateLinear;
		ID3D11SamplerState*					m_pSamplerStatePoint;
		ID3D11SamplerState*					m_pSamplerStateAnisotropicClamped;
		ID3D11SamplerState*					m_pSamplerStateLinearClamped;
		ID3D11SamplerState*					m_pSamplerStatePointClamped;
	};
};

#endif