//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Application\CInputControls.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CInputControls.h"

namespace App
{
	CControls::CControls()
	{
		m_pInput = NULL;
		m_pTimer = NULL;
	}

	CControls::~CControls()
	{
		Release();
	}

	void CControls::Initialize(CInput*& input, Util::CTimer*& timer)
	{
		m_pInput = input;
		m_pTimer = timer;
	}

	// Method for updating the state keybinds primary and secondary values.
	void CControls::Update()
	{
		KEY_BIND_OPTION optionTemp;
		for(u32 i = 0; i < m_sKeyBindStack.Length(); i++)
		{
			UpdateKeyBind(m_sKeyBindStack[i].primary);
			UpdateKeyBind(m_sKeyBindStack[i].secondary);
		}
	}

	// Method for updating a key bind sturcture.
	void CControls::UpdateKeyBind(KEY_BIND_OPTION::KEY_BIND& keyBind)
	{
		switch(keyBind.bindType)
		{
			case KEY_BIND_TYPE::Keyboard:
				// Update keyboard input type.
				if(keyBind.allowHold && m_pInput->Key(keyBind.code))
				{
					keyBind.activated = true;
					keyBind.timer += m_pTimer->Delta();
				}
				else if(m_pInput->KeyDown(keyBind.code))
				{
					keyBind.activated = true;
				}
				else
				{
					keyBind.activated = false;
					keyBind.timer = 0.0f;
				}

				break;
			case KEY_BIND_TYPE::Mouse:
				// Update mouse input type.
				if(keyBind.allowHold && m_pInput->Mouse(keyBind.code))
				{
					keyBind.activated = true;
					keyBind.timer += m_pTimer->Delta();
				}
				else if(m_pInput->MouseDown(keyBind.code))
				{
					keyBind.activated = true;
				}
				else
				{
					keyBind.activated = false;
					keyBind.timer = 0.0f;
				}

				break;
			default:
				break;
		}
	}

	void CControls::AddKeyBind(const KEY_BIND_OPTION& keyBind)
	{
		if(!CheckForKeyBind(keyBind))
			m_sKeyBindStack.Push(keyBind);
	}

	// Method for checking is a key bind with a id or a name is already stored in the key bind stack.
	bool CControls::CheckForKeyBind(const KEY_BIND_OPTION& keyBind, KEY_BIND_OPTION* output)
	{
		KEY_BIND_OPTION optionTemp;
		for(u32 i = 0; i < m_sKeyBindStack.Length(); i++)
		{
			optionTemp = m_sKeyBindStack[i];
			if(strncmp(optionTemp.name, keyBind.name, KEY_BIND_NAME_MAX) == 0 || optionTemp.id == keyBind.id)
			{
				if(output)
					*output = optionTemp;

				return true;
			}
		}

		return false;
	}

	// Method for finding a key bind with a id.
	bool CControls::FindKeyBind(u32 id, KEY_BIND_OPTION& output)
	{
		for(u32 i = 0; i < m_sKeyBindStack.Length(); i++)
		{
			if(m_sKeyBindStack[i].id == id)
			{
				output = m_sKeyBindStack[i];
				return true;
			}
		}

		return false;
	}
	
	// Method for finding a key bind with a name.
	bool CControls::FindKeyBind(const std::string& name, KEY_BIND_OPTION& output)
	{
		for(u32 i = 0; i < m_sKeyBindStack.Length(); i++)
		{
			if(strncmp(m_sKeyBindStack[i].name, name.c_str(), min(name.length(), KEY_BIND_NAME_MAX)) == 0)
			{
				output = m_sKeyBindStack[i];
				return true;
			}
		}

		return false;
	}
	
	// Method for checking is a key bind is activated with a id.
	bool CControls::Activated(u32 id)
	{
		KEY_BIND_OPTION optionTemp;
		for(u32 i = 0; i < m_sKeyBindStack.Length(); i++)
		{
			optionTemp = m_sKeyBindStack[i];
			if(optionTemp.id == id && (optionTemp.primary.activated || optionTemp.secondary.activated))
			{
				return true;
			}
		}

		return false;
	}
	
	// Method for checking is a key bind is activated with a name.
	bool CControls::Activated(const std::string& name)
	{
		KEY_BIND_OPTION optionTemp;
		for(u32 i = 0; i < m_sKeyBindStack.Length(); i++)
		{
			optionTemp = m_sKeyBindStack[i];
			if(strncmp(optionTemp.name, name.c_str(), min(name.length(), KEY_BIND_NAME_MAX)) == 0 &&
				 (optionTemp.primary.activated || optionTemp.secondary.activated))
			{
				return true;
			}
		}

		return false;
	}

	void CControls::Release()
	{
		m_sKeyBindStack.Release();
	}
};