//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Audio\CAudioList.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CAudioList.h"

namespace Audio
{
	void CAudioList::SetEngine(CXAudio2Engine* engine)
	{
		m_pAudioEngine = engine;
	}

	// Method for adding a new audio source to the audio list. 
	// If the source already exist on the list, return the existing audio source.
	CXAudio2Source* CAudioList::AddAudioSource(const std::string& filename)
	{
		CXAudio2Source* audioSrc = FindAudioSource(filename);

		if(!audioSrc)
		{
			AUDIO audio;

			audioSrc = new CXAudio2Source();
			if(audioSrc->Initialize(filename, m_pAudioEngine) != Util::Err::NONE)
				return NULL;

			std::string md5 = filename;
			md5 = Math::CMD5(md5).HexDigest();
			memcpy(audio.md5, md5.c_str(), 32);
			audio.audioSource = audioSrc;

			m_sAudioStack.Push(audio);

			return audio.audioSource;
		}

		return audioSrc;
	}

	// Method for finding an audio source based on its file name.
	CXAudio2Source* CAudioList::FindAudioSource(const std::string& filename)
	{
		std::string md5 = Math::CMD5(filename).HexDigest();

		for(u32 i = 0; i < m_sAudioStack.Length(); i++)
		{
			if(strncmp(m_sAudioStack[i].md5, md5.c_str(), 32) == 0)
			{
				return m_sAudioStack[i].audioSource;
			}
		}

		return NULL;
	}

	// Method for release each of the audio sources fromt the audio list. 
	void CAudioList::Release()
	{
		for(u32 i = 0; i < m_sAudioStack.Length(); i++)
		{
			if(m_sAudioStack[i].audioSource)
				SAFE_RELEASE_DELETE(m_sAudioStack[i].audioSource);
		}

		m_sAudioStack.Release();

		SAFE_RELEASE_DELETE(m_pAudioEngine);
	}
};