//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CWaveFile.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CWaveFile.h"

namespace Audio
{
	CWaveFile::CWaveFile()
	{
	}

	CWaveFile::CWaveFile(const CWaveFile& other)
	{
	}

	CWaveFile::~CWaveFile()
	{
	}

	// Method for opening a .wav file. File should have a 44100 sample-rate, and a 16 bit-rate.
	int CWaveFile::OpenFile(const std::string& filename, WAVEFORMATEXTENSIBLE& wfx, XAUDIO2_BUFFER& buffer)
	{
		int retCode = Util::Err::NONE;

		FILE*				file;
		WAVEHEADERTYPE		waveFileHeader;

		// Open and read the contents of the specified wave file.
		if(fopen_s(&file, filename.c_str(), "rb") != 0)
		{
			return Util::Err::ERR_WAVEFILE_FAILED_TO_OPEN_FILE;
		}

		if(fread(&waveFileHeader, sizeof(waveFileHeader), 1, file) != 1)
		{
			return Util::Err::ERR_WAVEFILE_FAILED_TO_READ_FILE;
		}

		// Checkf for a valid file format.
		if((strncmp(waveFileHeader.chunkId, fourccRIFF, 4)) != 0)
		{
			return Util::Err::ERR_WAVEFILE_INVALID_FILE_FORMAT_RIFF;
		}

		if((strncmp(waveFileHeader.format, fourccWAVE, 4)) != 0)
		{
			return Util::Err::ERR_WAVEFILE_INVALID_FILE_FORMAT_WAVE;
		}

		if((strncmp(waveFileHeader.subChunkId, fourccFMT, 4)) != 0)
		{
			return Util::Err::ERR_WAVEFILE_INVALID_FILE_FORMAT_FMT;
		}
		
		if(waveFileHeader.audioFormat != WAVE_FORMAT_PCM)
		{
			return Util::Err::ERR_WAVEFILE_INVALID_FILE_FORMAT_PCM;
		}

		if(waveFileHeader.sampleRate != 44100)
		{
			return Util::Err::ERR_WAVEFILE_INVALID_SAMPLE_RATE;
		}
	
		if(waveFileHeader.bitsPerSample != 16)
		{
			return Util::Err::ERR_WAVEFILE_INVALID_BIT_RATE;
		}
		
		if((strncmp(waveFileHeader.dataChunkId, fourccDATA, 4)) != 0)
		{
			return Util::Err::ERR_WAVEFILE_INVALID_DATA_HEADER;
		}

		// Store file data in the passed wave format structure.
		wfx.Format.wFormatTag			= waveFileHeader.audioFormat;
		wfx.Format.nSamplesPerSec		= waveFileHeader.sampleRate;
		wfx.Format.wBitsPerSample		= waveFileHeader.bitsPerSample;
		wfx.Format.nChannels			= waveFileHeader.numChannels;
		wfx.Format.nBlockAlign			= (wfx.Format.wBitsPerSample / 8) * wfx.Format.nChannels;
		wfx.Format.nAvgBytesPerSec		= wfx.Format.nSamplesPerSec * wfx.Format.nBlockAlign;
		wfx.Format.cbSize				= 0;

		// Extract and store data from the opened wave file.
		fseek(file, sizeof(WAVEHEADERTYPE), SEEK_SET);

		u8* pDataBuffer = new unsigned char[waveFileHeader.dataSize];
		
		if(fread(pDataBuffer, 1, waveFileHeader.dataSize, file) != waveFileHeader.dataSize)
		{
			return Util::Err::ERR_WAVEFILE_INVALID_FILE_LENGTH;
		}
	
		fclose(file);
		
		buffer.AudioBytes = waveFileHeader.dataSize;
		buffer.pAudioData = pDataBuffer;
		buffer.Flags = XAUDIO2_END_OF_STREAM;

		return retCode;
	}
};