//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CConstantBuffer.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CConstantBuffer.h"

namespace GFX
{
	CConstantBuffer::CConstantBuffer()
	{
		m_bufferWidth		= 0;
		m_pCBuffer			= NULL;
	}

	CConstantBuffer::CConstantBuffer(const CConstantBuffer& other)
	{
	}

	CConstantBuffer::~CConstantBuffer()
	{
	}

	// Method for initializing a constant buffer.
	int CConstantBuffer::Initialize(u32 bufferWidth, bool dynamic)
	{
		int retCode = Util::Err::NONE;
		
		D3D11_BUFFER_DESC bd;
		ZeroMemory(&bd, sizeof(bd));
		bd.ByteWidth			= m_bufferWidth = bufferWidth;
		bd.BindFlags			= D3D11_BIND_CONSTANT_BUFFER;

		bd.Usage				= dynamic ? D3D11_USAGE_DYNAMIC : D3D11_USAGE_DEFAULT;
		bd.CPUAccessFlags		= dynamic ? D3D11_CPU_ACCESS_WRITE : 0;

		if(FAILED(m_pDev->CreateBuffer(&bd, NULL, &m_pCBuffer)))
		{
			return Util::Err::ERR_SHADER_FAILED_TO_CREATE_CONSTANT_BUFFER;
		}

		return retCode;
	}
	
	// Method for resetting a constant buffer with data.
	int CConstantBuffer::ResetBuffer(void* updatedData)
	{
		int retCode = Util::Err::NONE;

		D3D11_MAPPED_SUBRESOURCE mappedSubRes;
		
		if(FAILED(m_pDevCon->Map(m_pCBuffer, 0, D3D11_MAP_WRITE_DISCARD, 0, &mappedSubRes)))
		{
			return Util::Err::ERR_SHADER_FAILED_TO_MAP_CONSTANT_BUFFER;
		}

		memcpy(mappedSubRes.pData, updatedData, m_bufferWidth);

		m_pDevCon->Unmap(m_pCBuffer, 0);

		return retCode;
	}

	// Method for updating a constant buffer's data.
	int CConstantBuffer::UpdateBuffer(void* updatedData)
	{
		int retCode = Util::Err::NONE;

		m_pDevCon->UpdateSubresource(m_pCBuffer, 0, NULL, updatedData, 0, 0);
		
		return retCode;
	}

	//
	// Binding methods.
	//

	// Vertex shader bind.
	int CConstantBuffer::BindBufferVS(u32 startSlot)
	{
		int retCode = Util::Err::NONE;

		m_pDevCon->VSSetConstantBuffers(startSlot, 1, &m_pCBuffer);

		return retCode;
	}
	
	// Hull shader bind.
	int CConstantBuffer::BindBufferHS(u32 startSlot)
	{
		int retCode = Util::Err::NONE;

		m_pDevCon->HSSetConstantBuffers(startSlot, 1, &m_pCBuffer);

		return retCode;
	}
	
	// Domain shader bind.
	int CConstantBuffer::BindBufferDS(u32 startSlot)
	{
		int retCode = Util::Err::NONE;

		m_pDevCon->DSSetConstantBuffers(startSlot, 1, &m_pCBuffer);

		return retCode;
	}
	
	// Pixel shader bind.
	int CConstantBuffer::BindBufferPS(u32 startSlot)
	{
		int retCode = Util::Err::NONE;

		m_pDevCon->PSSetConstantBuffers(startSlot, 1, &m_pCBuffer);

		return retCode;
	}

	// Gemoetry shader bind.
	int CConstantBuffer::BindBufferGS(u32 startSlot)
	{
		int retCode = Util::Err::NONE;

		m_pDevCon->GSSetConstantBuffers(startSlot, 1, &m_pCBuffer);

		return retCode;
	}

	// Compute shader bind.
	int CConstantBuffer::BindBufferCS(u32 startSlot)
	{
		int retCode = Util::Err::NONE;

		m_pDevCon->CSSetConstantBuffers(startSlot, 1, &m_pCBuffer);

		return retCode;
	}

	void CConstantBuffer::Release()
	{
		SAFE_RELEASE(m_pCBuffer);
	}
};