//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CDebugBase.h
//
//-------------------------------------------------------------------------------------------------

#include "CDebugBase.h"

namespace GFX
{
	CDebugBase::CDebugBase()
	{
		m_bSetCBuffer						= true;

		m_pVertices							= NULL;
		m_pVertexBuffer						= NULL;

		m_pRenderData						= NULL;

		m_lineCount							= 0;
		m_vertexCount						= 0;
	}

	CDebugBase::CDebugBase(const CDebugBase& other)
	{
	}

	CDebugBase::~CDebugBase()
	{
	}
	
	// Method for initializing a new debug object rendered with a line-list.
	int CDebugBase::Initialize()
	{
		int retCode = Util::Err::NONE;

		if(m_lineCount > 0)
		{
			DEBUG_LINE* line;
			u32 index = 0;

			m_vertexCount = m_lineCount * 2;
			m_pVertices = new DEBUG_VERTEX[m_vertexCount];

			while((line = (DEBUG_LINE*)m_lineQueue.Pop()))
			{
				m_pVertices[index + 0].Position = line->p0;
				m_pVertices[index + 1].Position = line->p1;
				delete line;

				index += 2;
			}
		}

		if(m_pVertices)
		{
			D3D11_BUFFER_DESC bd;

			ZeroMemory(&bd, sizeof(bd));

			// Vertex buffer description.
			bd.Usage						= D3D11_USAGE_DYNAMIC;
			bd.ByteWidth					= m_vertexCount * sizeof(DEBUG_VERTEX);
			bd.BindFlags					= D3D11_BIND_VERTEX_BUFFER;
			bd.CPUAccessFlags				= D3D11_CPU_ACCESS_WRITE;
			bd.MiscFlags					= 0;
			bd.StructureByteStride			= 0;

			// Create the vertex buffer.
			if(FAILED(m_pDev->CreateBuffer(&bd, NULL, &m_pVertexBuffer)))
			{
				return Util::Err::ERR_DEBUG_OBJECT_FAILED_TO_CREATE_VERTEX_BUFFER;
			}

			D3D11_MAPPED_SUBRESOURCE ms;

			m_pDevCon->Map(m_pVertexBuffer, NULL, D3D11_MAP_WRITE_DISCARD, NULL, &ms);
			memcpy(ms.pData, m_pVertices, m_vertexCount * sizeof(DEBUG_VERTEX));
			m_pDevCon->Unmap(m_pVertexBuffer, NULL);
		}

		m_debugCBuffer.color = Math::CColor(1);
		m_debugCBuffer.scale = Math::CVector3(1, 1, 1);
		
		return retCode;
	}

	// Method for rendering a debug object as a line-list.
	int CDebugBase::Render()
	{
		int retCode = Util::Err::NONE;

		if(m_bSetCBuffer)
		{
			((CMatDebug*)m_pMaterial)->m_debugCBuffer = m_debugCBuffer;
		}

		m_pMaterial->Transform(m_pTransform);

		if((retCode = m_pMaterial->Render()) != Util::Err::NONE)
			return retCode;

		u32 stride = sizeof(DEBUG_VERTEX);
		u32 offset = 0;
		m_pDevCon->IASetVertexBuffers(0, 1, &m_pVertexBuffer, &stride, &offset);
		m_pDevCon->IASetPrimitiveTopology(D3D11_PRIMITIVE_TOPOLOGY_LINELIST);
		m_pDevCon->Draw(m_vertexCount, 0);
		
		return retCode;
	}

	void CDebugBase::Release()
	{
		SAFE_RELEASE(m_pVertexBuffer);
		SAFE_DELETE_ARRAY(m_pVertices);
	}

	// Method for adding a line to the queue for processing on initialization.
	void CDebugBase::AddLine(DEBUG_LINE* line)
	{
		m_lineQueue.Push(line);
		m_lineCount++;
	}
};