//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CDebugCude.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CDebugCube.h"

namespace GFX
{
	CDebugCube::CDebugCube()
	{
	}

	CDebugCube::CDebugCube(const CDebugCube& other)
	{
	}

	CDebugCube::~CDebugCube()
	{
	}

	// Method for the debug cube's data.
	void CDebugCube::CubeExtents(const Math::CVector3& min, const Math::CVector3& max)
	{
		m_minBounds = min;
		m_maxBounds = max;
	}

	// Method for initializing the debug cube's lines.
	int CDebugCube::Initialize()
	{
		int retCode = Util::Err::NONE;

		// Top
		AddLine(new DEBUG_LINE(Math::CVector3(m_minBounds.x, m_maxBounds.y, m_minBounds.z), Math::CVector3(m_minBounds.x, m_maxBounds.y, m_maxBounds.z)));
		AddLine(new DEBUG_LINE(Math::CVector3(m_minBounds.x, m_maxBounds.y, m_maxBounds.z), Math::CVector3(m_maxBounds.x, m_maxBounds.y, m_maxBounds.z)));
		AddLine(new DEBUG_LINE(Math::CVector3(m_maxBounds.x, m_maxBounds.y, m_maxBounds.z), Math::CVector3(m_maxBounds.x, m_maxBounds.y, m_minBounds.z)));
		AddLine(new DEBUG_LINE(Math::CVector3(m_maxBounds.x, m_maxBounds.y, m_minBounds.z), Math::CVector3(m_minBounds.x, m_maxBounds.y, m_minBounds.z)));

		// Bottom
		AddLine(new DEBUG_LINE(Math::CVector3(m_minBounds.x, m_minBounds.y, m_minBounds.z), Math::CVector3(m_minBounds.x, m_minBounds.y, m_maxBounds.z)));
		AddLine(new DEBUG_LINE(Math::CVector3(m_minBounds.x, m_minBounds.y, m_maxBounds.z), Math::CVector3(m_maxBounds.x, m_minBounds.y, m_maxBounds.z)));
		AddLine(new DEBUG_LINE(Math::CVector3(m_maxBounds.x, m_minBounds.y, m_maxBounds.z), Math::CVector3(m_maxBounds.x, m_minBounds.y, m_minBounds.z)));
		AddLine(new DEBUG_LINE(Math::CVector3(m_maxBounds.x, m_minBounds.y, m_minBounds.z), Math::CVector3(m_minBounds.x, m_minBounds.y, m_minBounds.z)));

		// sides
		AddLine(new DEBUG_LINE(Math::CVector3(m_minBounds.x, m_minBounds.y, m_minBounds.z), Math::CVector3(m_minBounds.x, m_maxBounds.y, m_minBounds.z)));
		AddLine(new DEBUG_LINE(Math::CVector3(m_minBounds.x, m_minBounds.y, m_maxBounds.z), Math::CVector3(m_minBounds.x, m_maxBounds.y, m_maxBounds.z)));
		AddLine(new DEBUG_LINE(Math::CVector3(m_maxBounds.x, m_minBounds.y, m_maxBounds.z), Math::CVector3(m_maxBounds.x, m_maxBounds.y, m_maxBounds.z)));
		AddLine(new DEBUG_LINE(Math::CVector3(m_maxBounds.x, m_minBounds.y, m_minBounds.z), Math::CVector3(m_maxBounds.x, m_maxBounds.y, m_minBounds.z)));

		if((retCode = CDebugBase::Initialize()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	void CDebugCube::RecalculateBounds()
	{
		Math::CVector3 bounds0 = m_pTransform->WorldMatrix().TransformCoord(Math::CVector3(m_minBounds.x, m_minBounds.y, m_minBounds.z));
		Math::CVector3 bounds1 = m_pTransform->WorldMatrix().TransformCoord(Math::CVector3(m_maxBounds.x, m_minBounds.y, m_minBounds.z));
		Math::CVector3 bounds2 = m_pTransform->WorldMatrix().TransformCoord(Math::CVector3(m_minBounds.x, m_maxBounds.y, m_minBounds.z));
		Math::CVector3 bounds3 = m_pTransform->WorldMatrix().TransformCoord(Math::CVector3(m_minBounds.x, m_minBounds.y, m_maxBounds.z));
		Math::CVector3 bounds4 = m_pTransform->WorldMatrix().TransformCoord(Math::CVector3(m_maxBounds.x, m_maxBounds.y, m_minBounds.z));
		Math::CVector3 bounds5 = m_pTransform->WorldMatrix().TransformCoord(Math::CVector3(m_minBounds.x, m_maxBounds.y, m_maxBounds.z));
		Math::CVector3 bounds6 = m_pTransform->WorldMatrix().TransformCoord(Math::CVector3(m_maxBounds.x, m_minBounds.y, m_maxBounds.z));
		Math::CVector3 bounds7 = m_pTransform->WorldMatrix().TransformCoord(Math::CVector3(m_maxBounds.x, m_maxBounds.y, m_maxBounds.z));

		m_minBoundsTransformed = Math::CVector3::Min(
			Math::CVector3::Min(Math::CVector3::Min(Math::CVector3::Min(bounds0, bounds1), bounds2), bounds3),
			Math::CVector3::Min(Math::CVector3::Min(Math::CVector3::Min(bounds4, bounds5), bounds6), bounds7)
		);

		m_maxBoundsTransformed = Math::CVector3::Max(
			Math::CVector3::Max(Math::CVector3::Max(Math::CVector3::Max(bounds0, bounds1), bounds2), bounds3),
			Math::CVector3::Max(Math::CVector3::Max(Math::CVector3::Max(bounds4, bounds5), bounds6), bounds7)
		);
	}

	int CDebugCube::Render()
	{
		int retCode = Util::Err::NONE;

		m_cubeAABB = Math::CAABB(m_minBoundsTransformed, m_maxBoundsTransformed, m_pTransform->Position());

		if(m_pMaterial->Camera()->Frustum().TestAABB(m_cubeAABB))
			if((retCode = CDebugBase::Render()) != Util::Err::NONE)
				return retCode;

		return retCode;
	}
};