//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CDebugSphere.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CDebugSphere.h"

namespace GFX
{
	CDebugSphere::CDebugSphere()
	{
		m_bLookAtCamera						= false;

		m_iterations						= 0;

		m_pTargetCamera						= NULL;
		m_pCircle							= NULL;
	}

	CDebugSphere::CDebugSphere(const CDebugSphere& other)
	{
	}

	CDebugSphere::~CDebugSphere()
	{
	}
	
	// Method for setting the debug sphere's data.
	void CDebugSphere::SetupSphere(Math::CCamera*& targetCamera, float radius, u32 iterations)
	{
		m_pTargetCamera						= targetCamera;
		m_iterations						= iterations;
		m_radius							= radius;
		m_sphere.radius						= radius;
	}
	
	// Method for initialize the debug sphere's lines.
	int CDebugSphere::Initialize()
	{
		int retCode = Util::Err::NONE;
		
		float s0 = 0.0f;
		float c0 = m_sphere.radius;

		float s1, c1;
		float step = Math::_2Pi / m_iterations;
		float theta = step;

		// Create the circle out of lines.
		for(u32 i = 0; i < m_iterations; i++)
		{
			s1 = sinf(theta) * m_sphere.radius;
			c1 = cosf(theta) * m_sphere.radius;

			AddLine(new DEBUG_LINE(Math::CVector3(s0, 0.0f, c0), Math::CVector3(s1, 0.0f, c1)));

			s0 = s1;
			c0 = c1;

			theta += step;
		}

		if((retCode = CDebugBase::Initialize()) != Util::Err::NONE)
			return retCode;
		
		// Setup the debug circle.
		m_pCircle = new CDebugCircle();

		m_pCircle->DirectXData(m_pDX11Data);
		m_pCircle->SetupCircle(m_sphere.radius);
		m_pCircle->Material(m_pMaterial);
		m_pCircle->RenderData(m_pRenderData);

		m_pCircle->Transform(Math::CTransformSharedPtr(new Math::CTransform()));
		m_pCircle->Transform()->Update();

		if((retCode = m_pCircle->Initialize()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	// Function for rendering the debug sphere.
	int CDebugSphere::Render()
	{
		int retCode = Util::Err::NONE;

		if(m_bSetCBuffer)
			m_pCircle->m_debugCBuffer = m_debugCBuffer;
		
		Math::CVector3 scale = m_pTransform->Scale() * ((CMatDebug*)m_pMaterial)->m_debugCBuffer.scale;
		m_sphere.radius = m_radius * max(max(scale.x, scale.y), scale.z);
		m_sphere.center = m_pTransform->Position();
		
		if(m_pMaterial->Camera()->Frustum().TestSphere(m_sphere))
		{
			if((retCode = CDebugBase::Render()) != Util::Err::NONE)
				return retCode;
			
			m_pCircle->Transform()->Position(m_pTransform->Position());
			
			if(m_bLookAtCamera)
			{
				// Render a sphere with two circles, with one of the circles always looking at the target camera.
				m_pCircle->Transform()->LookAt((m_pTargetCamera->Transform()->Position() - m_pCircle->Transform()->Position()).Normalized());
				m_pCircle->Transform()->Update();

				if((retCode = m_pCircle->Render()) != Util::Err::NONE)
					return retCode;
			}
			else
			{
				// Render a sphere with three circles, with all of them having locked rotation.
				m_pCircle->Transform()->LookAt(Math::CVector3::Forward());
				m_pCircle->Transform()->Update();

				if((retCode = m_pCircle->Render()) != Util::Err::NONE)
					return retCode;

				m_pCircle->Transform()->LookAt(Math::CVector3::Right());
				m_pCircle->Transform()->Update();

				if((retCode = m_pCircle->Render()) != Util::Err::NONE)
					return retCode;
			}
		}

		return retCode;
	}

	void CDebugSphere::Release()
	{
		SAFE_RELEASE_DELETE(m_pCircle);

		CDebugBase::Release();
	}
};