//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CDebugSphere.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CDEBUGSPHERE_H
#define CDEBUGSPHERE_H

#include "CDebugBase.h"
#include "CDebugCircle.h"
#include "..\Math\CMathSphere.h"

namespace GFX
{
	// CDebugGrid class.
	// Class for creating and rendering a line-list sphere.
	class CDebugSphere : public CDebugBase
	{
	public:
		CDebugSphere();
		CDebugSphere(const CDebugSphere& other);
		virtual ~CDebugSphere();

		void								SetupSphere(Math::CCamera*& targetCamera, float radius, u32 iterations = 24);

		int									Initialize() override;
		int									Render() override;

		void								Release() override;

		inline bool							LookAtCamera() const;
		inline void							LookAtCamera(bool look);

		inline Math::CSphere				BoundingSphere() const;

		inline Math::CCamera*				TargetCamera() const;
		inline void							TargetCamera(Math::CCamera* pTargetCamera);

	private:
		CDebugCircle*						m_pCircle;
		Math::CCamera*						m_pTargetCamera;

		bool								m_bLookAtCamera;

		u32									m_iterations;
		float								m_radius;
		Math::CSphere						m_sphere;
	};

	// Look at camera getter and setter.

	inline bool CDebugSphere::LookAtCamera() const
	{
		return m_bLookAtCamera;
	}

	inline void CDebugSphere::LookAtCamera(bool look)
	{
		m_bLookAtCamera = look;
	}

	// Bounding sphere getter.
	inline Math::CSphere CDebugSphere::BoundingSphere() const
	{
		return m_sphere;
	}

	// Target camera getter and setter.

	inline Math::CCamera* CDebugSphere::TargetCamera() const
	{
		return m_pTargetCamera;
	}

	inline void CDebugSphere::TargetCamera(Math::CCamera* pTargetCamera) 
	{
		m_pTargetCamera = pTargetCamera;
	}
};

#endif
