//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CGeoObject.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CGEOOBJECT_H
#define CGEOOBJECT_H

#include "..\Utilities\CDataQueue.h"
#include "CGraphicalObject.h"
#include "CRenderingData.h"
#include "..\Utilities\CDebugLog.h"
#include "CDebugCube.h"

namespace GFX
{
	// CGeoObject class.
	// Base class for creating a new geometry object. Handles both normal rendering, and instanced rendering.
	// Handles both 16 and 32 bit index strides, and custom vertex structures, and strides.
	class CGeoObject : public CGraphicalObject
	{
	public:
		CGeoObject();
		CGeoObject(const CGeoObject& other);
		virtual ~CGeoObject();

		virtual int							Initialize();
		virtual int							Update();
		void								RecalculateBounds();
		virtual int							Render(CMaterial* material, bool count = false);
		virtual int							Render(bool count = false);
		virtual void						Release();

		inline CMaterial*					DebugMaterial();
		inline void							DebugMaterial(CMaterial* pMaterial);
		inline RENDERDATA*					RenderData();
		inline void							RenderData(RENDERDATA* pRenderData);

	protected:
		int									RenderDefault(bool count = false);
		int									RenderInstanced(bool count = false);

		void								AddVertex(void* v0);
		void								AddTriangle(void* v0, void* v1, void* v2, bool cw = true);
		void								AddQuad(void* v0, void* v1, void* v2, void* v3, bool cw = true);

		void								AddIndex(u32 index);

		bool								m_bEnableIndices;
		bool								m_bEnableInstancing;

		bool								m_bCreateBoundingAABB;
		bool								m_bRenderBoundingAABB;
		bool								m_bSeperateVerticesIndices;
		
		Math::CVector3						m_minBounds;
		Math::CVector3						m_maxBounds;
		Math::CVector3						m_minBoundsTransformed;
		Math::CVector3						m_maxBoundsTransformed;

		u8*									m_pBufferArray;
		void*								m_pVertices;
		void*								m_pInstances;
		u16*								m_pIndices16;
		u32*								m_pIndices32;
		
		u32									m_totalCount;
		u32									m_vertexCount;
		u32									m_vertexSize;
		u32									m_indexCount;
		u32									m_indexSize;
		u32									m_instanceCount;
		u32									m_instanceAllocation;
		u32									m_instanceSize;

		u32									m_vertexStride;
		u32									m_vertexPositionOffset;
		u32									m_instanceStride;
		u32									m_instancePositionOffset;
		u32									m_indexStride;

		D3D_PRIMITIVE_TOPOLOGY				m_primitiveTopology;

		RENDERDATA*							m_pRenderData;

	private:
		Util::CPtrQueue						m_vertexQueue;
		Util::CQueue<u32>					m_indexQueue;

		CMaterial*							m_pDebugMaterial;
		CDebugCube*							m_pDebugBounds;

		ID3D11Buffer*						m_pVertexBuffer;
		ID3D11Buffer*						m_pInstanceBuffer;
		ID3D11Buffer*						m_pIndexBuffer;
	};

	// Material getter and setter.

	inline CMaterial* CGeoObject::DebugMaterial()
	{
		return m_pDebugMaterial;
	}

	inline void CGeoObject::DebugMaterial(CMaterial* pMaterial)
	{
		m_pDebugMaterial = pMaterial;
	}

	// Render data getter and setter.

	inline RENDERDATA* CGeoObject::RenderData()
	{
		return m_pRenderData;
	}

	inline void CGeoObject::RenderData(RENDERDATA* pRenderData)
	{
		m_pRenderData = pRenderData;
	}
};

#endif