//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CLightDirectional.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CLightDirectional.h"

namespace GFX
{
	CLightDirectional::CLightDirectional()
	{
		m_dirLightCount						= 0;

		m_shadowMapSize						= 1024;
		
		m_dirShadowMapCount					= 0;
		m_pDirLightArray					= NULL;
		m_pDirLightPOVs						= NULL;
		m_pDirShadowMaps					= NULL;
		
		m_pMatShadowMap						= NULL;
		m_pRenderData						= NULL;
		
		m_pDirLightCBuffer					= NULL;

		m_pMatDebug							= NULL;
		m_pCurrentCamera					= NULL;
	}

	CLightDirectional::CLightDirectional(const CLightDirectional& other)
	{
		
	}

	CLightDirectional::~CLightDirectional()
	{
		
	}
	
	// Method for initializing all directional for the game.
	int CLightDirectional::Initialize()
	{
		int retCode = Util::Err::NONE;

		u32 i;

		// Directional lights.
		if(m_dirLightCount > 0)
		{
			m_pDirLightArray		= new DIRECTIONAL_LIGHT[m_dirLightCount];
			m_pDirLightPOVs			= new Math::CCamera*[m_dirLightCount];

			ZeroMemory(m_pDirLightArray, sizeof(DIRECTIONAL_LIGHT) * m_dirLightCount);
			ZeroMemory(m_pDirLightPOVs, sizeof(Math::CCamera*) * m_dirLightCount);
			
			i = 0;
			DIRECTIONAL_LIGHT_QUEUE* data;
			while((data = (DIRECTIONAL_LIGHT_QUEUE*)m_qDirLightQueue.Pop()))
			{
				m_pDirLightArray[i].viewProj				= data->viewProj;
				m_pDirLightArray[i].color					= data->color;
				m_pDirLightArray[i].direction				= data->direction;
				m_pDirLightArray[i].strength				= data->strength;
				m_pDirLightArray[i].shadowsOn				= data->shadowsOn;
				m_pDirLightArray[i].shadowStrength			= data->shadowStrength;
				if(data->lightPtr)
					*data->lightPtr							= &m_pDirLightArray[i];

				m_pDirLightPOVs[i] = new Math::CCamera();
				m_pDirLightPOVs[i]->CameraMode(Math::CAMERA_MODE::Orthographic);
				m_pDirLightPOVs[i]->FarView(50.0f);
				m_pDirLightPOVs[i]->NearView(0.3f);
				m_pDirLightPOVs[i]->Width(50.0f);
				m_pDirLightPOVs[i]->Height(50.0f);
				m_pDirLightPOVs[i]->FieldOfView(65.0f * Math::_PiOver180);
				m_pDirLightPOVs[i]->Transform(Math::CTransformSharedPtr(new Math::CTransform()));
				m_pDirLightPOVs[i]->Initialize(1.0f);
				m_pDirLightPOVs[i]->Update();

				SAFE_DELETE(data);
				i++;
			}

			m_dirShadowMapCount = 0;

			for(i = 0; i < m_dirLightCount; i++)
			{
				if(m_pDirLightArray[i].shadowsOn)
				{
					m_dirShadowMapCount++;
				}
			}

			if(m_dirShadowMapCount > 0)
			{
				m_pDirShadowMaps = new CTextureRenderTo();
				m_pDirShadowMaps->DirectXData(m_pDX11Data);

				if((retCode = m_pDirShadowMaps->Initialize(m_shadowMapSize, m_shadowMapSize, m_dirShadowMapCount, true)) != Util::Err::NONE)
					return retCode;
			}
		}

		// Store the light counts in strings so that they can be used as shader macros.
		m_dirLightCountStr = std::to_string(m_dirLightCount);
		
		if(m_dirLightCount > 0)
		{
			m_pDirLightCBuffer		= new CConstantBuffer();
			m_pDirLightCBuffer->DirectXData(m_pDX11Data);
			if((retCode = m_pDirLightCBuffer->Initialize(sizeof(DIRECTIONAL_LIGHT) * m_dirLightCount, true)) != Util::Err::NONE)
				return retCode;
		}

		return retCode;
	}

	int CLightDirectional::ResetBuffer()
	{
		int retCode = Util::Err::NONE;
		
		if(m_pDirLightCBuffer) if((retCode = m_pDirLightCBuffer->ResetBuffer(m_pDirLightArray)) != Util::Err::NONE) return retCode;

		return retCode;
	}

	int CLightDirectional::BindBuffer()
	{
		int retCode = Util::Err::NONE;
		
		if(m_pDirLightCBuffer) if((retCode = m_pDirLightCBuffer->BindBufferPS(11)) != Util::Err::NONE) return retCode;

		return retCode;
	}
	
	// Method for rendering directional light shadow maps.
	int CLightDirectional::RenderMaps(void* data, int (*RenderWorldObjects)(void*, CMaterial*))
	{
		int retCode = Util::Err::NONE;
		
		u32 i;
		u32 index;

		// Directional lights.
		if(m_dirLightCount > 0)
		{
			index = 0;

			for(i = 0; i < m_dirLightCount; i++)
			{
				if(m_pDirLightArray[i].shadowsOn)
				{
					Math::CVector3 directionLightPos = m_pCurrentCamera->Transform()->Position() + *(Math::CVector3*)&m_pDirLightArray[i].direction * -20;

					float offsetScale = 5.0f;
					directionLightPos.x = roundf(directionLightPos.x / offsetScale) * offsetScale;
					directionLightPos.y = roundf(directionLightPos.y / offsetScale) * offsetScale;
					directionLightPos.z = roundf(directionLightPos.z / offsetScale) * offsetScale;

					m_pDirLightPOVs[i]->Transform()->Position(directionLightPos);
					m_pDirLightPOVs[i]->Transform()->LookAt(*(Math::CVector3*)&m_pDirLightArray[i].direction);
					m_pDirLightPOVs[i]->Update();
					m_pDirLightArray[i].viewProj = m_pDirLightPOVs[i]->ProjectionMatrix() * m_pDirLightPOVs[i]->ViewMatrix();
					
					m_pDirShadowMaps->SetAsRenderTarget(true, index);
					m_pDirShadowMaps->ClearRenderTarget(DX11_CLEAR_COLOR, Math::CColor(0), true, index);

					m_pMatShadowMap->m_shadowCBuffer.shadowPerspective = m_pDirLightArray[i].viewProj;
					m_pMatShadowMap->Camera(m_pDirLightPOVs[i]);

					if((retCode = RenderWorldObjects(data, m_pMatShadowMap)) != Util::Err::NONE)
						return retCode;

					index++;
				}
			}
			
			if(m_pDirShadowMaps)
			{
				m_pDirShadowMaps->ResetRenderTarget(false);

				m_pDevCon->PSSetShaderResources(30, 1, m_pDirShadowMaps->ppShaderResourceView());
			}
		}

		return retCode;
	}
	
	int CLightDirectional::Render()
	{
		int retCode = Util::Err::NONE;

		return retCode;
	}	
	
	// Method for release all directional light data.
	void CLightDirectional::Release()
	{
		SAFE_RELEASE_DELETE(m_pDirLightCBuffer);
		
		// Release directional lights.
		SAFE_RELEASE_DELETE(m_pDirShadowMaps);
		
		if(m_pDirLightPOVs)
			for(u32 i = 0; i < m_dirLightCount; i++)
				SAFE_DELETE(m_pDirLightPOVs[i]);
				
		SAFE_DELETE_ARRAY(m_pDirLightPOVs);
		SAFE_DELETE_ARRAY(m_pDirLightArray);

		m_qDirLightQueue.Release();
	}
};