//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CMatDebug.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CMatDebug.h"

namespace GFX
{
	CMatDebug::CMatDebug()
	{
		m_elementCount						= 1;
		m_bufferCount						= 2;
	}

	CMatDebug::CMatDebug(const CMatDebug& other)
	{
	}

	CMatDebug::~CMatDebug()
	{
	}
	
	// Method for setting up the debug material shader.
	int CMatDebug::SetupShader()
	{
		int retCode = Util::Err::NONE;

		m_pIed								= new D3D11_INPUT_ELEMENT_DESC[m_elementCount];

		m_pIed[0].SemanticName				= "POSITION";
		m_pIed[0].SemanticIndex				= 0;
		m_pIed[0].Format					= DXGI_FORMAT_R32G32B32_FLOAT;
		m_pIed[0].InputSlot					= 0;
		m_pIed[0].AlignedByteOffset			= 0;
		m_pIed[0].InputSlotClass			= D3D11_INPUT_PER_VERTEX_DATA;
		m_pIed[0].InstanceDataStepRate		= 0;

		m_pVertexResName	= MAKEINTRESOURCE(HLSL_VDEBUG);
		m_pPixelResName		= MAKEINTRESOURCE(HLSL_PDEBUG);
		m_pLogFilename		= ".\\DebugShader.txt";

		if((retCode = m_pShader->InitializeConstantBuffer(1, sizeof(DEBUG_CBUFFER), true)) != Util::Err::NONE)
			return retCode;

		ZeroMemory(&m_debugCBuffer, sizeof(m_debugCBuffer));

		m_debugCBuffer.color				= Math::CColor(1.0f, 1.0f, 1.0f);
		m_debugCBuffer.scale				= Math::CVector3(1.0f, 1.0f, 1.0f);
		m_debugCBuffer.padding0				= 0.0f;

		return retCode;
	}

	// Method for setting the debug material for render.
	int CMatDebug::Render()
	{
		int retCode = Util::Err::NONE;

		if((retCode = m_pShader->ResetConstantBuffer(1, &m_debugCBuffer)) != Util::Err::NONE)
			return retCode;

		if((retCode = m_pShader->RenderShader(m_shaderType)) != Util::Err::NONE)
			return retCode;

		// Bind constant buffers to shader.
		if((retCode = m_pShader->BindCBuffer(0, m_shaderType)) != Util::Err::NONE) return retCode;
		if((retCode = m_pShader->BindCBuffer(1, m_shaderType)) != Util::Err::NONE) return retCode;

		if((retCode = CMaterial::Render()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	void CMatDebug::Release()
	{
		CMaterial::Release();
	}
};