//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CMatFT2Font.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CMatFT2Font.h"

namespace GFX
{
	CMatFT2Font::CMatFT2Font()
	{
		m_elementCount						= 6;
		m_bufferCount						= 2;
		m_textureCount						= 1;
	}

	CMatFT2Font::CMatFT2Font(const CMatFT2Font& other)
	{
	}

	CMatFT2Font::~CMatFT2Font()
	{
		
	}
	
	// Method for setting up the freetype2 font renderer material shader.
	int CMatFT2Font::SetupShader()
	{
		int retCode = Util::Err::NONE;

		m_pIed								= new D3D11_INPUT_ELEMENT_DESC[m_elementCount];

		m_pIed[0]	= { "POSITION",			0,		DXGI_FORMAT_R32G32B32_FLOAT,		0,		0,		D3D11_INPUT_PER_VERTEX_DATA,		0 };
		m_pIed[1]	= { "TEXCOORD",			0,		DXGI_FORMAT_R32G32_FLOAT,			0,		12,		D3D11_INPUT_PER_VERTEX_DATA,		0 };
		m_pIed[2]	= { "INSTPOS",			0,		DXGI_FORMAT_R32G32B32_FLOAT,		1,		0,		D3D11_INPUT_PER_INSTANCE_DATA,		1 };
		m_pIed[3]	= { "INSTSCALE",		0,		DXGI_FORMAT_R32G32B32_FLOAT,		1,		12,		D3D11_INPUT_PER_INSTANCE_DATA,		1 };
		m_pIed[4]	= { "INSTUV",			0,		DXGI_FORMAT_R32G32B32A32_FLOAT,		1,		24,		D3D11_INPUT_PER_INSTANCE_DATA,		1 };
		m_pIed[5]	= { "INSTCOLOR",		0,		DXGI_FORMAT_R32G32B32A32_FLOAT,		1,		40,		D3D11_INPUT_PER_INSTANCE_DATA,		1 };

		m_pVertexResName		= MAKEINTRESOURCE(HLSL_VFT2FONT);
		m_pGeometryResName		= MAKEINTRESOURCE(HLSL_GFT2FONT);
		m_pPixelResName			= MAKEINTRESOURCE(HLSL_PFT2FONT);
		m_pLogFilename			= ".\\FT2FontShader.txt";

		if((retCode = m_pShader->InitializeConstantBuffer(1, sizeof(FT2FONT_CBUFFER), true)) != Util::Err::NONE)
			return retCode;

		ZeroMemory(&m_ft2CBuffer, sizeof(m_ft2CBuffer));

		m_ft2CBuffer.color			= Math::CColor(1.0f, 1.0f, 1.0f);
		m_ft2CBuffer.textured		= true;
		m_ft2CBuffer.pointFilter	= false;

		return retCode;
	}
	
	// Method for setting the freetype2 font renderer material for render.
	int CMatFT2Font::Render()
	{
		int retCode = Util::Err::NONE;
		
		if((retCode = m_pShader->ResetConstantBuffer(1, &m_ft2CBuffer)) != Util::Err::NONE)
			return retCode;

		if((retCode = m_pShader->RenderShader(m_shaderType)) != Util::Err::NONE)
			return retCode;

		// Bind constant buffers to shader.
		if((retCode = m_pShader->BindCBuffer(0, m_shaderType)) != Util::Err::NONE) return retCode;
		if((retCode = m_pShader->BindCBuffer(1, m_shaderType)) != Util::Err::NONE) return retCode;

		if((retCode = CMaterial::Render()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	void CMatFT2Font::Release()
	{
		CMaterial::Release();
	}
};