//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CMatParticle.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CMatParticle.h"

namespace GFX
{
	CMatParticle::CMatParticle()
	{
		m_elementCount						= 6;
		m_bufferCount						= 2;
		m_textureCount						= 1;
		m_bUseInverse						= true;
	}

	CMatParticle::CMatParticle(const CMatParticle& other)
	{
	}

	CMatParticle::~CMatParticle()
	{
	}
	
	// Method for setting up the particle material shader.
	int CMatParticle::SetupShader()
	{
		int retCode = Util::Err::NONE;

		m_pIed								= new D3D11_INPUT_ELEMENT_DESC[m_elementCount];

		m_pIed[0]	= { "POSITION",			0,		DXGI_FORMAT_R32G32B32_FLOAT,		0,		0,		D3D11_INPUT_PER_VERTEX_DATA,		0 };
		m_pIed[1]	= { "INSTPOS",			0,		DXGI_FORMAT_R32G32B32_FLOAT,		1,		0,		D3D11_INPUT_PER_INSTANCE_DATA,		1 };
		m_pIed[2]	= { "INSTSCALE",		0,		DXGI_FORMAT_R32G32B32_FLOAT,		1,		12,		D3D11_INPUT_PER_INSTANCE_DATA,		1 };
		m_pIed[3]	= { "INSTUV",			0,		DXGI_FORMAT_R32G32B32A32_FLOAT,		1,		24,		D3D11_INPUT_PER_INSTANCE_DATA,		1 };
		m_pIed[4]	= { "INSTCOLOR",		0,		DXGI_FORMAT_R32G32B32A32_FLOAT,		1,		40,		D3D11_INPUT_PER_INSTANCE_DATA,		1 };
		m_pIed[5]	= { "INSTDATA",			0,		DXGI_FORMAT_R32G32B32A32_FLOAT,		1,		56,		D3D11_INPUT_PER_INSTANCE_DATA,		1 };
		
		m_pVertexResName		= MAKEINTRESOURCE(HLSL_VPARTICLE);
		m_pGeometryResName		= MAKEINTRESOURCE(HLSL_GPARTICLE);
		m_pPixelResName			= MAKEINTRESOURCE(HLSL_PPARTICLE);
		m_pLogFilename			= ".\\ParticleShader.txt";

		if((retCode = m_pShader->InitializeConstantBuffer(1, sizeof(PARTICLE_CBUFFER), true)) != Util::Err::NONE)
			return retCode;

		ZeroMemory(&m_particleCBuffer, sizeof(m_particleCBuffer));

		m_particleCBuffer.color				= Math::CColor(1, 1, 1, 1);
		m_particleCBuffer.softness			= 0.5f;

		return retCode;
	}
	
	// Method for setting the particle material for render.
	int CMatParticle::Render()
	{
		int retCode = Util::Err::NONE;

		m_particleCBuffer.screenWidth		= m_pDX11Data->viewport.Width;
		m_particleCBuffer.screenHeight		= m_pDX11Data->viewport.Height;
		m_particleCBuffer.cameraPosition	= m_pCamera->Transform()->Position();
		m_mtxInvCBuffer.projectionInv		= m_pCamera->ProjectionMatrixInv();
		m_mtxInvCBuffer.viewInv				= m_pCamera->ViewMatrixInv();
		
		if((retCode = m_pShader->ResetConstantBuffer(1, &m_particleCBuffer)) != Util::Err::NONE)
			return retCode;
		
		if((retCode = m_pShader->RenderShader(m_shaderType)) != Util::Err::NONE)
			return retCode;

		// Bind constant buffer to shader.
		if((retCode = m_pShader->BindCBuffer(0, m_shaderType)) != Util::Err::NONE)
			return retCode;

		if((retCode = m_pShader->BindCBuffer(1, m_shaderType)) != Util::Err::NONE)
			return retCode;

		if((retCode = CMaterial::Render()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	void CMatParticle::Release()
	{
		CMaterial::Release();
	}
};