//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CMatPostScreen.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CMATPOSTSCREEN_H
#define CMATPOSTSCREEN_H

#include <sstream>
#include "CMaterial.h"
#include "..\Math\CMathColor.h"
#include "CLighting.h"
#include "CTextureProjection.h"

namespace GFX
{
	const u32 POST_TEXTURE_COUNT = 4;

	// Post processing material constant buffer.
	struct POSTSCREEN_CBUFFER
	{
		Math::CVector4						scale;
		Math::CVector4						cameraPoint;
	};

	// CMatPostScreen class
	// Material class for rendering the post processing screen quad.
	// Applies lighting and texture projection with deferred rendering.
	class CMatPostScreen : public CMaterial
	{
	public:
		friend class CMatPostLighting;

	public:
		CMatPostScreen();
		CMatPostScreen(const CMatPostScreen& other);
		virtual ~CMatPostScreen();

		int									Render() override;

		void								Release() override;

		inline Math::CCamera*				WorldCamera() const;
		inline void							WorldCamera(Math::CCamera* pCamera);
		inline CLighting*					Lighting() const;
		inline void							Lighting(CLighting* pLighting);
		inline CTextureProjection*			TextureProjection() const;
		inline void							TextureProjection(CTextureProjection* pProjection);
		
		POSTSCREEN_CBUFFER					m_postCBuffer;

	protected:
		int									SetupShader() override;

	private:
		std::string							m_fogTypeStr;

		Math::CCamera*						m_pWorldCamera;
		CLighting*							m_pLighting;
		CTextureProjection*					m_pTextureProjection;
	};

	// World camera getter and setter.

	inline Math::CCamera* CMatPostScreen::WorldCamera() const
	{
		return m_pWorldCamera;
	}

	inline void CMatPostScreen::WorldCamera(Math::CCamera* pCamera)
	{
		m_pWorldCamera = pCamera;
	}

	// Lighting getter and setter.

	inline CLighting* CMatPostScreen::Lighting() const 
	{
		return m_pLighting;
	}

	inline void CMatPostScreen::Lighting(CLighting* pLighting)
	{
		m_pLighting = pLighting;
	}

	// Texture projection getter and setter.

	inline CTextureProjection* CMatPostScreen::TextureProjection() const
	{
		return m_pTextureProjection;
	}

	inline void CMatPostScreen::TextureProjection(CTextureProjection* pProjection)
	{
		m_pTextureProjection = pProjection;
	}
};

#endif