//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CMatShadowMap.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CMatShadowMap.h"

namespace GFX
{
	CMatShadowMap::CMatShadowMap()
	{
		m_bufferCount						= 2;
		m_elementCount						= 9;

		ZeroMemory(&m_shadowCBuffer, sizeof(m_shadowCBuffer));
	}

	CMatShadowMap::CMatShadowMap(const CMatShadowMap& other)
	{
	}

	CMatShadowMap::~CMatShadowMap()
	{
	}
	
	// Method for setting up the shadow material shader.
	int CMatShadowMap::SetupShader()
	{
		int retCode = Util::Err::NONE;

		m_pIed									= new D3D11_INPUT_ELEMENT_DESC[m_elementCount];
		
		int index = 0;
		m_pIed[index++]	= { "POSITION",			0,		DXGI_FORMAT_R32G32B32_FLOAT,		0,		D3D11_APPEND_ALIGNED_ELEMENT,		D3D11_INPUT_PER_VERTEX_DATA,		0 };
		m_pIed[index++]	= { "NORMAL",			0,		DXGI_FORMAT_R32G32B32_FLOAT,		0,		D3D11_APPEND_ALIGNED_ELEMENT,		D3D11_INPUT_PER_VERTEX_DATA,		0 };
		m_pIed[index++]	= { "BINORMAL",			0,		DXGI_FORMAT_R32G32B32_FLOAT,		0,		D3D11_APPEND_ALIGNED_ELEMENT,		D3D11_INPUT_PER_VERTEX_DATA,		0 };
		m_pIed[index++]	= { "TANGENT",			0,		DXGI_FORMAT_R32G32B32_FLOAT,		0,		D3D11_APPEND_ALIGNED_ELEMENT,		D3D11_INPUT_PER_VERTEX_DATA,		0 };
		m_pIed[index++]	= { "COLOR",			0,		DXGI_FORMAT_R32G32B32A32_FLOAT,		0,		D3D11_APPEND_ALIGNED_ELEMENT,		D3D11_INPUT_PER_VERTEX_DATA,		0 };
		
		for(u32 i = 0; i < 4; i++)
			m_pIed[index++]	= { "TEXCOORD",		i,		DXGI_FORMAT_R32G32_FLOAT,			0,		D3D11_APPEND_ALIGNED_ELEMENT,		D3D11_INPUT_PER_VERTEX_DATA,		0 };
		
		m_pVertexResName	= MAKEINTRESOURCE(HLSL_VSHADOWMAP);
		m_pPixelResName		= MAKEINTRESOURCE(HLSL_PSHADOWMAP);
		m_pLogFilename		= ".\\ShadowMapShader.txt";
		
		// Setup the constant buffers.
		if((retCode = m_pShader->InitializeConstantBuffer(1, sizeof(SHADOW_CBUFFER), true)) != Util::Err::NONE)
			return retCode;

		return retCode;
	}
	
	// Method for setting the shadow map material for render.
	int CMatShadowMap::Render()
	{
		int retCode = Util::Err::NONE;

		if((retCode = m_pShader->ResetConstantBuffer(1, &m_shadowCBuffer)) != Util::Err::NONE) return retCode;
		if((retCode = m_pShader->RenderShader(m_shaderType)) != Util::Err::NONE)
			return retCode;

		// Bind constant buffers to shader.
		if((retCode = m_pShader->BindCBuffer(0, m_shaderType)) != Util::Err::NONE) return retCode;
		if((retCode = m_pShader->BindCBuffer(1, m_shaderType)) != Util::Err::NONE) return retCode;
		
		if((retCode = CMaterial::Render()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	void CMatShadowMap::Release()
	{
		CMaterial::Release();
	}
};