//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CMesh.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CMESH_H
#define CMESH_H

#include "CGeoObject.h"
#include "CMatMesh.h"

namespace GFX
{
	//---------------------------------------------------------------------------------------------
	// Model mesh.
	//---------------------------------------------------------------------------------------------

	// CMesh class.
	// Class for storing data about an imported mesh from an .fbx file.
	class CMesh : public CGeoObject
	{
	public:
		friend class CMeshScene;
		friend class CFBXLoader;

		// Vertex structure for imported mesh vertex data.
		struct MESH_VERTEX
		{
			Math::CVector3					position;
			Math::CVector3					normal;
			Math::CVector3					binormal;
			Math::CVector3					tangent;
			Math::CColor					color;
			Math::CVector2					uv[4];

			MESH_VERTEX(const Math::CVector3& Position, const Math::CVector3& Normal, const Math::CVector3& Binormal, 
				const Math::CVector3& Tangent, const Math::CColor& Color, const Math::CVector2* UV)
			{
				position		= Position;
				normal			= Normal;
				binormal		= Binormal;
				tangent			= Tangent;
				color			= Color;

				memcpy(uv, UV, sizeof(Math::CVector2) * 4);
			}
		};

	public:
		CMesh();
		CMesh(const CMesh& other);
		virtual ~CMesh();

		int									Initialize() override;
		int									Update() override;
		int									Render() override;
		int									Render(CMaterial* material);
		void								Release() override;

		inline CMatMesh*					MeshMaterial();

	private:
		std::string							m_name;

		Util::CStack<CMatMesh*>				m_sMaterialStack;
	};

	// First stack material getter.

	inline CMatMesh* CMesh::MeshMaterial()
	{
		return m_sMaterialStack[0];
	}
};

#endif
