//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CMeshScene.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CMESHSCENE_H
#define CMESHSCENE_H

#include "..\Globals\CGlobals.h"
#include "..\Utilities\CDebugError.h"
#include "..\Utilities\CDebugLog.h"
#include "..\Utilities\CDataStack.h"
#include "..\Utilities\CSortInsertion.h"
#include "..\Utilities\CSortQuick.h"
#include "CMesh.h"
#include "CRenderingData.h"

namespace GFX
{
	// CMeshScene class.
	// Class for storing all meshes, and materials from a .fbx file.
	class CMeshScene : public App::CDX11Object
	{
	public:
		friend class CFBXLoader;

	public:
		CMeshScene();
		CMeshScene(const CMeshScene& other);
		virtual ~CMeshScene();

		int									Initialize();
		int									Update();
		int									Render();
		int									Render(CMaterial* material);
		int									RenderAlpha();
		int									RenderAlpha(CMaterial* material);
		void								Release();

		CMesh*								FindMesh(const std::string& name);
		
		inline RENDERDATA*					RenderData() const;
		inline void							RenderData(RENDERDATA* pRenderData);
		inline Math::CCamera*				Camera() const;
		inline void							Camera(Math::CCamera* pCamera);
		inline CMaterial*					DebugMaterial() const;
		inline void							DebugMaterial(CMaterial* pMaterial);

		inline void							PushMesh(CMesh*& mesh);

	private:
		Util::CStack<CMesh*>				m_sMeshList;
		Util::CStack<CMesh*>				m_sMeshAlphaList;
		Util::CStack<CMaterial*>			m_sMaterialList;

		CMaterial*							m_pDebugMaterial;

		RENDERDATA*							m_pRenderData;
		Math::CCamera*						m_pCamera;
	};
	
	// Render data getter and setter.

	inline RENDERDATA* CMeshScene::RenderData() const
	{
		return m_pRenderData;
	}

	inline void CMeshScene::RenderData(RENDERDATA* pRenderData)
	{
		m_pRenderData = pRenderData;
	}

	// Camera getter and setter.

	inline Math::CCamera* CMeshScene::Camera() const
	{
		return m_pCamera;
	}

	inline void CMeshScene::Camera(Math::CCamera* pCamera)
	{
		m_pCamera = pCamera;
	}

	// Debug material getter and setter.

	inline CMaterial* CMeshScene::DebugMaterial() const
	{
		return m_pDebugMaterial;
	}

	inline void CMeshScene::DebugMaterial(CMaterial* pMaterial)
	{
		m_pDebugMaterial = pMaterial;
	}

	// Method for pushing a mesh to this mesh scene.
	inline void CMeshScene::PushMesh(CMesh*& mesh)
	{
		if(mesh->m_sMaterialStack.Length() > 0)
		{
			if(((CMatMesh*)mesh->m_sMaterialStack[0])->m_colorBuffer.transparency < 0.99f)
				m_sMeshAlphaList.Push(mesh);
			else
				m_sMeshList.Push(mesh);
		}
	}
};

#endif