//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CSkydome.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CSkydome.h"

namespace GFX
{
	CSkydome::CSkydome()
	{
		m_bSeperateVerticesIndices			= false;
		m_bCreateBoundingAABB				= false;
		m_bEnableInstancing					= false;

		m_latLineCount						= 12;
		m_longLineCount						= 12;

		m_pDepthStencilState				= NULL;
		m_pDepthStencilStatePrev			= NULL;
		m_pWorldCamera						= NULL;
		m_pSkyMapTexture					= NULL;
		m_pMatSkydome						= NULL;
	}

	CSkydome::CSkydome(const CSkydome& other)
	{
	}

	CSkydome::~CSkydome()
	{
	}

	// Method for initializing a skydome. Create a procedurally generate sphere.
	int CSkydome::Initialize()
	{
		int retCode = Util::Err::NONE;

		m_vertexStride = sizeof(SKYDOME_VERTEX);
		m_indexStride = sizeof(u32);
		
		float pitch = 0.0f;
		float yaw = 0.0f;
		Math::CVector3 pos0, pos1, pos2, pos3;
		Math::CMatrix4x4 rotationX, rotationY;

		u32 i, j;
		
		// Create the bottom of the sphere.
		for(i = 0; i < m_latLineCount; i++)
		{
			yaw = (float)(i) * (Math::_2Pi / (m_longLineCount));
			rotationY = Math::CMatrix4x4::RotateY(yaw);

			pitch = ((float)m_latLineCount - 1) * (Math::_Pi / (m_latLineCount)) - Math::_PiOver2;
			rotationX = Math::CMatrix4x4::RotateX(pitch);

			pos0 = Math::CMatrix4x4::TransformCoord(Math::CVector3(0.0f, 0.0f, 1.0f), rotationY * rotationX).Normalized();
			
			yaw = (float)(i + 1) * (Math::_2Pi / (m_longLineCount));
			rotationY = Math::CMatrix4x4::RotateY(yaw);

			pos1 = Math::CMatrix4x4::TransformCoord(Math::CVector3(0.0f, 0.0f, 1.0f), rotationY * rotationX).Normalized();

			AddTriangle(new SKYDOME_VERTEX(Math::CVector3(0.0f, -1.0f, 0.0f)), new SKYDOME_VERTEX(pos1), new SKYDOME_VERTEX(pos0));
		}
		
		// Create the top of the sphere.
		for(i = 0; i < m_latLineCount; i++)
		{
			yaw = (float)(i) * (Math::_2Pi / (m_longLineCount));
			rotationY = Math::CMatrix4x4::RotateY(yaw);
			
			pitch = 1.0f * (Math::_Pi / (m_latLineCount)) - Math::_PiOver2;
			rotationX = Math::CMatrix4x4::RotateX(pitch);

			pos0 = Math::CMatrix4x4::TransformCoord(Math::CVector3(0.0f, 0.0f, 1.0f), rotationY * rotationX).Normalized();
			
			yaw = (float)(i + 1) * (Math::_2Pi / (m_longLineCount));
			rotationY = Math::CMatrix4x4::RotateY(yaw);

			pos1 = Math::CMatrix4x4::TransformCoord(Math::CVector3(0.0f, 0.0f, 1.0f), rotationY * rotationX).Normalized();

			AddTriangle(new SKYDOME_VERTEX(Math::CVector3(0.0f, 1.0f, 0.0f)), new SKYDOME_VERTEX(pos0), new SKYDOME_VERTEX(pos1));
		}
		
		// Create the bottom of the sides of the sphere.
		for(i = 0; i < m_latLineCount; i++)
		{
			for(j = 1; j < m_longLineCount - 1; j++)
			{
				yaw = (float)(i) * (Math::_2Pi / (m_longLineCount));
				rotationY = Math::CMatrix4x4::RotateY(yaw);

				pitch = ((float)j) * (Math::_Pi / (m_latLineCount)) - Math::_PiOver2;
				rotationX = Math::CMatrix4x4::RotateX(pitch);
				
				pos0 = Math::CMatrix4x4::TransformCoord(Math::CVector3(0.0f, 0.0f, 1.0f), rotationY * rotationX).Normalized();
				
				yaw = (float)(i + 1) * (Math::_2Pi / (m_longLineCount));
				rotationY = Math::CMatrix4x4::RotateY(yaw);

				pos1 = Math::CMatrix4x4::TransformCoord(Math::CVector3(0.0f, 0.0f, 1.0f), rotationY * rotationX).Normalized();

				yaw = (float)(i) * (Math::_2Pi / (m_longLineCount));
				rotationY = Math::CMatrix4x4::RotateY(yaw);

				pitch = ((float)j + 1) * (Math::_Pi / (m_latLineCount)) - Math::_PiOver2;
				rotationX = Math::CMatrix4x4::RotateX(pitch);
				
				pos2 = Math::CMatrix4x4::TransformCoord(Math::CVector3(0.0f, 0.0f, 1.0f), rotationY * rotationX).Normalized();
				
				yaw = (float)(i + 1) * (Math::_2Pi / (m_latLineCount));
				rotationY = Math::CMatrix4x4::RotateY(yaw);

				pos3 = Math::CMatrix4x4::TransformCoord(Math::CVector3(0.0f, 0.0f, 1.0f), rotationY * rotationX).Normalized();
				
				AddQuad(new SKYDOME_VERTEX(pos1), new SKYDOME_VERTEX(pos0), new SKYDOME_VERTEX(pos2), new SKYDOME_VERTEX(pos3));
			}
		}

		if((retCode = CGeoObject::Initialize()) != Util::Err::NONE)
			return retCode;

		m_pTransform = Math::CTransformSharedPtr(new Math::CTransform());
		m_pTransform->Update();

		// Setup skydome material.
		m_pMatSkydome				= new CMatSkydome();
		
		m_pMatSkydome->Camera(m_pWorldCamera);
		m_pMatSkydome->DirectXData(m_pDX11Data);
		
		if((retCode = m_pMatSkydome->Initialize()) != Util::Err::NONE)
			return retCode;

		m_pMaterial = m_pMatSkydome;

		// Skydome depth stencil state.
		D3D11_DEPTH_STENCIL_DESC dssDesc;
		ZeroMemory(&dssDesc, sizeof(dssDesc));
		dssDesc.DepthEnable = false;

		if(FAILED(m_pDev->CreateDepthStencilState(&dssDesc, &m_pDepthStencilState)))
			return Util::Err::ERR_DX11_CREATE_DEPTH_STENCIL_STATE;
		
		return retCode;
	}

	int CSkydome::Update()
	{
		int retCode = Util::Err::NONE;
		
		if((retCode = CGeoObject::Update()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	// Method for rendering the skydome.
	int CSkydome::Render()
	{
		int retCode = Util::Err::NONE;

		m_pMatSkydome->Texture(m_pSkyMapTexture);
		m_pTransform->Position(m_pWorldCamera->Transform()->Position());
		m_pTransform->Update();
		
		m_pDevCon->OMGetDepthStencilState(&m_pDepthStencilStatePrev, 0);
		m_pDevCon->OMSetDepthStencilState(m_pDepthStencilState, 0);

		if((retCode = CGeoObject::Render(true)) != Util::Err::NONE)
			return retCode;

		m_pDevCon->OMSetDepthStencilState(m_pDepthStencilStatePrev, 0);

		return retCode;
	}

	void CSkydome::Release()
	{
		SAFE_RELEASE(m_pDepthStencilState);
		SAFE_RELEASE_DELETE(m_pMatSkydome);
		CGeoObject::Release();
	}
};