//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CSkydome.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CSKYDOME_H
#define CSKYDOME_H

#include "CGeoObject.h"
#include "CMatSkydome.h"

namespace GFX
{
	// CSkydome class.
	// Class for creating a skydome with a .dds cubemap.
	class CSkydome : public CGeoObject
	{
	public:
		// Skydome vertex structure.
		struct SKYDOME_VERTEX
		{
			Math::CVector3					position;

			SKYDOME_VERTEX()
			{
			}

			SKYDOME_VERTEX(const Math::CVector3& Position)
			{
				position = Position;
			}
		};

	public:
		CSkydome();
		CSkydome(const CSkydome& other);
		virtual ~CSkydome();

		int									Initialize() override;
		int									Update() override;
		int									Render() override;
		void								Release() override;

		inline Math::CColor					SkydomeColor() const;
		inline void							SkydomeColor(const Math::CColor& color);

		inline Math::CCamera*				WorldCamera() const;
		inline void							WorldCamera(Math::CCamera* pWorldCamera);
		inline CTexture*					SkyMapTexture() const;
		inline void							SkyMapTexture(CTexture* pSkyMapTexture);

	private:
		u32									m_latLineCount;
		u32									m_longLineCount;

		ID3D11DepthStencilState*			m_pDepthStencilStatePrev;
		ID3D11DepthStencilState*			m_pDepthStencilState;
		Math::CCamera*						m_pWorldCamera;
		CTexture*							m_pSkyMapTexture;
		CMatSkydome*						m_pMatSkydome;
	};

	// Color getter and setter.
	
	inline Math::CColor CSkydome::SkydomeColor() const
	{
		if(m_pMatSkydome)
			return m_pMatSkydome->m_skydomeBuffer.color;
		else
			return Math::CColor(0);
	}

	inline void CSkydome::SkydomeColor(const Math::CColor& color)
	{
		if(m_pMatSkydome)
			m_pMatSkydome->m_skydomeBuffer.color = color;
	}

	// World camera getter and setter.

	inline Math::CCamera* CSkydome::WorldCamera() const
	{
		return m_pWorldCamera;
	}

	inline void CSkydome::WorldCamera(Math::CCamera* pWorldCamera)
	{
		m_pWorldCamera = pWorldCamera;
	}

	// Texture getter and setter.

	inline CTexture* CSkydome::SkyMapTexture() const
	{
		return m_pSkyMapTexture;
	}

	inline void CSkydome::SkyMapTexture(CTexture* pSkyMapTexture)
	{
		m_pSkyMapTexture = pSkyMapTexture;
	}
};

#endif