//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CTexture.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CTEXTURE_H
#define CTEXTURE_H

#include <d3d11.h>
#include <d3dx11.h>
#include <codecvt>
#include <WICTextureLoader.h>
#include <DDSTextureLoader.h>
#include "..\Application\CDX11Object.h"
#include "..\Utilities\CDebugError.h"
#include "..\Utilities\CMemoryFree.h"

#pragma comment(lib, "DirectXTK.lib")

namespace GFX
{
	//---------------------------------------------------------------------------------------------
	// Texture2D
	//---------------------------------------------------------------------------------------------

	// CTexture class.
	// Class for loading, or creating a single texture.
	class CTexture : public App::CDX11Object
	{
	public:
		CTexture();
		CTexture(const CTexture& other);
		virtual ~CTexture();

		int									Initialize(const std::string& textureFilename, bool cubemap = false);
		int									Initialize(ID3D11Texture2D* texture2d);
		int									Initialize(ID3D11Texture2D* texture2d, D3D11_SHADER_RESOURCE_VIEW_DESC* srvd);
		void								Release();

		inline ID3D11ShaderResourceView*	Texture() const;
		inline ID3D11ShaderResourceView**	ppTexture();
		inline ID3D11ShaderResourceView*	Texture(ID3D11ShaderResourceView* pTexture);

	private:
		ID3D11ShaderResourceView*			m_pTexture;
	};

	// Texture getters and setters.

	inline ID3D11ShaderResourceView* CTexture::Texture() const
	{
		return m_pTexture;
	}

	inline ID3D11ShaderResourceView** CTexture::ppTexture()
	{
		return &m_pTexture;
	}

	inline ID3D11ShaderResourceView* CTexture::Texture(ID3D11ShaderResourceView* pTexture)
	{
		m_pTexture = pTexture;
	}

	//---------------------------------------------------------------------------------------------
	// Texture2D Array
	//---------------------------------------------------------------------------------------------

	// CTextureArray class.
	// Class for creating Texture2DArray from a list of filenames, for from a list of shader resources
	class CTextureArray : public App::CDX11Object
	{
	public:
		CTextureArray();
		CTextureArray(const CTextureArray& other);
		virtual ~CTextureArray();
		
		int									Initialize(u32 width, u32 height, u32 arraySize, std::string* filenames);
		int									Initialize(u32 width, u32 height, u32 arraySize, ID3D11ShaderResourceView** shaderResources);
		void								Release();

		inline ID3D11ShaderResourceView*	TextureArray() const;
		inline ID3D11ShaderResourceView**	ppTextureArray();
		inline void							TextureArray(ID3D11ShaderResourceView* pTextureArray);

	private:
		u32									m_width;
		u32									m_height;
		u32									m_arraySize;

		ID3D11Texture2D*					m_pTexture2DArray;
		ID3D11ShaderResourceView*			m_pTextureArray;
	};

	// Texture getters and setters.

	inline ID3D11ShaderResourceView* CTextureArray::TextureArray() const
	{
		return m_pTextureArray;
	}

	inline ID3D11ShaderResourceView** CTextureArray::ppTextureArray()
	{
		return &m_pTextureArray;
	}

	inline void CTextureArray::TextureArray(ID3D11ShaderResourceView* pTextureArray)
	{
		m_pTextureArray = pTextureArray;
	}
};

#endif