//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Graphics\CTextureRenderTo.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CTEXTURERENDERTO_H
#define CTEXTURERENDERTO_H

#include "CTexture.h"
#include "..\Math\CMathColor.h"

namespace GFX
{
	// CTextureRenderTo class
	// Class for creating a new render to texture.
	// Uses with a Texture2D, or a Texture2DArray shader resource view.
	class CTextureRenderTo : public App::CDX11Object
	{
	public:
		CTextureRenderTo();
		CTextureRenderTo(const CTextureRenderTo& other);
		virtual ~CTextureRenderTo();

		int									Initialize(int textureWidth, int textureHeight, u32 textureCount = 1, bool noStencil = false);

		int									SetAsRenderTarget(bool setDepthBuffer);
		int									SetAsRenderTarget(bool setDepthBuffer, u32 index, u32 range = 1);
		int									ResetRenderTarget(bool setDepthBuffer);
		int									ClearRenderTarget(const DX11_CLEAR_MODE& clearMode, const Math::CColor& bgColor, bool setDepthBuffer);
		int									ClearRenderTarget(const DX11_CLEAR_MODE& clearMode, const Math::CColor& bgColor, bool setDepthBuffer, u32 index, u32 range = 1);
		int									ClearBackBuffer(const DX11_CLEAR_MODE& clearMode, const Math::CColor& bgColor, bool setDepthBuffer);

		int									PreResize();
		int									PostResize(int textureWidth, int textureHeight);

		void								Release();

		inline ID3D11ShaderResourceView*	ShaderResourceView() const;
		inline ID3D11ShaderResourceView**	ppShaderResourceView();
		inline void							ShaderResourceView(ID3D11ShaderResourceView* pResView);
		inline ID3D11ShaderResourceView*	DepthStencilShaderResourceView() const;
		inline ID3D11ShaderResourceView**	ppDepthStencilShaderResourceView();
		inline void							DepthStencilShaderResourceView(ID3D11ShaderResourceView* pResView);

	private:
		int														CreateRenderTargetView					();
		int														CreateDepthStencilView					(bool noStencil = false);

		int														m_textureWidth;
		int														m_textureHeight;

		u32														m_textureCount;
		D3D11_VIEWPORT											m_viewport;

		ID3D11Texture2D*										m_pRenderTargetTexture;
		ID3D11RenderTargetView**								m_pRenderTargetView;
		ID3D11ShaderResourceView*								m_pShaderResourceView;

		ID3D11Texture2D*										m_pDepthStencilBuffer;
		ID3D11DepthStencilState*								m_pDepthStencilState;
		ID3D11DepthStencilView*									m_pDepthStencilView;
		ID3D11ShaderResourceView*								m_pDepthStencilResView;
	};

	// Shader resource view setters and getters.

	inline ID3D11ShaderResourceView* CTextureRenderTo::ShaderResourceView() const
	{
		return m_pShaderResourceView;
	}

	inline ID3D11ShaderResourceView** CTextureRenderTo::ppShaderResourceView()
	{
		return &m_pShaderResourceView;
	}

	inline void CTextureRenderTo::ShaderResourceView(ID3D11ShaderResourceView* pResView)
	{
		m_pShaderResourceView = pResView;
	}
	
	// Shader depth resource view setters and getters.

	inline ID3D11ShaderResourceView* CTextureRenderTo::DepthStencilShaderResourceView() const
	{
		return m_pDepthStencilResView;
	}

	inline ID3D11ShaderResourceView** CTextureRenderTo::ppDepthStencilShaderResourceView()
	{
		return &m_pDepthStencilResView;
	}

	inline void CTextureRenderTo::DepthStencilShaderResourceView(ID3D11ShaderResourceView* pResView)
	{
		m_pShaderResourceView = pResView;
	}
};

#endif
