//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Math\CMathColor.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CMATHCOLOR_H
#define CMATHCOLOR_H

#include "..\Globals\CGlobals.h"
#include "CMathFloat.h"
#include "CMathVector4.h"

namespace Math
{
	// CColor class.
	// Class for creating a vector designed to act as a color in a rgba float format.
	class CColor
	{
	public:
		union
		{
			struct
			{
				float r, g, b, a;
			};
			struct
			{
				float x, y, z, w;
			};
		};

		CColor();
		CColor(float f);
		CColor(float R, float G, float B);
		CColor(float R, float G, float B, float A);
		CColor(const CColor& other);
		CColor(CColor&& other);

		operator const float*() const		{ return (float*)this; }
		operator CVector4() const			{ return CVector4(x, y, z, w); }

		CColor&								operator =	(float f);
		CColor&								operator =	(const CColor& c);

		CColor								operator +	(const CColor& c) const;
		CColor								operator +	(float f) const;
		CColor								operator -	(const CColor& c) const;
		CColor								operator -	(float f) const;
		CColor								operator *	(const CColor& c) const;
		CColor								operator *	(float f) const;
		CColor								operator /	(const CColor& c) const;
		CColor								operator /	(float f) const;
		
		CColor&								operator +=	(const CColor& c);
		CColor&								operator +=	(float f);
		CColor&								operator -=	(const CColor& c);
		CColor&								operator -=	(float f);
		CColor&								operator *=	(const CColor& c);
		CColor&								operator *=	(float f);
		CColor&								operator /=	(const CColor& c);
		CColor&								operator /=	(float f);
		
		bool								operator ==	(const CColor& c) const;
		bool								operator !=	(const CColor& c) const;

		friend CColor						operator +	(float f, const CColor& c);
		friend CColor						operator -	(float f, const CColor& c);
		friend CColor						operator *	(float f, const CColor& c);
		friend CColor						operator /	(float f, const CColor& c);

		inline CColor						Saturate();
		inline CColor						Saturated();
		inline float						Grayscale();
		
		friend CColor						MoveTowards(const CColor& from, const CColor& to, float t);
		friend CColor						MoveTowards(const CColor& from, const CColor& to, const CVector4& t);
		friend CColor						Lerp(const CColor& from, const CColor& to, float t);
		friend CColor						Lerp(const CColor& from, const CColor& to, const CVector4& t);

		static void							Saturate(CColor& c);
		static CColor						Saturated(const CColor& c);
		static float						Grayscale(const CColor& c);
	};

	// Saturated value getter and setter.

	inline CColor CColor::Saturate()
	{
		*this = CColor(Math::Saturate(r), Math::Saturate(g), Math::Saturate(b), Math::Saturate(a));
	}

	inline CColor CColor::Saturated()
	{
		return CColor(Math::Saturate(r), Math::Saturate(g), Math::Saturate(b), Math::Saturate(a));
	}

	// Grayscale value getter.
	inline float CColor::Grayscale()
	{
		return (0.299f * r + 0.587f * g + 0.114f * b);
	}
};

#endif