//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Math\CMathPlane.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CMATHPLANE_H
#define CMATHPLANE_H

#include "CMathVector3.h"
#include "CMathFloat.h"

namespace Math
{
	// CPlane class
	// Class for creating a plane in 3d space.
	class CPlane
	{
	public:
		float a;
		float b;
		float c;
		float d;

	public:

		CPlane();
		CPlane(float A, float B, float C, float D);
		CPlane(const CVector3& point, const CVector3& normal);
		CPlane(const CVector3& a0, const CVector3& b0, const CVector3& c0);
		CPlane(const CPlane& other);
		CPlane(CPlane&& other);
		
		operator const float*() const		{ return (float*)this; }
		
		bool								operator ==	(const CPlane& p) const;
		bool								operator !=	(const CPlane& p) const;
		
		void								FromPointNormal(const CVector3& point, const CVector3& normal);
		void								FromPoints(const CVector3& a0, const CVector3& b0, const CVector3& c0);
		void								Normalize();
		CPlane								Normalized();
		CVector3							Normal() const;

		bool								IntersectLine(const CVector3& v0, const CVector3& v1, CVector3& closestPoint);
		bool								IntersectPlane(CPlane p, CVector3& v0, CVector3& v1);

		// Static
		static float						Dot(const CPlane& p, const CVector4& v);
		static float						DotCoord(const CPlane& p, const CVector3& v);
		static float						DotNormal(const CPlane& p, const CVector3& v);
		
		static void							FromPointNormal(CPlane& p, const CVector3& point, const CVector3& normal);
		static void							FromPoints(CPlane& p, const CVector3& a0, const CVector3& b0, const CVector3& c0);
		static void							Normalize(CPlane& p);
		static CPlane						Normalized(const CPlane& p);

		// Inline.
		inline float						Dot(const CVector4& v);
		inline float						DotCoord(const CVector3& v);
		inline float						DotNormal(const CVector3& v);

	private:
	};

	// Dot product methods.

	inline float CPlane::Dot(const CVector4& v)
	{
		return a * v.x + b * v.y + c * v.z + d * v.w;
	}

	inline float CPlane::DotCoord(const CVector3& v)
	{
		return a * v.x + b * v.y + c * v.z + d;
	}

	inline float CPlane::DotNormal(const CVector3& v)
	{
		return a * v.x + b * v.y + c * v.z;
	}

	// Plane normal vector getter.
	inline CVector3 CPlane::Normal() const
	{
		return CVector3(a, b, c);
	}
};

#endif
