//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Physics\CPhysics.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CPHYSICS_H
#define CPHYSICS_H

#include "..\Math\CMathAABB.h"
#include "..\Math\CMathVector3.h"
#include "..\Math\CMathTransform.h"
#include "..\Math\CMathRay.h"
#include "..\Math\CMathPlane.h"
#include "..\Application\CInputControls.h"

namespace Physics
{
	bool	RayIntersectPlane(const Math::CRay& ray, const Math::CPlane& plane, float& t, Math::CVector3* intersectionPoint);
	bool	LineLineDistance(const Math::CVector3& p0, const Math::CVector3& p1, const Math::CVector3& p2, const Math::CVector3& p3, Math::CVector3& point0, Math::CVector3& point1);
	bool	LineLineIntersect(const Math::CVector3& p0, const Math::CVector3& p1, const Math::CVector3& p2, const Math::CVector3& p3, Math::CVector3& pa, Math::CVector3& pb, float& mua, float& mub);
	bool	LinePointIntersect(const Math::CVector3& point, const Math::CVector3& p0, const Math::CVector3& p1, Math::CVector3& closestPoint, float& distance);

	bool	PointInTriangle(const Math::CVector3& point, const Math::CVector3& v0, const Math::CVector3& v1, const Math::CVector3& v2, const Math::CVector3& triNormal);
	bool	PointInQuad(const Math::CVector3& point, const Math::CVector3& v0, const Math::CVector3& v1, const Math::CVector3& v2, const Math::CVector3& v3, const Math::CVector3& triNormal);
	bool	PointInNGon(const Math::CVector3& point, Math::CVector3*& vertices, u32 vertexCount, const Math::CVector3& triNormal);

	bool	CollistionSolver(float a, float b, float c, float& t);

	bool	SphereIntersectPlane(const Math::CVector3& center, float radius, const Math::CVector3& velocity, const Math::CVector3& planeNormal, const Math::CVector3& planePoint, float& tMax);
	bool	SphereIntersectTriangle(const Math::CVector3& center, float radius, const Math::CVector3& velocity, const Math::CVector3& v0, const Math::CVector3& v1, const Math::CVector3& v2, const Math::CVector3& triNormal, float& tMax, Math::CVector3& collisionNormal);
	bool	SphereIntersectQuad(const Math::CVector3& center, float radius, const Math::CVector3& velocity, const Math::CVector3& v0, const Math::CVector3& v1, const Math::CVector3& v2, const Math::CVector3& v3, const Math::CVector3& triNormal, float& tMax, Math::CVector3& collisionNormal);
	bool	SphereIntersectNGon(const Math::CVector3& center, float radius, const Math::CVector3& velocity, Math::CVector3*& vertices, u32 vertexCount, const Math::CVector3& triNormal, float& tMax, Math::CVector3& collisionNormal);
	bool	SphereIntersectLineSegment(const Math::CVector3& center, float radius, const Math::CVector3& velocity, const Math::CVector3& v0, const Math::CVector3& v1, float& tMax, Math::CVector3& collisionNormal);
	bool	SphereIntersectPoint(const Math::CVector3& center, float radius, const Math::CVector3& velocity, const Math::CVector3& point, float& tMax, Math::CVector3& collisionNormal);
	bool	SphereIntersectSphere(const Math::CVector3& center, float radius, const Math::CVector3& velocity, const Math::CVector3& point, float pointRadius, float& tMax, Math::CVector3& collisionNormal);
	
	bool	CapsuleIntersectTriangle(const Math::CVector3& point0, const Math::CVector3& point1, float radius, const Math::CVector3& velocity, const Math::CVector3& v0, const Math::CVector3& v1, const Math::CVector3& v2, const Math::CVector3& triNormal, float& tMax, Math::CVector3& collisionNormal);
	bool	CapsuleIntersectQuad(const Math::CVector3& point0, const Math::CVector3& point1, float radius, const Math::CVector3& velocity, const Math::CVector3& v0, const Math::CVector3& v1, const Math::CVector3& v2, const Math::CVector3& v3, const Math::CVector3& triNormal, float& tMax, Math::CVector3& collisionNormal);
	bool	CapsuleIntersectNGon(const Math::CVector3& point0, const Math::CVector3& point1, float radius, const Math::CVector3& velocity, Math::CVector3*& vertices, u32 vertexCount, const Math::CVector3& triNormal, float& tMax, Math::CVector3& collisionNormal);
	bool	CapsuleIntersectSphere(const Math::CVector3& point0, const Math::CVector3& point1, float radius, const Math::CVector3& velocity, const Math::CVector3& point, float pointRadius, float& tMax, Math::CVector3& collisionNormal);
};

#endif