//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Resources\Shaders\ShadowMap.hlsl
//
//-------------------------------------------------------------------------------------------------

// Matrix constant buffer.
cbuffer MatrixBuffer : register(b0)
{
	float4x4	World;
	float4x4	View;
	float4x4	Projection;
};

cbuffer ConstantBuffer : register(b1)
{
	float4x4	ShadowPerspective;
};

// Vertex input structure.
struct VIn
{
	float4 Position			: POSITION;
	float4 Normal			: NORMAL;
	float4 Binormal			: BINORMAL;
	float4 Tangent			: TANGENT;
	float4 Color			: COLOR;

	float2 TexCoord0		: TEXCOORD0;
	float2 TexCoord1		: TEXCOORD1;
	float2 TexCoord2		: TEXCOORD2;
	float2 TexCoord3		: TEXCOORD3;
};

// Vertex output structure.
struct VOut
{
	float4 Position			: SV_POSITION;
	float2 Depth			: TEXCOORD0;
};

// Pixel output structure.
struct POut
{
	float4 SM				: SV_TARGET;
};

// Vertex shader function.
VOut VShader(VIn input)
{
	VOut output;
	
	output = (VOut)0;

	input.Position.w = 1.0f;
	
	output.Position = mul(input.Position, World);
	output.Position = mul(output.Position, ShadowPerspective);
	output.Depth.xy = output.Position.zw;

	return output;
}

// Pixel shader function.
POut PShader(VOut input)
{
	POut output;
	
	float depth = saturate(input.Depth.x / input.Depth.y);
	output.SM = float4(depth, 1, 1, 1);

	return output;
}