//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Resources\Shaders\Skydome.hlsl
//
//-------------------------------------------------------------------------------------------------

#include "RenderUtil.hlsli"

// Matrix constant buffer.
cbuffer MatrixBuffer : register(b0)
{
	float4x4	World;
	float4x4	View;
	float4x4	Projection;
};

// Data constant buffer.
cbuffer ConstantBuffer
{
	float4		Color;
};

// Vertex input structure.
struct VIn
{
	float4	Position		: POSITION;
};

// Vertex output structure.
struct VOut
{
	float4	Position		: SV_POSITION;
	float3	TexCoord		: TEXCOORD;
};

// Pixel output structure.
struct POut
{
	float4		Color			: SV_TARGET0;
	float4		Diffuse			: SV_TARGET1;
};

// Vertex shader function.
VOut VShader(VIn input)
{
	VOut output = (VOut)0;

	input.Position.w = 1.0f;
	output.Position = mul(input.Position, World);
	output.Position = mul(output.Position, View);
	output.Position = mul(output.Position, Projection);

	output.TexCoord.xyz = input.Position.xyz;

	return output;
}

TextureCube SkyMapTexture;

// Pixel shader function.
POut PShader(VOut input) : SV_TARGET
{
	POut output = (POut)0;

	output.Diffuse = float4(1, 1, 1, 0.0f);
	output.Color = SkyMapTexture.Sample(samplerAnisoClamp, input.TexCoord);
	output.Color.rgb = saturate(output.Color.rgb * Color.rgb);

	return output;
}