//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Resources\Shaders\TextureProj.hlsli
//
//-------------------------------------------------------------------------------------------------

#ifndef TEXTUREPROJ_COUNT
#define TEXTUREPROJ_COUNT 0
#endif

// If texture projection count is greater than zero, create an array of texture projector data.
#if TEXTUREPROJ_COUNT

// Texture projector data structure.
struct TextureProjector
{
	float4x4								Projection;
	float4x4								View;
	float3									Color;
	int										TextureIndex;
	float									Near;
	float									Far;
	float2									Padding0;
};

cbuffer TextureProjBuffer : register(b10)
{
	TextureProjector	TEXTUREPROJ[TEXTUREPROJ_COUNT];
};

Texture2DArray	projectionTexture	: register(t20);
Texture2DArray	projectionGradiant	: register(t21);

// Function for projecting a texutre in the world using the deferred buffer.
float4 ProjectTexture(float3 color, float3 worldPosition, TextureProjector texProj)
{
	float4 projPos = mul(float4(worldPosition, 1.0f), texProj.View);
	projPos /= projPos.w;
	float4 viewPos = projPos;

	projPos = mul(projPos, texProj.Projection);
	projPos /= projPos.w;

	float2 projectTexCoord;
	projectTexCoord.x = (projPos.x) * 0.5f + 0.5f;
	projectTexCoord.y = (-projPos.y) * 0.5f + 0.5f;

	float alpha = 0.0f;

	// Check for projection.
	if(projPos.z > 0.0f && (saturate(projectTexCoord.x) == projectTexCoord.x) && saturate(projectTexCoord.y) == projectTexCoord.y)
	{
		float4 texC = projectionTexture.Sample(samplerLinear, float3(projectTexCoord.xy, texProj.TextureIndex & 0xFFFF));

		// Apply the gradient falloff
		float4 gradC = projectionGradiant.Sample(samplerLinearClamp, float3((viewPos.z) / abs(texProj.Far - texProj.Near), 0.5f, texProj.TextureIndex & 0xFFFF0000));
		alpha = texC.a * gradC.r;
		texC.a *= gradC.r;
		
		color.rgb = ((1.0f - texC.a) * color.rgb + (texC.a) * texC.rgb) * texProj.Color.rgb;
	}

	return float4(color.rgb, alpha);
}

#endif
