//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Actors\CScene.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CSCENE_H
#define CSCENE_H

#include "..\Application\CInput.h"
#include "..\Application\CDX11Object.h"
#include "..\Application\CDXStates.h"
#include "..\Application\CWinData.h"
#include "..\Audio\CXAudio2.h"
#include "..\Graphics\CRenderingData.h"
#include "..\Graphics\CLighting.h"
#include "..\Graphics\CFBXLoader.h"
#include "..\Graphics\CSkydome.h"
#include "..\Graphics\CPostScreenQuad.h"
#include "..\Utilities\CDebugError.h"
#include "..\Utilities\CTimer.h"

namespace Scene
{
	// CScene class.
	// Base class for creating a scene for the application.
	class CScene : public App::CDX11Object
	{
	public:
		CScene();
		CScene(const CScene& other);
		virtual ~CScene();

		virtual int							PreInitialize();
		virtual int							Initialize();
		virtual int							Update();
		virtual int							PreRender();
		virtual int							Render();
		virtual int							PostRender();

		virtual int							PreResize();
		virtual int							PostResize();

		virtual void						Release();
		
		inline App::CInput*					Input() const;
		inline void							Input(App::CInput* pInput);
		inline App::WINDATA*				WinData() const;
		inline void							WinData(App::WINDATA* pWinData);

		inline App::CDXStates*				DXStates() const;
		inline void							DXStates(App::CDXStates* pDXStates);
		inline Audio::CXAudio2Engine*		AudioEngine() const;
		inline void							AudioEngine(Audio::CXAudio2Engine* pAudioEngine);

		inline GFX::CLighting*				Lighting() const;
		inline void							Lighting(GFX::CLighting* pLighting);
		inline GFX::CFBXLoader*				FBXLoader() const;
		inline void							FBXLoader(GFX::CFBXLoader* pFBXLoader);
		inline GFX::CPostScreenQuad*		PostQuad() const;
		inline void							PostQuad(GFX::CPostScreenQuad* pPostQuad);

		inline Util::CTimer*				Timer() const;
		inline void							Timer(Util::CTimer* pTimer);
		inline GFX::RENDERDATA*				RenderData() const;
		inline void							RenderData(GFX::RENDERDATA* pRenderData);

	protected:
		App::CInput*						m_pInput;
		App::WINDATA*						m_pWinData;

		App::CDXStates*						m_pDXStates;
		Audio::CXAudio2Engine*				m_pAudioEngine;
		
		GFX::CLighting*						m_pLighting;
		GFX::CFBXLoader*					m_pFBXLoader;
		GFX::CSkydome*						m_pSkydome;
		GFX::CPostScreenQuad*				m_pPostQuad;

		Util::CTimer*						m_pTimer;
		GFX::RENDERDATA*					m_pRenderData;

	private:
	};
	
	// Input getter and setter.

	inline App::CInput* CScene::Input() const
	{
		return m_pInput;
	}

	inline void CScene::Input(App::CInput* pInput)
	{ 
		m_pInput = pInput;
	}

	// Window data getter and setter.

	inline App::WINDATA* CScene::WinData() const
	{ 
		return m_pWinData;
	}

	inline void CScene::WinData(App::WINDATA* pWinData) {
	m_pWinData = pWinData;
	}

	// DirectX 11 states object getter and setter.

	inline App::CDXStates* CScene::DXStates() const
	{
		return m_pDXStates;
	}

	inline void CScene::DXStates(App::CDXStates* pDXStates)
	{
		m_pDXStates = pDXStates;
	}

	// Audio engine getter and setter.

	inline Audio::CXAudio2Engine* CScene::AudioEngine() const
	{
		return m_pAudioEngine;
	}

	inline void CScene::AudioEngine(Audio::CXAudio2Engine* pAudioEngine)
	{
		m_pAudioEngine = pAudioEngine;
	}

	// Lighting getter and setter.

	inline GFX::CLighting* CScene::Lighting() const
	{
		return m_pLighting;
	}

	inline void CScene::Lighting(GFX::CLighting* pLighting)
	{
		m_pLighting = pLighting;
	}

	// FBX getter and setter.

	inline GFX::CFBXLoader* CScene::FBXLoader() const
	{
		return m_pFBXLoader;
	}

	inline void CScene::FBXLoader(GFX::CFBXLoader* pFBXLoader)
	{
		m_pFBXLoader = pFBXLoader;
	}

	// Post processing quad getter and setter.

	inline GFX::CPostScreenQuad* CScene::PostQuad() const
	{
	return m_pPostQuad;
	}

	inline void CScene::PostQuad(GFX::CPostScreenQuad* pPostQuad)
	{
	m_pPostQuad = pPostQuad;
	}

	// Timer getter and setter.

	inline Util::CTimer* CScene::Timer() const
	{
		return m_pTimer;
	}

	inline void CScene::Timer(Util::CTimer* pTimer)
	{
		m_pTimer = pTimer;
	}

	// Render data getter and setter.

	inline GFX::RENDERDATA* CScene::RenderData() const
	{
		return m_pRenderData;
	}

	inline void CScene::RenderData(GFX::RENDERDATA* pRenderData)
	{
		m_pRenderData = pRenderData;
	}
};

#endif