//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: UI\CUIButton.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CUIBUTTON_H
#define CUIBUTTON_H

#include "CUIObject.h"

namespace UI
{
	// CButton class
	// Class for creating a UI button.
	class CButton : public CUIObject
	{
	private:
		// Button text data structure.
		struct TextData
		{
			std::string			text;
			Math::CVector3		offset;
			Math::CColor		color;
			Math::CColor		selectedColor;
			float				size;
			UI_HALIGN			hAlign;
			UI_VALIGN			vAlign;
		};

	public:
		CButton();
		CButton(const CButton& other);
		virtual ~CButton();

		int									SetupButton(App::CInput*& input);
		
		int									SetupToggleButton(std::string* toggleOptions = NULL, int optionCount = 0, int currentIndex = 0);

		int									SetTextData(const std::string& text, const Math::CVector3& offset, float size, const Math::CColor& color, const Math::CColor& selectedColor, const UI_HALIGN& hAlign, const UI_VALIGN& vAlign);
		int									DisplayButton(const Math::CVector3& position, const Math::CVector2& scale, const Math::CColor& color, const Math::CColor& selectedColor, const UI_HALIGN& hAlign, const UI_VALIGN& vAlign);
		int									DisplayTextButton(const std::string& text, const Math::CVector3& position, float size, const Math::CColor& color, const Math::CColor& selectedColor, const UI_HALIGN& hAlign, const UI_VALIGN& vAlign);
		
		inline void							Background(CImage* background);
		inline void							FontRenderer(CFontRenderer* font);

	private:
		void								CheckInteraction() override;

		bool								m_bToggleButton;
		int									m_toggleIndex;
		int									m_toggleMax;
		std::string*						m_pToggleOptions;

		bool								m_bUseBackground;

		TextData							m_textData;
	};

	// Background image setter.
	inline void CButton::Background(CImage* background)
	{
		m_bUseBackground = background ? true : false;
		m_pImage = background;
	}

	// Font renderer setter.
	inline void CButton::FontRenderer(CFontRenderer* font)
	{
		m_pFont = font;
	}
};

#endif