//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: UI\CUIFreeType.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CUIFREETYPE_H
#define CUIFREETYPE_H

#include <d3d11.h>
#include <d3dx11.h>

#include <stdio.h>
#include <string>
#include <ft2build.h>
#include FT_FREETYPE_H

#include "..\Graphics\CGeoObject.h"
#include "..\Math\CMathVector2.h"
#include "..\Graphics\CTexture.h"

#ifdef _DEBUG
#pragma comment(lib, "legacy_stdio_definitions.lib")
#pragma comment(lib, "freetype26d.lib")
#else
#pragma comment(lib, "legacy_stdio_definitions.lib")
#pragma comment(lib, "freetype26.lib")
#endif

namespace UI
{
	const u32								FONT_CHAR_COUNT							= 95;

	// CFreetype class.
	// Class for creating a font image from a font file with the FreeType2 library.
	class CFreeType : public App::CDX11Object
	{
	public:
		friend class CFontRenderer;

	public:
		// Structure for storing data about each character taken from a font.
		struct CharacterData
		{
			float width;
			float height;
			float hang;
			Math::CVector4 fontDim;
			Math::CVector2 kerning;
			Math::CVector4 uvDim;
		};

	public:
		CFreeType();
		CFreeType(const CFreeType& other);
		virtual ~CFreeType();

		int									Initialize(bool genFont = true);
		void								Release();

		int									GenerateFontFile(const std::string& filename);
		int									GenerateSDFFontFile(const std::string& filename);
		int									OpenFontFile(const std::string& filename);

		inline void							FontFilename(const std::string& filename);
		inline std::string					FontFilename() const;

		inline GFX::CTexture*				FontTexture() const;
		inline void							FontTexture(GFX::CTexture* pTexture);

	private:
		int									CreateFontTexture();
		int									WriteGlyph(int index, u8*& bits, FT_GlyphSlot slot, u32 x, u32 y, FT_Bitmap bitmap, int left, int top, int pitch);
		int									CreateTexture2D();
		int									CreateTextureFrom2D();

		std::string							m_filename;

		u32									m_pt;
		float								m_ptScale;

		u32									m_dpi;
		u32									m_padding;

		u32									m_width;
		u32									m_height;

		u32									m_pixelSize;
		u8*									m_pPixels;
		CharacterData						m_characterData[FONT_CHAR_COUNT];

		FT_Library							m_library;
		FT_Face								m_face;
		FT_GlyphSlot						m_slot;

		ID3D11Texture2D*					m_pTexture2d;
		GFX::CTexture*						m_pTexture;
	};

	// Font filename getter and setter.

	inline void CFreeType::FontFilename(const std::string& filename)
	{
		m_filename = filename;
	}

	inline std::string CFreeType::FontFilename() const
	{
		return m_filename;
	}

	// Font texture getter and setter.

	inline GFX::CTexture* CFreeType::FontTexture() const
	{
		return m_pTexture;
	}

	inline void CFreeType::FontTexture(GFX::CTexture* pTexture)
	{
		m_pTexture = pTexture;
	}
};

#endif