//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: UI\CUITextField.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CUITEXTFIELD_H
#define CUITEXTFIELD_H

#include "..\Application\CInputTyping.h"
#include "CUIObject.h"

namespace UI
{
	// CTextField class
	// Class for create a UI text field.
	class CTextField : public CUIObject
	{
	private:
		// Text field text data structure.
		struct TextData
		{
			std::string						initialText;
			std::string						currentText;
			Math::CVector3					offset;
			float							size;
			Math::CColor					color;
		};

	public:
		CTextField();
		CTextField(const CTextField& other);
		virtual ~CTextField();

		int									SetupTextField(App::CInput*& input, Util::CTimer*& timer, const std::string& initialText);
		
		int									Update() override;
		int									SetTextData(float size, const Math::CVector3& offset, const Math::CColor& color);
		int									DisplayTextField(const Math::CVector3& position, const Math::CVector2& scale, const Math::CColor& color, const UI_HALIGN& hAlign, const UI_VALIGN& vAlign);
		
		void								Release() override;
		
		inline void							Background(CImage* background);
		inline void							FontRenderer(CFontRenderer* font);

	private:
		void								CheckInteraction() override;

		bool								m_bSelected;

		int									m_caretIndex;
		float								m_caretPosition;
		float								m_caretBlinkRate;
		float								m_caretBlinkTimer;

		TextData							m_textData;

		App::CInputTyping*					m_pInputTyping;
	};

	// Background image setter.
	inline void CTextField::Background(CImage* background)
	{
		m_pImage = background;
	}

	// Font renderer setter.
	inline void CTextField::FontRenderer(CFontRenderer* font)
	{
		m_pFont = font;
	}
};

#endif