//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Utilities\CDataList.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CDATALIST_H
#define CDATALIST_H

#include "..\Globals\CGlobals.h"
#include "CMemoryFree.h"

namespace Util
{
	// CList class (template)
	// Class for creating a template list data structure.
	template<typename T>
	class CList
	{
	private:
		// CList node structure.
		typedef struct Node
		{
			Node*		next;
			T			data;

		} NODE;

		NODE *m_pHead, *m_pTail;
		
	public:
		CList()
		{
			m_pHead = NULL;
			m_pTail = NULL;
		}

		CList(const CList& other)
		{
		}

		CList(CList&& other)
		{
		}

		virtual ~CList()
		{
			Release();
		}

		// Method for pushing data to the back of the list.
		void PushBack(T data)
		{
			NODE* newNode = new NODE();
			newNode->data = data;

			if(!m_pHead)
			{
				m_pHead = newNode;
				m_pTail = newNode;
			}

			m_pTail->next = newNode;
			newNode->next = NULL;
			m_pTail = newNode;
		}

		// Method for popping from the front of the list.
		bool PopFront(T& data)
		{
			if(!m_pHead)
				return false;

			NODE* temp = m_pHead;
			data = m_pHead->data;

			if(m_pHead->next)
			{
				m_pHead = m_pHead->next;
				delete temp;
			}
			else
			{
				delete temp;
				m_pHead = NULL;
				m_pTail = NULL;
			}

			return true;
		}

		// Method of inserting data at a point in the list.
		bool InsertAt(T data, u32 index)
		{
			if(m_pHead)
			{
				u32 counter = 0;

				NODE* newNode = new NODE();
				newNode->data = data;
				NODE* prev = m_pHead;
				NODE* temp = m_pHead;

				while(temp)
				{
					// Find the node at the correct index, and add the inputted data in between it and the previous node.
					if(counter == index)
					{
						if(temp == m_pHead)
						{
							newNode->next = m_pHead;
							m_pHead = newNode;
						}
						else
						{
							prev->next = newNode;
							newNode->next = temp;
							if(m_pTail == prev)
							{
								m_pTail = newNode;
							}
						}

						break;
					}

					prev = temp;
					temp = temp->next;

					counter++;
				}
			}

			return true;
		}
		
		// Method for removing an element from the list at an index.
		bool RemoveAt(u32 index)
		{
			if(m_pHead)
			{
				u32 counter = 0;

				NODE* prev = m_pHead;
				NODE* temp = m_pHead;

				while(temp)
				{
					// Follow the list until the index parameter is reached by the counter variable.
					if(counter == index)
					{
						if(temp == m_pHead)
						{
							// Head node data.
							temp = temp->next;

							if(m_pTail == m_pHead)
								m_pTail = temp;

							delete m_pHead;
							m_pHead = temp;
						}
						else
						{
							// Non-head node data.
							prev->next = temp->next;
					
							if(m_pTail == temp)
								m_pTail = prev;

							delete temp;
						}

						break;
					}

					prev = temp;
					temp = temp->next;

					counter++;
				}
			}

			return true;
		}

		// Method for removing inputted data from the list.
		void Remove(T data)
		{
			if(m_pHead)
			{
				NODE* prev = m_pHead;
				NODE* temp = m_pHead;

				while(temp)
				{
					// Find and remove the correct data.
					if(temp->data == data)
					{
						if(temp == m_pHead)
						{
							temp = temp->next;

							if(m_pTail == m_pHead)
								m_pTail = temp;

							delete m_pHead;
							m_pHead = temp;
						}
						else
						{
							prev->next = temp->next;

							if(m_pTail == temp)
								m_pTail = prev;

							delete temp;
						}

						break;
					}

					prev = temp;
					temp = temp->next;
				}
			}
		}

		// Method for releasing all data from the list, and for destroy the list.
		void Release()
		{
			NODE *current, *temp;
			current = temp = m_pHead;
			while(current)
			{
				current = current->next;
				delete temp;
				temp = current;
			}

			m_pHead = NULL;
			m_pTail = NULL;
		}

		//
		// List inline functions.
		//

		inline NODE* Head()
		{
			return m_pHead;
		}

		inline NODE* Tail()
		{
			return m_pTail;
		}

		inline bool Empty()
		{
			return m_pHead ? false : true;
		}
	};
	
	// CListPtr class
	// Class for creating a list data structure w/ void pointers.
	class CListPtr
	{
	public:
		// Pointer list node structure.
		typedef struct Node
		{
			Node*		next;
			void*		data;

		} NODE;
		
	private:
		NODE *m_pHead, *m_pTail;

	public:
		CListPtr();
		CListPtr(const CListPtr& other);
		CListPtr(CListPtr&& other);
		virtual ~CListPtr();

		void								PushBack(void* data);
		void*								PopFront();
		bool								InsertAt(void* data, u32 index);
		bool								RemoveAt(u32 index);
		void								Remove(void* data);

		void								Release();

		inline NODE*						Head();
		inline NODE*						Tail();
		inline bool							Empty();
	};

	//
	// pointer list inline functions.
	//

	inline CListPtr::NODE* CListPtr::Head()
	{
		return m_pHead;
	}

	inline CListPtr::NODE* CListPtr::Tail()
	{
		return m_pTail;
	}

	inline bool CListPtr::Empty()
	{
		return m_pHead ? false : true;
	}
};

#endif