//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Utilities\CDataStack.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CDataStack.h"

namespace Util
{
	//---------------------------------------------------------------------------------------------
	// Pointer stack class.
	//---------------------------------------------------------------------------------------------

	CPtrStack::CPtrStack()
	{
		m_stack.elems = NULL;
	}

	CPtrStack::CPtrStack(const CPtrStack& other)
	{
	}

	CPtrStack::CPtrStack(CPtrStack&& other)
	{
	}
	
	CPtrStack::~CPtrStack()
	{
		Release();
	}

	// Method for creating the stack.
	void CPtrStack::Initialize(int elemSize, void(*Release) (void*))
	{
		m_stack.elemSize				= elemSize;
		m_stack.length					= 0;
		m_stack.allocLength				= 4;
		m_stack.elems					= (u32*)malloc(m_stack.allocLength * elemSize);
		m_stack.Release					= Release;
	}
	
	// Method for pushing an elements to the stack.
	void CPtrStack::Push(void* elem)
	{
		// If the stack hasn't been initialized yet, initialize it now.
		if(!m_stack.elems)
			Initialize();

		if(m_stack.length == m_stack.allocLength)
			IncreaseStack();

		void* target = (u8*)m_stack.elems + m_stack.length * m_stack.elemSize;
		memcpy(target, elem, m_stack.elemSize);

		m_stack.length++;
	}
	
	// Method for popping an elements from the stack and store in the the elem parameter.
	void CPtrStack::Pop(void* elem)
	{
		if(!m_stack.elems)
			return;

		void* source = (u8*)m_stack.elems + (m_stack.length - 1) * m_stack.elemSize;
		memcpy(elem, source, m_stack.elemSize);
		m_stack.length--;
	}
	
	// Method for getting an elements from the stack at an index.
	u8* CPtrStack::At(u32 index)
	{
		return (u8*)m_stack.elems + index * m_stack.elemSize;
	}

	// Method for releasing the stack's data.
	void CPtrStack::Release()
	{
		if(m_stack.Release)
		{
			for(u32 i = 0; i < m_stack.length; i++)
			{
				m_stack.Release((u8*)m_stack.elems + i * m_stack.length);
			}
		}

		SAFE_DELETE(m_stack.elems);
		m_stack.length = 0;
	}
	
	// Method for increasing the allocated size of the stack.
	void CPtrStack::IncreaseStack()
	{
		m_stack.allocLength		*= 2;
		m_stack.elems			= (u32*)realloc(m_stack.elems, m_stack.allocLength * m_stack.elemSize);
	}
};