//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Utilities\CDebugError.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CDebugError.h"

namespace Util
{
	CErrorLog::CErrorLog()
	{
	}
	
	CErrorLog::CErrorLog(const CErrorLog& other)
	{
	}
	
	CErrorLog::~CErrorLog()
	{
	}
	
	// Method for saving an error
	int CErrorLog::SaveLog(int code, HWND hWnd)
	{
		if(code != Err::NONE)
		{
			time_t _time;
			struct tm* timeInfo = new struct tm();
			char timeStr[256];

			// Get the current local system time.
			time(&_time);
			localtime_s(timeInfo, &_time);
			asctime_s(timeStr, timeInfo);

			// Remove trailing return character from the time string.
			for(int i = 0; i < 256; i++)
			{
				if(timeStr[i] == '\n')
				{
					timeStr[i] = 0;
					break;
				}
			}

			std::string errorMsg;
			std::string errorMsgPlain = "";
			std::string filename = ".\\ERROR.txt";

			// Set the error string prefix.
			std::stringstream ss;
			ss << std::hex << code;
			errorMsg = "0x" + ss.str() + ":   ";
			errorMsg += std::string(timeStr) + ":   ";

			// Append error message to string.
			FindError(code, errorMsgPlain);

			errorMsg += errorMsgPlain;
				
			if(errorMsg != "")
			{
				// Save the error log message.
				std::ofstream file(filename, std::ios::app);

				file << errorMsg;
				file << '\n';
			}

			// Display a message box with the error message is a window handle exists, and is passed to this function.
			if(hWnd) MessageBox(hWnd, (errorMsgPlain + "\nError log saved to " + filename).c_str(), "An Error Has Occured", MB_OK);
		}

		return code;
	}

	// Method for finding the error code, and write its log to the 'errorMsg' string.
	void CErrorLog::FindError(int code, std::string& errorMsg)
	{
		if(WindowErrors(code, errorMsg))		return;
		if(DirectX11Errors(code, errorMsg))		return;
		if(ShaderErrors(code, errorMsg))		return;
		if(TextureErrors(code, errorMsg))		return;
		if(GeometryErrors(code, errorMsg))		return;
		if(FBXLoaderErrors(code, errorMsg))		return;
		if(AudioErrors(code, errorMsg))			return;
		if(SysInfoErrors(code, errorMsg))		return;
		if(InputErrors(code, errorMsg))			return;
		if(DebugObjectErrors(code, errorMsg))	return;

		ExtendedFindError(code, errorMsg);
	}

	// Window errors method.
	bool CErrorLog::WindowErrors(int code, std::string& errorMsg)
	{
		switch(code)
		{
			case Err::ERR_WINDOW_DISPLAY_TO_SMALL:
				errorMsg += "FATAL ERROR: Window display settings are to small!";
				return true;
			case Err::ERR_WINDOW_HANDLE_NULL:
				errorMsg += "FATAL ERROR: Application window is null!";
				return true;
			default:
				return false;
		}
	}
	
	// DirectX 11 errors method.
	bool CErrorLog::DirectX11Errors(int code, std::string& errorMsg)
	{
		switch(code)
		{
			case Err::ERR_DX11_FAILED_TO_CREATE_FACTORY:
				errorMsg += "FATAL ERROR: Failed to create the DirectX graphics interface factory!";
				return true;
			case Err::ERR_DX11_FAILED_TO_RETRIEVE_PRIMARY_ADAPTER:
				errorMsg += "FATAL ERROR: Failed to retrieve the primary video card!";
				return true;
			case Err::ERR_DX11_FAILED_TO_RETRIEVE_PRIMARY_MONITOR:
				errorMsg += "FATAL ERROR: Failed to retrieve the primary monitor!";
				return true;
			case Err::ERR_DX11_FAILED_TO_RETRIEVE_MONITOR_DISPLAY_MODE_COUNT:
				errorMsg += "FATAL ERROR: Failed to retrieve the amount of display modes for the primary moniter!";
				return true;
			case Err::ERR_DX11_FAILED_TO_RETRIEVE_MONITOR_DISPLAY_MODE_LIST:
				errorMsg += "FATAL ERROR: Failed to store a list of the display modes for the primary monitor!";
				return true;
			case Err::ERR_DX11_FAILED_TO_RETRIVE_ADAPTER_DESCRIPTION:
				errorMsg += "FATAL ERROR: Failed to retrieve infomation about the primary video card!";
				return true;
			case Err::ERR_DX11_FAILED_TO_CREATE_DEVICE_AND_SWAPCHAIN:
				errorMsg += "FATAL ERROR: Failed to create the DirectX 11 device and/or swapchain!";
				return true;
			case Err::ERR_DX11_FAILED_TO_RETRIEVE_BACK_BUFFEER_POINTER:
				errorMsg += "FATAL ERROR: Failed to retrieve the back buffer!";
				return true;
			case Err::ERR_DX11_FAILED_TO_CREATE_RENDER_TARGET_VIEW:
				errorMsg += "FATAL ERROR: Failed to create the render target view!";
				return true;
			case Err::ERR_DX11_FAILED_TO_CREATE_DEPTH_STENCIL_BUFFER:
				errorMsg += "FATAL ERROR: Failed to create the depth stencil buffer!";
				return true;
			case Err::ERR_DX11_CREATE_DEPTH_STENCIL_STATE:
				errorMsg += "FATAL ERROR: Failed to create the depth stencil state!";
				return true;
			case Err::ERR_DX11_FAILED_TO_CREATE_DEPTH_STENCIL_VIEW:
				errorMsg += "FATAL ERROR: Failed to create the depth stencil view!";
				return true;
			case Err::ERR_DX11_FAILED_TO_CREATE_RASTERIZER_STATE:
				errorMsg += "FATAL ERROR: Failed to create the rasterizer state!";
				return true;
			case Err::ERR_DX11_FAILED_TO_CREATE_BLEND_STATE:
				errorMsg += "FATAL ERROR: Failed to create the blend state!";
				return true;
			case Err::ERR_DX11_FAILED_TO_CREATE_SAMPLER_STATE:
				errorMsg += "FATAL ERROR: Failed to create the sampler state!";
				return true;
			case Err::ERR_DX11_FAILED_TO_RESIZE_SWAPCHAIN:
				errorMsg += "FATAL ERROR: Failed to resize the swapchain!";
				return true;
			case Err::ERR_DX11_CREATE_DEPTH_STENCIL_RESOURCE_VIEW:
				errorMsg += "FATAL ERROR: Failed to create the depth stencil resource view!";
				return true;
			case Err::ERR_DX11_FAILED_TO_UPDATE_WINDOW_MODE:
				errorMsg += "FATAL ERROR: Failed to update swapchain window mode!";
				return true;
			default:
				return false;
		}
	}
	
	// Shader errors method.
	bool CErrorLog::ShaderErrors(int code, std::string& errorMsg)
	{
		switch(code)
		{
			case Err::ERR_SHADER_INVALID_CONSTANT_BUFFER_INDEX:
				errorMsg += "FATAL ERROR: Invalid constant buffer!";
				return true;
			case Err::ERR_SHADER_FAILED_TO_CREATE_CONSTANT_BUFFER:
				errorMsg += "FATAL ERROR: Failed to create a constant buffer!";
				return true;
			case Err::ERR_SHADER_FAILED_TO_COMPILE_VERTEX_SHADER:
				errorMsg += "FATAL ERROR: Failed to compile a vertex shader!";
				return true;
			case Err::ERR_SHADER_FAILED_TO_CREATE_INPUT_LAYOUT:
				errorMsg += "FATAL ERROR: Failed to create an input layout!";
				return true;
			case Err::ERR_SHADER_FAILED_TO_COMPILE_HULL_SHADER:
				errorMsg += "FATAL ERROR: Failed to compile a hull shader!";
				return true;
			case Err::ERR_SHADER_FAILED_TO_COMPILE_DOMAIN_SHADER:
				errorMsg += "FATAL ERROR: Failed to compile a domain shader!";
				return true;
			case Err::ERR_SHADER_FAILED_TO_COMPILE_PIXEL_SHADER:
				errorMsg += "FATAL ERROR: Failed to compile a pixel shader!";
				return true;
			case Err::ERR_SHADER_FAILED_TO_COMPILE_GEOMETRY_SHADER:
				errorMsg += "FATAL ERROR: Failed to compile a geometry shader!";
				return true;
			case Err::ERR_SHADER_FAILED_TO_MAP_CONSTANT_BUFFER:
				errorMsg += "FATAL ERROR: Failed to map a constant buffer!";
				return true;
			case Err::ERR_SHADER_FAILED_TO_SAVE_COMPILATION_ERROR:
				errorMsg += "FATAL ERROR: Failed to save a shader compilation error!";
				return true;
			case Err::ERR_SHADER_FAILED_TO_CREATE_VERTEX_SHADER:
				errorMsg += "FATAL ERROR: Failed to create a vertex shader!";
				return true;
			case Err::ERR_SHADER_FAILED_TO_CREATE_HULL_SHADER:
				errorMsg += "FATAL ERROR: Failed to create a hull shader!";
				return true;
			case Err::ERR_SHADER_FAILED_TO_CREATE_DOMAIN_SHADER:
				errorMsg += "FATAL ERROR: Failed to create a domain shader!";
				return true;
			case Err::ERR_SHADER_FAILED_TO_CREATE_PIXEL_SHADER:
				errorMsg += "FATAL ERROR: Failed to create a pixel shader!";
				return true;
			case Err::ERR_SHADER_FAILED_TO_CREATE_GEOMETRY_SHADER:
				errorMsg += "FATAL ERROR: Failed to create a geometry shader!";
				return true;
			case Err::ERR_SHADER_FAILED_TO_COMPILE_COMPUTE_SHADER:
				errorMsg += "FATAL ERROR: Failed to compile a compute shader!";
				return true;
			case Err::ERR_SHADER_FAILED_TO_CREATE_COMPUTE_SHADER:
				errorMsg += "FATAL ERROR: Failed to create a compute shader!";
				return true;
			default:
				return false;
		}
	}
	
	// Texture errors method.
	bool CErrorLog::TextureErrors(int code, std::string& errorMsg)
	{
		switch(code)
		{
			case Err::ERR_TEXTURE_FAILED_TO_CREATE_TEXTURE2D:
				errorMsg += "FATAL ERROR: Failed to create a 2d texture!";
				return true;
			case Err::ERR_TEXTURE_FAILED_TO_CREATE_RESOURCE_VIEW:
				errorMsg += "FATAL ERROR: Failed to create a shader resource view!";
				return true;
			case Err::ERR_TEXTURE_RENDER_TO_FAILED_TO_CREATE_TEXTURE2D:
				errorMsg += "FATAL ERROR: Failed to create a 2d render to texture!";
				return true;
			case Err::ERR_TEXTURE_RENDER_TO_FAILED_TO_CREATE_TARGET:
				errorMsg += "FATAL ERROR: Failed to create a render to texture target view!";
				return true;
			case Err::ERR_TEXTURE_RENDER_TO_FAILED_TO_CREATE_RESOURCE_VIEW:
				errorMsg += "FATAL ERROR: Failed to create a render to texture shader resourse view!";
				return true;
			case Err::ERR_TEXTURE_FAILED_TO_OPEN_TEXTURE_FILE:
				errorMsg += "FATAL ERROR: Failed to open texture file!";
				return true;
			default:
				return false;
		}
	}

	// Geometry errors method.
	bool CErrorLog::GeometryErrors(int code, std::string& errorMsg)
	{
		switch(code)
		{
			case Err::ERR_GEOMETRY_FAILED_TO_CREATE_VERTEX_BUFFER:
				errorMsg += "FATAL ERROR: Failed to create a vertex buffer!";
				return true;
			case Err::ERR_GEOMETRY_FAILED_TO_CREATE_INDEX_BUFFER:
				errorMsg += "FATAL ERROR: Failed to create an index buffer!";
				return true;
			case Err::ERR_GEOMETRY_FAILED_TO_CREATE_INSTANCE_BUFFER:
				errorMsg += "FATAL ERROR: Failed to create a instance buffer!";
				return true;
			default:
				return false;
		}
	}

	// FBX loader errors method.
	bool CErrorLog::FBXLoaderErrors(int code, std::string& errorMsg)
	{
		switch(code)
		{
			case Err::ERR_FBX_LOADER_FAILED_TO_INITIALIZE_FBX_IMPORTER:
				errorMsg += "FATAL ERROR: Failed to the FBX importer!";
				return true;
			case Err::ERR_FBX_LOADER_FAILED_TO_IMPORT_SCENE:
				errorMsg += "FATAL ERROR: Failed to import an FBX scene!";
				return true;
			case Err::ERR_FBX_LOADER_INVALID_MATERIAL_INDEX:
				errorMsg += "FATAL ERROR: Invalid material index!";
				return true;
			default:
				return false;
		}
	}
	
	// XAudio2 errors method.
	bool CErrorLog::AudioErrors(int code, std::string& errorMsg)
	{
		switch(code)
		{
			case Err::ERR_XAUDIO2_FAILED_TO_CREATE_SOUND_ENGINE:
				errorMsg += "FATAL ERROR: Failed to create the xaudio2 engine!";
				return true;
			case Err::ERR_XAUDIO2_FAILED_TO_CREATE_MASTER_VOICE:
				errorMsg += "FATAL ERROR: Failed to create the xaudio2 master voice!";
				return true;
				
			case Err::ERR_WAVEFILE_FAILED_TO_OPEN_FILE:
				errorMsg += "FATAL ERROR: Failed to open wave file!";
				return true;
			case Err::ERR_WAVEFILE_FAILED_TO_READ_FILE:
				errorMsg += "FATAL ERROR: Failed to read wave file!";
				return true;
			case Err::ERR_WAVEFILE_INVALID_FILE_FORMAT_RIFF:
				errorMsg += "FATAL ERROR: Invalid wave file format!";
				return true;
			case Err::ERR_WAVEFILE_INVALID_FILE_FORMAT_WAVE:
				errorMsg += "FATAL ERROR: Invalid wave file format!";
				return true;
			case Err::ERR_WAVEFILE_INVALID_FILE_FORMAT_FMT:
				errorMsg += "FATAL ERROR: Invalid wave file format!";
				return true;
			case Err::ERR_WAVEFILE_INVALID_FILE_FORMAT_PCM:
				errorMsg += "FATAL ERROR: Invalid wave file format!";
				return true;
			case Err::ERR_WAVEFILE_INVALID_SAMPLE_RATE:
				errorMsg += "FATAL ERROR: Invalid wave file sample rate!";
				return true;
			case Err::ERR_WAVEFILE_INVALID_BIT_RATE:
				errorMsg += "FATAL ERROR: Invalid wave file bit rate!";
				return true;
			case Err::ERR_WAVEFILE_INVALID_DATA_HEADER:
				errorMsg += "FATAL ERROR: Invalid wave file data header!";
				return true;
			case Err::ERR_WAVEFILE_INVALID_FILE_LENGTH:
				errorMsg += "FATAL ERROR: Invalid wave file length!";
				return true;
				
			case Err::ERR_XAUDIO2_FAILED_TO_CREATE_SOURCE_VOICE:
				errorMsg += "FATAL ERROR: Failed to create a xaudio2 source voice!";
				return true;
			case Err::ERR_XAUDIO2_FAILED_TO_SUBMIT_SOURCE_BUFFER:
				errorMsg += "FATAL ERROR: Failed to submit a source buffer!";
				return true;
			case Err::ERR_XAUDIO2_FAILED_TO_START_SOURCE_VOICE:
				errorMsg += "FATAL ERROR: Failed to start a source voice!";
				return true;
			case Err::ERR_XAUDIO2_FAILED_INITIALIZE_X3DAUDIO:
				errorMsg += "FATAL ERROR: Failed to initialize X3DAudio!";
				return true;
			default:
				return false;
		}
	}

	// System info errors method.
	bool CErrorLog::SysInfoErrors(int code, std::string& errorMsg)
	{
		switch(code)
		{
			case Err::ERR_SYSINFO_FAILED_TO_CALC_GLOBAL_MEM_STATUS:
				errorMsg += "FATAL ERROR: Failed to calculate global memory status!";
				return true;
			case Err::ERR_SYSINFO_FAILED_TO_CALC_PROCESS_MEM_USAGE:
				errorMsg += "FATAL ERROR: Failed to calculate the current process memory usage!";
				return true;
			case Err::ERR_SYSINFO_FAILED_TO_OPEN_CPU_QUERY:
				errorMsg += "FATAL ERROR: Failed to open a cpu query!";
				return true;
			case Err::ERR_SYSINFO_FAILED_TO_ADD_CPU_COUNTER:
				errorMsg += "FATAL ERROR: Failed to add a cpu counter!";
				return true;
			case Err::ERR_SYSINFO_FAILED_TO_COLLECT_CPU_DATA:
				errorMsg += "FATAL ERROR: Failed to collect cpu data!";
				return true;
			case Err::ERR_SYSINFO_FAILED_GET_FORMATTED_COUNTER_VALUE:
				errorMsg += "FATAL ERROR: Failed to get formatted counter value!";
				return true;
			case Err::ERR_SYSINFO_FAILED_GET_PROCESS_TIMES:
				errorMsg += "FATAL ERROR: Failed to get process times!";
				return true;
			default:
				return false;
		}
	}

	// Input error method.
	bool CErrorLog::InputErrors(int code, std::string& errorMsg)
	{
		switch(code)
		{
			case Err::ERR_RAW_INPUT_DEVICE_REGISTRATION:
				errorMsg += "FATAL ERROR: Failed to register raw input device!";
				return true;
			default:
				return false;
		}
	}

	// Input error method.
	bool CErrorLog::DebugObjectErrors(int code, std::string& errorMsg)
	{
		switch(code)
		{
			case Err::ERR_DEBUG_OBJECT_FAILED_TO_CREATE_VERTEX_BUFFER:
				errorMsg += "FATAL ERROR: Failed create a debug object's vertex buffer!";
				return true;
			default:
				return false;
		}
	}
};