//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Utilities\CDebugError.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CDEBUGERROR_H
#define CDEBUGERROR_H

#include <Windows.h>
#include <time.h>
#include <string>
#include <sstream>
#include <fstream>

namespace Util
{
	namespace Err
	{
		// Enumerator for engine error codes.
		enum : int
		{
			NONE																	= 0,

			// Window errors (1000 - 1999).
			ERR_WINDOW_DISPLAY_TO_SMALL												= 1000,
			ERR_WINDOW_HANDLE_NULL													= 1001,

			// DirectX 11 errors (2000 - 2999).
			ERR_DX11_FAILED_TO_CREATE_FACTORY										= 2000,
			ERR_DX11_FAILED_TO_RETRIEVE_PRIMARY_ADAPTER								= 2001,
			ERR_DX11_FAILED_TO_RETRIEVE_PRIMARY_MONITOR								= 2002,
			ERR_DX11_FAILED_TO_RETRIEVE_MONITOR_DISPLAY_MODE_COUNT					= 2003,
			ERR_DX11_FAILED_TO_RETRIEVE_MONITOR_DISPLAY_MODE_LIST					= 2004,
			ERR_DX11_FAILED_TO_RETRIVE_ADAPTER_DESCRIPTION							= 2005,
			ERR_DX11_FAILED_TO_CREATE_DEVICE_AND_SWAPCHAIN							= 2006,
			ERR_DX11_FAILED_TO_RETRIEVE_BACK_BUFFEER_POINTER						= 2007,
			ERR_DX11_FAILED_TO_CREATE_RENDER_TARGET_VIEW							= 2008,
			ERR_DX11_FAILED_TO_CREATE_DEPTH_STENCIL_BUFFER							= 2009,
			ERR_DX11_CREATE_DEPTH_STENCIL_STATE										= 2010,
			ERR_DX11_FAILED_TO_CREATE_DEPTH_STENCIL_VIEW							= 2011,
			ERR_DX11_FAILED_TO_CREATE_RASTERIZER_STATE								= 2012,
			ERR_DX11_FAILED_TO_CREATE_BLEND_STATE									= 2013,
			ERR_DX11_FAILED_TO_CREATE_SAMPLER_STATE									= 2014,
			ERR_DX11_FAILED_TO_RESIZE_SWAPCHAIN										= 2015,
			ERR_DX11_CREATE_DEPTH_STENCIL_RESOURCE_VIEW								= 2016,
			ERR_DX11_FAILED_TO_UPDATE_WINDOW_MODE									= 2017,

			// Shader errors (3000 - 3999).
			ERR_SHADER_INVALID_CONSTANT_BUFFER_INDEX								= 3000,
			ERR_SHADER_FAILED_TO_CREATE_CONSTANT_BUFFER								= 3001,
			ERR_SHADER_FAILED_TO_COMPILE_VERTEX_SHADER								= 3002,
			ERR_SHADER_FAILED_TO_CREATE_INPUT_LAYOUT								= 3003,
			ERR_SHADER_FAILED_TO_COMPILE_HULL_SHADER								= 3004,
			ERR_SHADER_FAILED_TO_COMPILE_DOMAIN_SHADER								= 3005,
			ERR_SHADER_FAILED_TO_COMPILE_PIXEL_SHADER								= 3006,
			ERR_SHADER_FAILED_TO_COMPILE_GEOMETRY_SHADER							= 3007,
			ERR_SHADER_FAILED_TO_MAP_CONSTANT_BUFFER								= 3008,
			ERR_SHADER_FAILED_TO_SAVE_COMPILATION_ERROR								= 3009,
			ERR_SHADER_FAILED_TO_CREATE_VERTEX_SHADER								= 3010,
			ERR_SHADER_FAILED_TO_CREATE_HULL_SHADER									= 3011,
			ERR_SHADER_FAILED_TO_CREATE_DOMAIN_SHADER								= 3012,
			ERR_SHADER_FAILED_TO_CREATE_PIXEL_SHADER								= 3013,
			ERR_SHADER_FAILED_TO_CREATE_GEOMETRY_SHADER								= 3014,
			ERR_SHADER_FAILED_TO_COMPILE_COMPUTE_SHADER								= 3015,
			ERR_SHADER_FAILED_TO_CREATE_COMPUTE_SHADER								= 3016,

			// Texture errors (4000 - 4999).
			ERR_TEXTURE_FAILED_TO_CREATE_TEXTURE2D									= 4000,
			ERR_TEXTURE_FAILED_TO_CREATE_RESOURCE_VIEW								= 4001,
			ERR_TEXTURE_RENDER_TO_FAILED_TO_CREATE_TEXTURE2D						= 4002,
			ERR_TEXTURE_RENDER_TO_FAILED_TO_CREATE_TARGET							= 4003,
			ERR_TEXTURE_RENDER_TO_FAILED_TO_CREATE_RESOURCE_VIEW					= 4004,
			ERR_TEXTURE_FAILED_TO_OPEN_TEXTURE_FILE									= 4005,
			
			// Geometry errors (5000 - 5999).
			ERR_GEOMETRY_FAILED_TO_CREATE_VERTEX_BUFFER								= 5000,
			ERR_GEOMETRY_FAILED_TO_CREATE_INDEX_BUFFER								= 5001,
			ERR_GEOMETRY_FAILED_TO_CREATE_INSTANCE_BUFFER							= 5002,

			// FBX loader errors (6000 - 6999).
			ERR_FBX_LOADER_FAILED_TO_INITIALIZE_FBX_IMPORTER						= 6000,
			ERR_FBX_LOADER_FAILED_TO_IMPORT_SCENE									= 6001,
			ERR_FBX_LOADER_INVALID_MATERIAL_INDEX									= 6002,

			// Audio errors (7000 - 7999).
			ERR_XAUDIO2_FAILED_TO_CREATE_SOUND_ENGINE								= 7000,
			ERR_XAUDIO2_FAILED_TO_CREATE_MASTER_VOICE								= 7001,

			ERR_WAVEFILE_FAILED_TO_OPEN_FILE										= 7002,
			ERR_WAVEFILE_FAILED_TO_READ_FILE										= 7003,
			ERR_WAVEFILE_INVALID_FILE_FORMAT_RIFF									= 7004,
			ERR_WAVEFILE_INVALID_FILE_FORMAT_WAVE									= 7005,
			ERR_WAVEFILE_INVALID_FILE_FORMAT_FMT									= 7006,
			ERR_WAVEFILE_INVALID_FILE_FORMAT_PCM									= 7007,
			ERR_WAVEFILE_INVALID_SAMPLE_RATE										= 7008,
			ERR_WAVEFILE_INVALID_BIT_RATE											= 7009,
			ERR_WAVEFILE_INVALID_DATA_HEADER										= 7010,
			ERR_WAVEFILE_INVALID_FILE_LENGTH										= 7011,
			
			ERR_XAUDIO2_FAILED_TO_CREATE_SOURCE_VOICE								= 7012,
			ERR_XAUDIO2_FAILED_TO_SUBMIT_SOURCE_BUFFER								= 7013,
			ERR_XAUDIO2_FAILED_TO_START_SOURCE_VOICE								= 7014,
			ERR_XAUDIO2_FAILED_INITIALIZE_X3DAUDIO									= 7015,
			
			// System info errors (8000 - 8999).
			ERR_SYSINFO_FAILED_TO_CALC_GLOBAL_MEM_STATUS							= 8000,
			ERR_SYSINFO_FAILED_TO_CALC_PROCESS_MEM_USAGE							= 8001,

			ERR_SYSINFO_FAILED_TO_OPEN_CPU_QUERY									= 8002,
			ERR_SYSINFO_FAILED_TO_ADD_CPU_COUNTER									= 8003,
			ERR_SYSINFO_FAILED_TO_COLLECT_CPU_DATA									= 8004,
			ERR_SYSINFO_FAILED_GET_FORMATTED_COUNTER_VALUE							= 8005,
			ERR_SYSINFO_FAILED_GET_PROCESS_TIMES									= 8006,

			// Input errors (9000 - 9999).
			ERR_RAW_INPUT_DEVICE_REGISTRATION										= 9000,

			// Debug object errors (10000 - 10999).
			ERR_DEBUG_OBJECT_FAILED_TO_CREATE_VERTEX_BUFFER							= 10000,

		};
	};

	// CErrorLog class.
	// Class for creating, and updating the error log file.
	class CErrorLog
	{
	public:
		CErrorLog();
		CErrorLog(const CErrorLog& other);
		virtual ~CErrorLog();

		int									SaveLog(int code, HWND hWnd = NULL);

	protected:
		virtual void						ExtendedFindError(int code, std::string& errorMsg) { return; }
	
	private:
		void								FindError(int code, std::string& errorMsg);
		
		bool								WindowErrors(int code, std::string& errorMsg);
		bool								DirectX11Errors(int code, std::string& errorMsg);
		bool								ShaderErrors(int code, std::string& errorMsg);
		bool								TextureErrors(int code, std::string& errorMsg);
		bool								GeometryErrors(int code, std::string& errorMsg);
		bool								FBXLoaderErrors(int code, std::string& errorMsg);
		bool								AudioErrors(int code, std::string& errorMsg);
		bool								SysInfoErrors(int code, std::string& errorMsg);
		bool								InputErrors(int code, std::string& errorMsg);
		bool								DebugObjectErrors(int code, std::string& errorMsg);
	};
};

#endif