//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Engine
//
// File: Utilities\CSystemInfo.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CSYSTEMINFO_H
#define CSYSTEMINFO_H

#include <Windows.h>
#include <Psapi.h>
#include <Pdh.h>
#include "..\Application\CDX11Object.h"
#include "..\Utilities\CDebugError.h"
#include "..\Globals\CGlobals.h"

#pragma comment(lib, "pdh.lib")

namespace Util
{
	// CSysInfo class (singleton)
	// Class that store varies information about the current systems information.
	// Primarily for debugging purposes.
	class CSysInfo
	{
	public:
		static CSysInfo& Instance()
		{
			static CSysInfo instance;
			return instance;
		}
		
		int									CalculateMemoryInfo();
		int									CalculateMemoryUsage(u32* vmemory = NULL, u32* pmemory = NULL);
		
		int									InitCPUCalculations();
		int									CalculateCPUUsage();

		inline u64							TotalVMemory() const;
		inline u64							TotalVMemoryUsed() const;
		inline u64							TotalPMemory() const;
		inline u64							TotalPMemoryUsed() const;

		inline u32							ProcessLastVMemUsage() const;
		inline u32							ProcessLastPMemUsage() const;

		inline float						TotalCPUUsage() const;
		inline float						ProcessCPUUsage() const;

	private:
		CSysInfo();
		CSysInfo(const CSysInfo& other) = delete;
		virtual ~CSysInfo();

		int									CalculateProcessCPUUsage();
		
		void								Release();

	private:
		u64									m_totalVMemory;
		u64									m_totalVMemoryUsed;
		u64									m_totalPMemory;
		u64									m_totalPMemoryUsed;

		u32									m_processVMemoryUsage;
		u32									m_processPMemoryUsage;

		float								m_totalCPUUsage;
		float								m_processCPUUsage;

		u32									m_lastMemSampleTime;
		u32									m_lastCpuSampleTime;

		u32									m_processorCount;
		ULARGE_INTEGER						m_lastCPU;
		ULARGE_INTEGER						m_lastSysCPU;
		ULARGE_INTEGER						m_lastUserCPU;
		
		HANDLE								m_currentProcess;

		HQUERY								m_cpuQuery;
		HCOUNTER							m_cpuCounter;
	};

	// Virtual memory getters.

	inline u64 CSysInfo::TotalVMemory() const
	{
		return m_totalVMemory;
	}

	inline u64 CSysInfo::TotalVMemoryUsed() const
	{
		return m_totalVMemoryUsed;
	}

	// Physical memory getters.

	inline u64 CSysInfo::TotalPMemory() const
	{
		return m_totalPMemory;
	}

	inline u64 CSysInfo::TotalPMemoryUsed() const
	{
		return m_totalPMemoryUsed;
	}
	
	// Process virtual memory getters.

	inline u32 CSysInfo::ProcessLastVMemUsage() const
	{
		return m_processVMemoryUsage;
	}

	inline u32 CSysInfo::ProcessLastPMemUsage() const
	{
		return m_processPMemoryUsage;
	}
	
	// Total and process CPU usage getters.

	inline float CSysInfo::TotalCPUUsage() const
	{
		return m_totalCPUUsage;
	}

	inline float CSysInfo::ProcessCPUUsage() const
	{
		return m_processCPUUsage;
	}
};

#endif