//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Project II
//
// File: Actors\CPlayer.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CPlayer.h"

namespace Actor
{
	CPlayer::CPlayer()
	{
		m_acceleration						= 5.0f;
		m_speed								= 2.0f;

		m_lookSensitivity					= Math::CVector2(0.1f, 0.1f);

		m_pCamera							= NULL;
		m_pPlayerHUD						= NULL;

		m_pPlayerMesh						= NULL;

		m_pInput							= NULL;
		m_pWinData							= NULL;

		m_pRenderData						= NULL;
		m_pTimer							= NULL;
		m_pGameUI							= NULL;
	}

	CPlayer::CPlayer(const CPlayer& other)
	{
	}

	CPlayer::~CPlayer()
	{
	}

	int CPlayer::Initialize()
	{
		int retCode = Util::Err::NONE;

		// Allocate memory.
		m_pTransform						= Math::CTransformSharedPtr(new Math::CTransform());
		m_pCamera							= new CPlayerCamera();
		m_pPlayerHUD						= new UI::CPlayerHUD();

		if((retCode = m_pCamera->Initialize(m_pDX11Data->viewport.Width, m_pDX11Data->viewport.Height)) != Util::Err::NONE)
			return retCode;

		// Player HUD.
		m_pPlayerHUD->DirectXData(m_pDX11Data);
		m_pPlayerHUD->GameUI(m_pGameUI);
		m_pPlayerHUD->Input(m_pInput);
		m_pPlayerHUD->RenderData(m_pRenderData);
		m_pPlayerHUD->Timer(m_pTimer);
		if((retCode = m_pPlayerHUD->Initialize()) != Util::Err::NONE)
			return retCode;
		
		m_position = Math::CVector3(0, 2, 5);
		m_eulerAngles = Math::CVector3(0.0f, Math::_Pi, 0.0f);
		m_pTransform->AddChild(m_pCamera->Camera()->Transform().get());

		// If the player mesh was set before the transform was created, 
		// set the player mesh transform to be a child of the player transform.
		if(m_pPlayerMesh)
		{
			m_pTransform->AddChild(m_pPlayerMesh->Transform().get());
			m_pPlayerMesh->Transform()->LocalPosition(Math::CVector3::Zero());
			m_pPlayerMesh->Transform()->LocalEuler(Math::CVector3::Zero());
		}

		m_pTransform->Rotation(Math::CQuaternion(m_eulerAngles));
		m_pTransform->Position(m_position);
		m_pTransform->Update();

		return retCode;
	}

	int CPlayer::Update()
	{
		int retCode = Util::Err::NONE;
		
		UpdateRotation();
		UpdateMotion();

		m_pTransform->Update();
		
		if((retCode = m_pPlayerHUD->Update()) != Util::Err::NONE)
			return retCode;

		if((retCode = m_pCamera->Update()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	int CPlayer::Render()
	{
		int retCode = Util::Err::NONE;

		return retCode;
	}

	int CPlayer::RenderHUD()
	{
		int retCode = Util::Err::NONE;

		if((retCode = m_pPlayerHUD->Render()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	// Method for caluclating player rotation.
	void CPlayer::UpdateRotation()
	{
		// Mouse player rotation only when the right mouse button is held down.
		if(m_pInput->Mouse(VM_MOUSE_RIGHT))
		{
			RECT winRect;
			GetWindowRect(m_pWinData->hWnd, &winRect);
			App::CCursor::CenterCursor(winRect);
			App::CCursor::HideCursor(true);
			
			m_eulerAngles.x += m_pInput->MouseData().y * m_pTimer->SmoothDelta() * m_lookSensitivity.y;
			m_eulerAngles.y += m_pInput->MouseData().x * m_pTimer->SmoothDelta() * m_lookSensitivity.x;
			m_eulerAngles.x = Math::Clamp(m_eulerAngles.x, -Math::_PiOver2, Math::_PiOver2);

			// Check if player moved the mouse with the right mouse button down.
			if(m_pInput->MouseData().x != 0 || m_pInput->MouseData().y != 0)
			{
				m_pPlayerHUD->PlayerLooked(true);
			}

			m_pTransform->Rotation(Math::CQuaternion(m_eulerAngles));
			m_pPlayerHUD->ActivateHUD(false);
		}
		else
		{
			App::CCursor::HideCursor(false);
			m_pPlayerHUD->ActivateHUD(true);
		}
	}

	// Method for calculating player motion.
	void CPlayer::UpdateMotion()
	{
		m_targetVelocity = Math::CVector3::Zero();

		if(App::CControls::Instance().Activated("Forward"))
			m_targetVelocity += m_pTransform->Forward();
			
		if(App::CControls::Instance().Activated("Backward"))
			m_targetVelocity -= m_pTransform->Forward();
			
		if(App::CControls::Instance().Activated("Right"))
			m_targetVelocity += m_pTransform->Right();
			
		if(App::CControls::Instance().Activated("Left"))
			m_targetVelocity -= m_pTransform->Right();
			
		if(App::CControls::Instance().Activated("Up"))
			m_targetVelocity += m_pTransform->Up();
			
		if(App::CControls::Instance().Activated("Down"))
			m_targetVelocity -= m_pTransform->Up();

		// Check if the player moved.
		float lenSq = m_targetVelocity.LengthSq();

		if(lenSq > Math::_Epsilon)
		{
			m_pPlayerHUD->PlayerMoved(true);

			if(lenSq > 1.0f)
				m_targetVelocity.Normalize();
		}

		m_velocity = Math::Lerp(m_velocity, m_targetVelocity, m_acceleration * m_pTimer->SmoothDelta());
		m_position += m_velocity * m_speed * m_pTimer->SmoothDelta();

		m_pTransform->Position(m_position);
	}

	int CPlayer::PostResize()
	{
		int retCode = Util::Err::NONE;

		float width = m_pWinData->bFullscreen ? (float)GetSystemMetrics(SM_CXSCREEN) : m_pDX11Data->viewport.Width;
		float height = m_pWinData->bFullscreen ? (float)GetSystemMetrics(SM_CYSCREEN) : m_pDX11Data->viewport.Height;

		if(m_pCamera && (retCode = m_pCamera->PostResize(width, height)) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	void CPlayer::Release()
	{
		SAFE_RELEASE_DELETE(m_pCamera);
		SAFE_RELEASE_DELETE(m_pPlayerHUD);
	}

	// Method of setting a mesh to act as the player's body.
	void CPlayer::PlayerMesh(GFX::CMesh* playerMesh)
	{
		if(!playerMesh)
			return;

		m_pPlayerMesh = playerMesh;

		// If a player transform exists, parent it to the player mesh transform.
		if(m_pTransform)
		{
			m_pTransform->AddChild(m_pPlayerMesh->Transform().get());
			m_pPlayerMesh->Transform()->LocalPosition(Math::CVector3::Zero());
			m_pPlayerMesh->Transform()->LocalEuler(Math::CVector3::Zero());
			m_pPlayerMesh->Transform()->Update();
		}
	}
};