//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Project II
//
// File: Actors\CPlayerCamera.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CPlayerCamera.h"

namespace Actor
{
	CPlayerCamera::CPlayerCamera()
	{
		m_pCamera							= NULL;
		m_pUICamera							= NULL;
	}

	CPlayerCamera::CPlayerCamera(const CPlayerCamera& other)
	{
	}

	CPlayerCamera::~CPlayerCamera()
	{
	}

	int CPlayerCamera::Initialize(float screenWidth, float screenHeight)
	{
		int retCode = Util::Err::NONE;

		// Memory allocation.
		m_pCamera							= new Math::CCamera();
		m_pUICamera							= new Math::CCamera();
		
		// Setup the world camera.
		m_pCamera->CameraMode(Math::CAMERA_MODE::Perspective);
		m_pCamera->Width(100.0f);
		m_pCamera->Height(100.0f);
		m_pCamera->NearView(0.05f);
		m_pCamera->FarView(200.0f);
		m_pCamera->FieldOfView(65.0f * Math::_PiOver180);

		m_pCamera->Initialize(screenWidth, screenHeight);

		m_pCamera->Transform(Math::CTransformSharedPtr(new Math::CTransform()));
		m_pCamera->Transform()->Position(Math::CVector3(0.0f, 0.0f, 0.0f));
		m_pCamera->Update();
		
		// Setup the UI camera.
		m_pUICamera->CameraMode(Math::CAMERA_MODE::Orthographic);
		m_pUICamera->Width(100.0f);
		m_pUICamera->Height(100.0f);
		m_pUICamera->NearView(0.1f);
		m_pUICamera->FarView(100.0f);
		m_pUICamera->Initialize(screenWidth, screenHeight);

		m_pUICamera->Transform(Math::CTransformSharedPtr(new Math::CTransform()));
		m_pUICamera->Transform()->Position(Math::CVector3(0.0f, 0.0f, -10.0f));
		m_pUICamera->Update();
		
		return retCode;
	}

	int CPlayerCamera::Update()
	{
		int retCode = Util::Err::NONE;

		m_pCamera->Update();
		m_pUICamera->Update();

		return retCode;
	}

	int CPlayerCamera::PostResize(float screenWidth, float screenHeight)
	{
		int retCode = Util::Err::NONE;
		
		if(m_pCamera)
			m_pCamera->Initialize(screenWidth, screenHeight);
			
		if(m_pUICamera)
			m_pUICamera->Initialize(screenWidth, screenHeight);

		return retCode;
	}

	void CPlayerCamera::Release()
	{
		SAFE_DELETE(m_pUICamera);
		SAFE_DELETE(m_pCamera);
	}
};