//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Project II
//
// File: Game\CGame.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CGame.h"

namespace Game
{
	CGame::CGame()
	{
		m_pMatList							= NULL;

		m_pFBXLoader						= NULL;

		m_pSceneMain						= NULL;
		m_pGameUI							= NULL;
	}

	CGame::CGame(const CGame& other)
	{
	}

	CGame::~CGame()
	{
	}

	// Method for setting up the data for the game before initialization.
	int CGame::Setup()
	{
		int retCode = Util::Err::NONE;
		
		m_pWinData->bResizable = false;
		m_pWinData->width = 1600;
		m_pWinData->height = 900;
		m_pWinData->bFullscreen = false;
		m_pWinData->pWindowTitle = "DX11 Graphics";
		m_pDirectX->DXData()->backgroundColor = Math::CColor(0.0f, 0.0f, 0.0f);
		
		m_renderData.fogType = GFX::FOG_TYPE::Off;
		m_renderData.fogBuffer.fogColor = m_pDirectX->DXData()->backgroundColor;
		m_renderData.fogBuffer.extents = Math::CVector2(0.0f, 50.0f);
		m_renderData.fogBuffer.density = 0.02f;

		// Run the main scenes pre-initialization function.
		m_pSceneMain = new Scene::CSceneMain();
		m_pSceneMain->PostQuad(m_pPostQuad);

		if((retCode = m_pSceneMain->PreInitialize()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	// Method for initializing the game.
	int CGame::Initialize()
	{
		int retCode = Util::Err::NONE;

		if((retCode = CAppBase::Initialize()) != Util::Err::NONE)
			return retCode;
			
		// Allocate memory.
		m_pMatList							= new GFX::CMatList();
		m_pFBXLoader						= new GFX::CFBXLoader();
		m_pGameUI							= new UI::CGameUI();
		
		// Setup the material list.
		m_pMatList->DirectXData(m_pDX11Data);
		m_pMatList->RenderData(&m_renderData);

		if((retCode = m_pMatList->Initialize()) != Util::Err::NONE)
			return retCode;

		m_pSceneMain->Lighting()->DebugMaterial(m_pMatList->DebugMaterial());

		// Setup the game UI.
		m_pGameUI->DirectXData(m_pDX11Data);
		m_pGameUI->FT2Material(m_pMatList->FT2Material());
		m_pGameUI->UIImageMaterial(m_pMatList->UIImageMaterial());
		m_pGameUI->RenderData(&m_renderData);
		if((retCode = m_pGameUI->Initialize()) != Util::Err::NONE)
			return retCode;

		// Setup the scene.
		m_pSceneMain->DirectXData(m_pDX11Data);

		m_pSceneMain->MatList(m_pMatList);
		m_pSceneMain->GameUI(m_pGameUI);

		m_pSceneMain->Input(m_pInput);
		m_pSceneMain->WinData(m_pWinData);

		m_pSceneMain->DXStates(m_pDXStates);
		m_pSceneMain->AudioEngine(m_pAudioEngine);

		m_pSceneMain->FBXLoader(m_pFBXLoader);

		m_pSceneMain->Timer(m_pTimer);
		m_pSceneMain->RenderData(&m_renderData);
		
		if((retCode = InitializePostQuad()) != Util::Err::NONE)
			return retCode;
			
		if((retCode = m_pSceneMain->Initialize()) != Util::Err::NONE)
			return retCode;
			
		m_bInitialized = true;

		return retCode;
	}
	
	// Method for updating the game.
	int CGame::Update()
	{
		int retCode = Util::Err::NONE;
		
		m_pTimer->Tick();

		if((retCode = m_pSceneMain->Update()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	// Method for rendering the game.
	int CGame::Render()
	{
		int retCode = Util::Err::NONE;
			
		if((retCode = m_pDirectX->BeginScene()) != Util::Err::NONE)
			return retCode;
		
		// Main scene pre-render.
		if((retCode = m_pSceneMain->PreRender()) != Util::Err::NONE)
			return retCode;
		
		if((retCode = m_pPostQuad->BeginRenderToTexture()) != Util::Err::NONE)
			return retCode;

		// Render the main scene.
		if((retCode = m_pSceneMain->Render()) != Util::Err::NONE)
			return retCode;
		
		if((retCode = m_pPostQuad->Render()) != Util::Err::NONE)
			return retCode;

		if((retCode = m_pSceneMain->PostRender()) != Util::Err::NONE)
			return retCode;
		
		if((retCode = m_pDirectX->EndScene()) != Util::Err::NONE)
			return retCode;

		if((retCode = Screenshot()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	// Method for save screenshots of the main render target.
	int CGame::Screenshot()
	{
		int retCode = Util::Err::NONE;

		if(App::CControls::Instance().Activated("Screenshot"))
		{
			time_t _time;
			struct tm* timeInfo = new struct tm();
			wchar_t timeStr[256];

			// Get the current local system time.
			time(&_time);
			localtime_s(timeInfo, &_time);
			_wasctime_s(timeStr, timeInfo);

			for(u32 i = 0; i < 256; i++)
			{
				if(timeStr[i] < 0x20)
					timeStr[i] = 0;
				if(timeStr[i] == L':')
					timeStr[i] = L'-';
			}

			Util::CScreenshot::Instance().Screenshot(std::wstring(L"Screenshots\\SS - ") + std::wstring(timeStr) + L".jpg");
		}

		return retCode;
	}

	//
	// Pre and post resize methods.
	//

	int CGame::PreResize()
	{
		int retCode = Util::Err::NONE;
		
		if(m_pSceneMain && (retCode = m_pSceneMain->PreResize()) != Util::Err::NONE)
			return retCode;
		
		if((retCode = CAppBase::PreResize()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	int CGame::PostResize()
	{
		int retCode = Util::Err::NONE;
		
		if((retCode = CAppBase::PostResize()) != Util::Err::NONE)
			return retCode;
			
		if(m_pSceneMain && (retCode = m_pSceneMain->PostResize()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	// Method for releasing the objects in the game
	void CGame::Release()
	{
		SAFE_RELEASE_DELETE(m_pGameUI);
		SAFE_RELEASE_DELETE(m_pFBXLoader);
		SAFE_RELEASE_DELETE(m_pMatList);

		SAFE_RELEASE_DELETE(m_pSceneMain);

		CAppBase::Release();
	}

	int CGame::MessageHandler(MSG msg)
	{
		int retCode = Util::Err::NONE;

		return retCode;
	}
};