//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Project II
//
// File: Scenes\CMatList.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CMatList.h"

namespace GFX
{
	CMatList::CMatList()
	{
		m_pPlayerCamera						= NULL;

		m_pMatDebug							= NULL;
		m_pMatFT2Font						= NULL;
		m_pMatUIImage						= NULL;
		m_pMatFireEffect					= NULL;
	}

	CMatList::CMatList(const CMatList& other)
	{
	}

	CMatList::~CMatList()
	{
	}
	
	// Method for initializing the game material list.
	int CMatList::Initialize()
	{
		int retCode = Util::Err::NONE;

		// Allocate memory.
		m_pMatDebug							= new GFX::CMatDebug();
		m_pMatFT2Font						= new GFX::CMatFT2Font();
		m_pMatUIImage						= new GFX::CMatUIImage();
		m_pMatFireEffect					= new GFX::CMatFireEffect();
		
		// Setup the debug material.
		m_pMatDebug->DirectXData(m_pDX11Data);
		m_pMatDebug->RenderData(m_pRenderData);
		if(m_pPlayerCamera) m_pMatDebug->Camera(m_pPlayerCamera->Camera());

		if((retCode = m_pMatDebug->Initialize()) != Util::Err::NONE)
			return retCode;

		m_pMatDebug->m_debugCBuffer.color = Math::CColor(1);
		m_pMatDebug->m_debugCBuffer.scale = Math::CVector3::One();
		
		// Setup the freetype2 material.
		m_pMatFT2Font->DirectXData(m_pDX11Data);
		m_pMatFT2Font->RenderData(m_pRenderData);
		if(m_pPlayerCamera) m_pMatFT2Font->Camera(m_pPlayerCamera->UICamera());

		if((retCode = m_pMatFT2Font->Initialize()) != Util::Err::NONE)
			return retCode;

		m_pMatFT2Font->m_ft2CBuffer.color = Math::CColor(1);
		m_pMatFT2Font->m_ft2CBuffer.pointFilter = false;
		m_pMatFT2Font->m_ft2CBuffer.textured = true;

		// Setup the freetype2 material.
		m_pMatUIImage->DirectXData(m_pDX11Data);
		m_pMatUIImage->RenderData(m_pRenderData);
		if(m_pPlayerCamera) m_pMatUIImage->Camera(m_pPlayerCamera->UICamera());

		if((retCode = m_pMatUIImage->Initialize()) != Util::Err::NONE)
			return retCode;

		m_pMatUIImage->m_imageCBuffer.color = Math::CColor(1);
		m_pMatUIImage->m_imageCBuffer.pointFilter = false;
		m_pMatUIImage->m_imageCBuffer.textured = true;

		// Setup the fire effect material.
		m_pMatFireEffect->DirectXData(m_pDX11Data);
		m_pMatFireEffect->RenderData(m_pRenderData);
		if(m_pPlayerCamera) m_pMatFireEffect->Camera(m_pPlayerCamera->UICamera());

		if((retCode = m_pMatFireEffect->Initialize()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	// Method for setting the cameras for the material in the material list.
	void CMatList::SetCameras()
	{
		if(m_pPlayerCamera) m_pMatDebug->Camera(m_pPlayerCamera->Camera());
		if(m_pPlayerCamera) m_pMatFT2Font->Camera(m_pPlayerCamera->UICamera());
		if(m_pPlayerCamera) m_pMatUIImage->Camera(m_pPlayerCamera->UICamera());
		if(m_pPlayerCamera) m_pMatFireEffect->Camera(m_pPlayerCamera->UICamera());
	}

	// Method for releasing the material in the material list.
	void CMatList::Release()
	{
		SAFE_RELEASE_DELETE(m_pMatFireEffect);
		SAFE_RELEASE_DELETE(m_pMatUIImage);
		SAFE_RELEASE_DELETE(m_pMatFT2Font);
		SAFE_RELEASE_DELETE(m_pMatDebug);
	}
};