//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Project II
//
// File: Scenes\CMatList.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CMATLIST_H
#define CMATLIST_H

#include <Application\CDX11Object.h>

#include <Graphics\CMatDebug.h>
#include <Graphics\CMatFT2Font.h>
#include <Graphics\CMatUIImage.h>
#include "CMatFireEffect.h"

#include "..\Actors\CPlayerCamera.h"

namespace GFX
{
	// CMatList class.
	// Class that stores all of the materials that are used by this application, 
	// outside of the ones stored in the mesh scene object.
	class CMatList : public App::CDX11Object
	{
	public:
		CMatList();
		CMatList(const CMatList& other);
		virtual ~CMatList();

		int									Initialize();
		void								SetCameras();

		void								Release();
		
		inline RENDERDATA*					RenderData() const;
		inline void							RenderData(RENDERDATA* pRenderData);
		inline Actor::CPlayerCamera*		PlayerCamera() const;
		inline void							PlayerCamera(Actor::CPlayerCamera* pPlayerCamera);

		inline CMatDebug*					DebugMaterial() const;
		inline void							DebugMaterial(CMatDebug* pMatDebug);
		inline CMatFT2Font*					FT2Material() const;
		inline void							FT2Material(CMatFT2Font* m_pMatFT2Font);
		inline CMatUIImage*					UIImageMaterial() const;
		inline void							UIImageMaterial(CMatUIImage* m_pMatUIImage);
		inline CMatFireEffect*				FireMaterial() const;
		inline void							FireMaterial(CMatFireEffect* m_pMatFireEffect);

	private:
		RENDERDATA*							m_pRenderData;
		Actor::CPlayerCamera*				m_pPlayerCamera;

		CMatDebug*							m_pMatDebug;
		CMatFT2Font*						m_pMatFT2Font;
		CMatUIImage*						m_pMatUIImage;
		CMatFireEffect*						m_pMatFireEffect;
	};
	
	// Render data getter and setter.

	inline RENDERDATA* CMatList::RenderData() const
	{
		return m_pRenderData;
	}

	inline void CMatList::RenderData(RENDERDATA* pRenderData)
	{
		m_pRenderData = pRenderData;
	}

	// Player camera getter and setter.

	inline Actor::CPlayerCamera* CMatList::PlayerCamera() const
	{
		return m_pPlayerCamera;
	}

	inline void CMatList::PlayerCamera(Actor::CPlayerCamera* pPlayerCamera)
	{
		m_pPlayerCamera = pPlayerCamera;
	}

	// Debug material getter and setter.

	inline CMatDebug* CMatList::DebugMaterial() const
	{
		return m_pMatDebug;
	}

	inline void CMatList::DebugMaterial(CMatDebug* pMatDebug)
	{
		m_pMatDebug = pMatDebug;
	}

	// FreeType2 material getter and setter.

	inline CMatFT2Font* CMatList::FT2Material() const
	{
		return m_pMatFT2Font;
	}

	inline void CMatList::FT2Material(CMatFT2Font* pMatFT2Font)
	{
		m_pMatFT2Font = pMatFT2Font;
	}

	// UI image material getter and setter.

	inline CMatUIImage* CMatList::UIImageMaterial() const
	{
		return m_pMatUIImage;
	}

	inline void CMatList::UIImageMaterial(CMatUIImage* pMatUIImage)
	{
		m_pMatUIImage = pMatUIImage;
	}

	// Fire particle material getter and setter.

	inline CMatFireEffect* CMatList::FireMaterial() const
	{
		return m_pMatFireEffect;
	}

	inline void CMatList::FireMaterial(CMatFireEffect* pMatFireEffect)
	{
		m_pMatFireEffect = pMatFireEffect;
	}
};

#endif