//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Project II
//
// File: Scenes\CScene.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CSceneMain.h"

namespace Scene
{
	CSceneMain::CSceneMain()
	{
		m_bDebugMode						= false;

		m_pElemDebug						= NULL;
		m_pPlayer							= NULL;

		m_pBackgroundAudioSource			= NULL;
		m_pBackgroundAudioVoice				= NULL;

		m_pMeshScene						= NULL;
		m_pParticleSystem					= NULL;

		for(u32 i = 0; i < BARREL_FIRES; i++)
			m_pBarrelFire[i]				= NULL;

		m_pFlashlight						= NULL;

		m_pTextureArrayTest					= NULL;
		m_pTextureArrayGradients			= NULL;

		m_pTextureProjection				= NULL;

		m_pMatList							= NULL;
		m_pGameUI							= NULL;
	}

	CSceneMain::CSceneMain(const CSceneMain& other)
	{
	}

	CSceneMain::~CSceneMain()
	{
	}

	// Method for preparing the scene for initializing.
	// This is where lights, projections, and particles are first created.
	int CSceneMain::PreInitialize()
	{
		int retCode = Util::Err::NONE;
		
		if((retCode = CScene::PreInitialize()) != Util::Err::NONE)
			return retCode;
			
		m_pLighting = new GFX::CLighting();
		m_pPostQuad->Lighting(m_pLighting);

		// Railing point lights.
		m_pLighting->AddPointLight(new GFX::POINT_LIGHT_QUEUE(Math::CColor(1, 1, 1), Math::CVector3(4.45f, 2.75f, 4.45f), 
			Math::CVector4(0.4f, 1.0f, 1.0f, 0.0f), 7.0f, 1.5f, 1, 0.5f));
			
		m_pLighting->AddPointLight(new GFX::POINT_LIGHT_QUEUE(Math::CColor(1, 1, 1), Math::CVector3(-4.45f, 2.75f, -4.45f), 
			Math::CVector4(0.4f, 1.0f, 1.0f, 0.0f), 7.0f, 1.5f, 1, 0.5f));

		m_pLighting->AddPointLight(new GFX::POINT_LIGHT_QUEUE(Math::CColor(1, 1, 1), Math::CVector3(4.45f, 2.75f, -4.45f), 
			Math::CVector4(0.4f, 1.0f, 1.0f, 0.0f), 7.0f, 1.5f, 0, 0.5f));
			
		m_pLighting->AddPointLight(new GFX::POINT_LIGHT_QUEUE(Math::CColor(1, 1, 1), Math::CVector3(-4.45f, 2.75f, 4.45f), 
			Math::CVector4(0.4f, 1.0f, 1.0f, 0.0f), 7.0f, 1.5f, 0, 0.5f));

		// Main gate point lights.
		m_pLighting->AddPointLight(new GFX::POINT_LIGHT_QUEUE(Math::CColor(1, 0.9f, 0.8f), Math::CVector3( 3.55f, 4.35f, 12.75f), 
			Math::CVector4(0.3f, 1.0f, 1.0f, 0.0f), 15.0f, 1.5f, 0, 0.5f));
			
		m_pLighting->AddPointLight(new GFX::POINT_LIGHT_QUEUE(Math::CColor(1, 0.9f, 0.8f), Math::CVector3(-3.55f, 4.35f, 12.75f), 
			Math::CVector4(0.3f, 1.0f, 1.0f, 0.0f), 15.0f, 1.5f, 0, 0.5f));

		m_pLighting->AddPointLight(new GFX::POINT_LIGHT_QUEUE(Math::CColor(1, 0.9f, 0.8f), Math::CVector3( 3.55f, 4.35f, -12.75f), 
			Math::CVector4(0.3f, 1.0f, 1.0f, 0.0f), 15.0f, 1.5f, 0, 0.5f));

		m_pLighting->AddPointLight(new GFX::POINT_LIGHT_QUEUE(Math::CColor(1, 0.9f, 0.8f), Math::CVector3(-3.55f, 4.35f, -12.75f), 
			Math::CVector4(0.3f, 1.0f, 1.0f, 0.0f), 15.0f, 1.5f, 0, 0.5f));

		// Watch tower point lights.
		m_pLighting->AddPointLight(new GFX::POINT_LIGHT_QUEUE(Math::CColor(1, 0.85f, 0.8f), Math::CVector3(-13.5f, 11.5f, -15.0f), 
			Math::CVector4(0.2f, 1.0f, 1.0f, 0.0f), 7.0f, 1.5f, 0, 0.5f));

		m_pLighting->AddPointLight(new GFX::POINT_LIGHT_QUEUE(Math::CColor(1, 0.85f, 0.8f), Math::CVector3(-13.5f, 11.5f,  15.0f), 
			Math::CVector4(0.2f, 1.0f, 1.0f, 0.0f), 7.0f, 1.5f, 0, 0.5f));

		m_pLighting->AddPointLight(new GFX::POINT_LIGHT_QUEUE(Math::CColor(1, 0.85f, 0.8f), Math::CVector3( 13.5f, 11.5f, -15.0f), 
			Math::CVector4(0.2f, 1.0f, 1.0f, 0.0f), 7.0f, 1.5f, 0, 0.5f));

		m_pLighting->AddPointLight(new GFX::POINT_LIGHT_QUEUE(Math::CColor(1, 0.85f, 0.8f), Math::CVector3( 13.5f, 11.5f,  15.0f), 
			Math::CVector4(0.2f, 1.0f, 1.0f, 0.0f), 7.0f, 1.5f, 0, 0.5f));
			
		// Flashlight object
		m_pFlashlight = new World::CFlashlight();
		m_pLighting->AddSpotlight((new GFX::SPOTLIGHT_QUEUE(Math::CColor(1.0f, 1.0f, 1.0f), Math::CVector3::Zero(), Math::CVector3::Zero(), 
			Math::CVector4(0.4f, 1.0f, 1.0f, 0.0f), 15.0f, 1.0f, 1, Math::_PiOver180 * 60.0f, Math::_PiOver180 * 40.0f, 1, 0.0f, m_pFlashlight->ppSpotlight())));
		
		// Fire object point lights.
		for(u32 i = 0; i < BARREL_FIRES; i++)
		{
			m_pBarrelFire[i] = new World::CFireObject();

			m_pLighting->AddPointLight((new GFX::POINT_LIGHT_QUEUE(Math::CColor(1, 0.6f, 0.35f), Math::CVector3(0.0f, 0.0f, 0.0f), 
				Math::CVector4(0.2f, 1.0f, 1.0f, 0.0f), 4.0f, 1.0f, 0, 0.5f, m_pBarrelFire[i]->ppPointLight())));
		}
		
		// Texture projection.
		m_pTextureProjection = new GFX::CTextureProjection();
		m_pPostQuad->TextureProjection(m_pTextureProjection);
		
		return retCode;
	}

	// Method for initialzing the main scene.
	// Most of the objects seen in the game are created here.
	int CSceneMain::Initialize()
	{
		int retCode = Util::Err::NONE;

		if((retCode = CScene::Initialize()) != Util::Err::NONE)
			return retCode;

		// Allocate memory.
		m_pElemDebug						= new CElemDebug();
		m_pPlayer							= new Actor::CPlayer();

		m_pSkydome							= new GFX::CSkydome();
		m_pMeshScene						= new GFX::CMeshScene();
		m_pParticleSystem					= new GFX::CParticleSystem();

		// Setup the player.
		m_pPlayer->DirectXData(m_pDX11Data);
		m_pPlayer->RenderData(m_pRenderData);
		m_pPlayer->Timer(m_pTimer);
		m_pPlayer->Input(m_pInput);
		m_pPlayer->WinData(m_pWinData);
		m_pPlayer->GameUI(m_pGameUI);
		if((retCode = m_pPlayer->Initialize()) != Util::Err::NONE)
			return retCode;

		m_pMatList->PlayerCamera(m_pPlayer->PlayerCamera());
		m_pMatList->SetCameras();
			
		m_pLighting->CurrentCamera(m_pPlayer->PlayerCamera()->Camera());
		m_pLighting->RenderData(m_pRenderData);
		((GFX::CMatPostScreen*)m_pPostQuad->Material())->WorldCamera(m_pPlayer->PlayerCamera()->Camera());

		//
		// Audio.
		//
		
		// Background ambience.
		m_pBackgroundAudioSource = Audio::CAudioList::Instance().AddAudioSource(".\\SFX\\Ambience\\NightSounds.wav");
			
		if(m_pBackgroundAudioSource)
		{
			m_pBackgroundAudioSource->Sound3D(false);
			m_pBackgroundAudioSource->Play(0.75f, true);
			m_pBackgroundAudioSource->Update();
		}

		//
		// Setup scene elements.
		//

		// Debug elements.
		m_pElemDebug->DirectXData(m_pDX11Data);
		m_pElemDebug->DebugMaterial(m_pMatList->DebugMaterial());
		m_pElemDebug->RenderData(m_pRenderData);
		if((retCode = m_pElemDebug->Initialize()) != Util::Err::NONE)
			return retCode;

		//
		// Skydome.
		//

		m_pSkydome->DirectXData(m_pDX11Data);
		m_pSkydome->WorldCamera(m_pPlayer->PlayerCamera()->Camera());
		m_pSkydome->RenderData(m_pRenderData);
		m_pSkydome->SkyMapTexture(GFX::CTextureList::Instance().AddTexture("GFX\\Skyboxes\\Unity Skyboxes\\StarryNight\\StarryNight.dds", true));

		if((retCode = m_pSkydome->Initialize()) != Util::Err::NONE)
			return retCode;

		m_pSkydome->SkydomeColor(Math::CColor(0.5f, 0.5f, 0.5f, 1.0f));

		//
		// Meshes.
		//

		if((retCode = m_pFBXLoader->LoadFBXFile(".\\GFX\\Mesh\\MainScene.fbx", m_pMeshScene, 0.0254f)) != Util::Err::NONE)
			return retCode;

		m_pMeshScene->DirectXData(m_pDX11Data);
		m_pMeshScene->RenderData(m_pRenderData);
		m_pMeshScene->Camera(m_pPlayer->PlayerCamera()->Camera());
		m_pMeshScene->DebugMaterial(m_pMatList->DebugMaterial());
		if((retCode = m_pMeshScene->Initialize()) != Util::Err::NONE)
			return retCode;

		m_pPlayer->PlayerMesh(m_pMeshScene->FindMesh("PlayerOrb000"));
			
		//
		// World objects.
		//

		GFX::CMesh* mesh;
		
		// Setup spotlight object.
		m_pFlashlight->RenderData(m_pRenderData);
		
		mesh = m_pMeshScene->FindMesh("Flashlight000");
		if(mesh)
		{
			m_pFlashlight->Transform(mesh->Transform());
		}

		m_pFlashlight->DirectXData(m_pDX11Data);
		if((retCode = m_pFlashlight->Initialize()) != Util::Err::NONE)
			return retCode;

		//
		// Particles.
		//

		m_pParticleSystem->Timer(m_pTimer);
		m_pParticleSystem->Camera(m_pPlayer->PlayerCamera()->Camera());
		m_pParticleSystem->DirectXData(m_pDX11Data);
		m_pParticleSystem->SetRenderData(m_pRenderData);

		// --- Initialize particle devices here.
		
		for(u32 i = 0; i < BARREL_FIRES; i++)
		{
			// Setup fire object.
			m_pBarrelFire[i]->Material(m_pMatList->FireMaterial());
			m_pBarrelFire[i]->RenderData(m_pRenderData);
			m_pBarrelFire[i]->ParticleSystem(m_pParticleSystem);
			m_pBarrelFire[i]->FireSoundFilename(".\\SFX\\Effects\\fire.wav");

			mesh = m_pMeshScene->FindMesh(std::string("MetalBarrelOpen00") + std::to_string(i));
			if(mesh)
			{
				m_pBarrelFire[i]->Transform(mesh->Transform());
				m_pBarrelFire[i]->PositionOffset(Math::CVector3(0, 1.25f, 0.0f));
			}

			m_pBarrelFire[i]->DirectXData(m_pDX11Data);
		
			if((retCode = m_pBarrelFire[i]->Initialize()) != Util::Err::NONE)
				return retCode;
		}

		// End particle device initialization ---

		if((retCode = m_pParticleSystem->Initialize()) != Util::Err::NONE)
			return retCode;
			
		m_pAudioEngine->ListenerTransform(m_pPlayer->PlayerCamera()->Camera()->Transform().get());

		KeyBinds();

		if((retCode = m_pLighting->Finalize()) != Util::Err::NONE)
			return retCode;
			
		return retCode;
	}

	// Method for updating the main scene.
	int CSceneMain::Update()
	{
		int retCode = Util::Err::NONE;

		if((retCode = CScene::Update()) != Util::Err::NONE)
			return retCode;

		if((retCode = m_pPlayer->Update()) != Util::Err::NONE)
			return retCode;
			
		if((retCode = m_pMeshScene->Update()) != Util::Err::NONE)
			return retCode;
		
		if((retCode = m_pFlashlight->Update()) != Util::Err::NONE)
			return retCode;

		for(u32 i = 0; i < BARREL_FIRES; i++)
		{
			if((retCode = m_pBarrelFire[i]->Update()) != Util::Err::NONE)
				return retCode;
		}

		if((retCode = m_pParticleSystem->Update()) != Util::Err::NONE)
			return retCode;
			
		if((retCode = m_pGameUI->Update()) != Util::Err::NONE)
			return retCode;

		if((retCode = m_pAudioEngine->Update()) != Util::Err::NONE)
			return retCode;

		// Update debug mode state.
		m_bDebugMode = m_pPlayer->PlayerHUD()->DebugToggle();

		m_pLighting->DebugMode(m_bDebugMode);
			
		return retCode;
	}

	// Method for preparing for render. 
	// This is where lighting is rendered.
	int CSceneMain::PreRender()
	{
		int retCode = Util::Err::NONE;

		if((retCode = CScene::PreRender()) != Util::Err::NONE)
			return retCode;
			
		if((retCode = m_pLighting->RenderMaps(this, RenderWorldObjects)) != Util::Err::NONE)
			return retCode;
			
		return retCode;
	}

	// Method for rendering the objects in the scene.
	int CSceneMain::Render()
	{
		int retCode = Util::Err::NONE;

		if((retCode = CScene::Render()) != Util::Err::NONE)
			return retCode;
			
		if((retCode = m_pSkydome->Render()) != Util::Err::NONE)
			return retCode;
		
		if((retCode = m_pPlayer->Render()) != Util::Err::NONE)
			return retCode;
			
		if((retCode = RenderWorldObjects(this, NULL)) != Util::Err::NONE)
			return retCode;
		
		if((retCode = m_pFlashlight->Render()) != Util::Err::NONE)
			return retCode;

		for(u32 i = 0; i < BARREL_FIRES; i++)
		{
			if((retCode = m_pBarrelFire[i]->Render()) != Util::Err::NONE)
				return retCode;
		}
		
		if(m_bDebugMode)
		{
			if((retCode = m_pElemDebug->Render()) != Util::Err::NONE)
				return retCode;
		}
			
		if((retCode = m_pPlayer->RenderHUD()) != Util::Err::NONE)
			return retCode;
		
		// Lighting render function is primarily for lighting debug elements.
		if((retCode = m_pLighting->Render()) != Util::Err::NONE)
			return retCode;

		if((retCode = m_pDXStates->SetBlend(App::BlendState::UI)) != Util::Err::NONE)
			return retCode;
			
		if((retCode = m_pGameUI->Render()) != Util::Err::NONE)
			return retCode;
			
		if((retCode = m_pDXStates->SetBlend(App::BlendState::Default)) != Util::Err::NONE)
			return retCode;
			
		return retCode;
	}

	// Method for post render. This is where particles are put in the world.
	int CSceneMain::PostRender()
	{
		int retCode = Util::Err::NONE;
		
		if((retCode = m_pParticleSystem->Render()) != Util::Err::NONE)
			return retCode;

		if(App::CControls::Instance().Activated("Quit"))
		{
			PostQuitMessage(0);
		}
		
		return retCode;
	}

	// Seperate method for world rendering so that shadow mapping 
	// has easy access to rendering the object in the world.
	int RenderWorldObjects(void* scene, GFX::CMaterial* material)
	{
		int retCode = Util::Err::NONE;
		
		// Meshes.
		
		if((retCode = ((CSceneMain*)scene)->m_pMeshScene->Render(material)) != Util::Err::NONE)
			return retCode;
		
		if((retCode = ((CSceneMain*)scene)->m_pMeshScene->RenderAlpha(material)) != Util::Err::NONE)
			return retCode;
			
		return retCode;
	}

	//
	// Pre and post resize methods.
	//

	int CSceneMain::PreResize()
	{
		int retCode = Util::Err::NONE;
		
		if(m_pPlayer && (retCode = m_pPlayer->PreResize()) != Util::Err::NONE)
			return retCode;

		if((retCode = CScene::PreResize()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	int CSceneMain::PostResize()
	{
		int retCode = Util::Err::NONE;

		if((retCode = CScene::PostResize()) != Util::Err::NONE)
			return retCode;

		if(m_pPlayer && (retCode = m_pPlayer->PostResize()) != Util::Err::NONE)
			return retCode;
			
		return retCode;
	}

	// Method for releaseng scene objects.
	void CSceneMain::Release()
	{
		for(u32 i = 0; i < BARREL_FIRES; i++)
			SAFE_RELEASE_DELETE(m_pBarrelFire[i]);
			
		SAFE_RELEASE_DELETE(m_pFlashlight);
		SAFE_RELEASE_DELETE(m_pParticleSystem);
		SAFE_RELEASE_DELETE(m_pMeshScene);
		SAFE_RELEASE_DELETE(m_pSkydome);
		
		SAFE_RELEASE_DELETE(m_pElemDebug);
		SAFE_RELEASE_DELETE(m_pPlayer);

		SAFE_RELEASE_DELETE(m_pTextureProjection);

		SAFE_RELEASE_DELETE(m_pLighting);

		CScene::Release();
	}

	// Method for creating keybinds for game.
	void CSceneMain::KeyBinds()
	{
		u32 index = App::CControls::Instance().KeyBindCount();

		// Player key binds
		App::CControls::Instance().AddKeyBind(App::KEY_BIND_OPTION(index++, "Forward", 
			App::KEY_BIND_OPTION::KEY_BIND(VK_KEY_W,		App::KEY_BIND_TYPE::Keyboard,	true),
			App::KEY_BIND_OPTION::KEY_BIND(VK_UP,			App::KEY_BIND_TYPE::Keyboard,	true)));

		App::CControls::Instance().AddKeyBind(App::KEY_BIND_OPTION(index++, "Backward", 
			App::KEY_BIND_OPTION::KEY_BIND(VK_KEY_S,		App::KEY_BIND_TYPE::Keyboard,	true),
			App::KEY_BIND_OPTION::KEY_BIND(VK_DOWN,			App::KEY_BIND_TYPE::Keyboard,	true)));
			
		App::CControls::Instance().AddKeyBind(App::KEY_BIND_OPTION(index++, "Right", 
			App::KEY_BIND_OPTION::KEY_BIND(VK_KEY_D,		App::KEY_BIND_TYPE::Keyboard,	true),
			App::KEY_BIND_OPTION::KEY_BIND(VK_RIGHT,		App::KEY_BIND_TYPE::Keyboard,	true)));

		App::CControls::Instance().AddKeyBind(App::KEY_BIND_OPTION(index++, "Left", 
			App::KEY_BIND_OPTION::KEY_BIND(VK_KEY_A,		App::KEY_BIND_TYPE::Keyboard,	true),
			App::KEY_BIND_OPTION::KEY_BIND(VK_LEFT,			App::KEY_BIND_TYPE::Keyboard,	true)));

		App::CControls::Instance().AddKeyBind(App::KEY_BIND_OPTION(index++, "Up", 
			App::KEY_BIND_OPTION::KEY_BIND(VK_KEY_Q,		App::KEY_BIND_TYPE::Keyboard,	true),
			App::KEY_BIND_OPTION::KEY_BIND(VK_NUMPAD1,		App::KEY_BIND_TYPE::Keyboard,	true)));

		App::CControls::Instance().AddKeyBind(App::KEY_BIND_OPTION(index++, "Down", 
			App::KEY_BIND_OPTION::KEY_BIND(VK_KEY_E,		App::KEY_BIND_TYPE::Keyboard,	true),
			App::KEY_BIND_OPTION::KEY_BIND(VK_NUMPAD0,		App::KEY_BIND_TYPE::Keyboard,	true)));

		App::CControls::Instance().AddKeyBind(App::KEY_BIND_OPTION(index++, "Screenshot", 
			App::KEY_BIND_OPTION::KEY_BIND(VK_F3,		App::KEY_BIND_TYPE::Keyboard,	false),
			App::KEY_BIND_OPTION::KEY_BIND(VK_F3,		App::KEY_BIND_TYPE::Keyboard,	false)));

		App::CControls::Instance().AddKeyBind(App::KEY_BIND_OPTION(index++, "Quit", 
			App::KEY_BIND_OPTION::KEY_BIND(VK_ESCAPE,	App::KEY_BIND_TYPE::Keyboard,	false),
			App::KEY_BIND_OPTION::KEY_BIND(VK_ESCAPE,	App::KEY_BIND_TYPE::Keyboard,	false)));
	}
};