//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Project II
//
// File: Scenes\CSceneMain.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CSCENEMAIN_H
#define CSCENEMAIN_H

#include <Scenes\CScene.h>
#include <Application\CDXStates.h>
#include <Audio\CXAudio2.h>
#include <Graphics\CParticleSystem.h>
#include <Graphics\CTextureProjection.h>
#include <Graphics\CTextureProjector.h>
#include "..\Actors\CPlayer.h"
#include "..\Graphics\CMatList.h"
#include "..\World\CFireObject.h"
#include "..\World\CFlashlight.h"
#include "CElemDebug.h"
#include "..\UI\CGameUI.h"

namespace Scene
{
	const u32								BARREL_FIRES							= 3;

	// CSceneMain class.
	// Class that acts as the main scene for this application.
	class CSceneMain : public CScene
	{
	public:
		CSceneMain();
		CSceneMain(const CSceneMain& other);
		virtual ~CSceneMain();

		int									PreInitialize() override;
		int									Initialize() override;
		int									Update() override;
		int									PreRender() override;
		int									Render() override;
		int									PostRender() override;

		int									PreResize() override;
		int									PostResize() override;

		void								Release() override;
		
		inline GFX::CMatList*				MatList() const;
		inline void							MatList(GFX::CMatList* pMatList);
		inline UI::CGameUI*					GameUI() const;
		inline void							GameUI(UI::CGameUI* pGameUI);

	private:
		void								KeyBinds();

		friend int							RenderWorldObjects(void* scene, GFX::CMaterial* material);
		
		bool								m_bDebugMode;

		CElemDebug*							m_pElemDebug;
		Actor::CPlayer*						m_pPlayer;

		Audio::CXAudio2Source*				m_pBackgroundAudioSource;
		Audio::CXAudio2Source::AUDIO_VOICE*	m_pBackgroundAudioVoice;
		
		GFX::CMeshScene*					m_pMeshScene;

		GFX::CParticleSystem*				m_pParticleSystem;
		World::CFireObject*					m_pBarrelFire[BARREL_FIRES];
		World::CFlashlight*					m_pFlashlight;

		GFX::CTextureArray*					m_pTextureArrayTest;
		GFX::CTextureArray*					m_pTextureArrayGradients;

		GFX::CTextureProjection*			m_pTextureProjection;

		GFX::CMatList*						m_pMatList;
		UI::CGameUI*						m_pGameUI;
	};

	// Material list getter and setter.
		
	inline GFX::CMatList* CSceneMain::MatList() const
	{
		return m_pMatList;
	}

	inline void CSceneMain::MatList(GFX::CMatList* pMatList)
	{
		m_pMatList = pMatList;
	}

	// Game UI getter and setter.

	inline UI::CGameUI* CSceneMain::GameUI() const
	{
		return m_pGameUI;
	}

	inline void CSceneMain::GameUI(UI::CGameUI* pGameUI)
	{
		m_pGameUI = pGameUI;
	}
};

#endif