//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Project II
//
// File: UI\CGameUI.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CGameUI.h"

namespace UI
{
	CGameUI::CGameUI()
	{
		m_pMatFT2Font						= NULL;
		m_pMatUIImage						= NULL;

		m_pFreeType							= NULL;
		m_pFontRenderer						= NULL;

		m_pCheckboxBGImage					= NULL;
		m_pCheckboxFGImage					= NULL;

		m_pRenderData						= NULL;
	}

	CGameUI::CGameUI(const CGameUI& other)
	{
	}

	CGameUI::~CGameUI()
	{
	}

	// Method for initializing the game UI elements.
	int CGameUI::Initialize()
	{
		int retCode = Util::Err::NONE;

		// Allocate memory.
		m_pFreeType							= new UI::CFreeType();
		m_pFontRenderer						= new UI::CFontRenderer();

		m_pCheckboxBGImage					= new UI::CImage();
		m_pCheckboxFGImage					= new UI::CImage();

		m_pFreeType->DirectXData(m_pDX11Data);
		m_pFreeType->FontFilename("GFX\\UI\\Fonts\\corbel.ttf");
		if((retCode = m_pFreeType->Initialize(false)) != Util::Err::NONE)
			return retCode;

		m_pFreeType->OpenFontFile("GFX\\UI\\Images\\corbel.data");

		// Setup the font renderer.
		m_pFontRenderer->DirectXData(m_pDX11Data);
		m_pFontRenderer->FreeType(m_pFreeType);
		m_pFontRenderer->Material(m_pMatFT2Font);
		m_pFontRenderer->RenderData(m_pRenderData);
		m_pFontRenderer->Transform(Math::CTransformSharedPtr(new Math::CTransform()));
		m_pFontRenderer->Transform()->Update();

		if((retCode = m_pFontRenderer->Initialize()) != Util::Err::NONE)
			return retCode;

		//
		// Setup default UI images.
		//

		// Default checkbox background image.
		m_pCheckboxBGImage->Material(m_pMatUIImage);
		m_pCheckboxBGImage->RenderData(m_pRenderData);
		m_pCheckboxBGImage->DirectXData(m_pDX11Data);
		m_pCheckboxBGImage->SetupImage(GFX::CTextureList::Instance().AddTexture(".\\GFX\\Textures\\UI\\CheckboxBG.png"), 10);
		m_pCheckboxBGImage->Transform(Math::CTransformSharedPtr(new Math::CTransform()));
		m_pCheckboxBGImage->Transform()->Update();

		if((retCode = m_pCheckboxBGImage->Initialize()) != Util::Err::NONE)
			return retCode;

		// Default checkbox foreground image.
		m_pCheckboxFGImage->Material(m_pMatUIImage);
		m_pCheckboxFGImage->RenderData(m_pRenderData);
		m_pCheckboxFGImage->DirectXData(m_pDX11Data);
		m_pCheckboxFGImage->SetupImage(GFX::CTextureList::Instance().AddTexture(".\\GFX\\Textures\\UI\\CheckboxFG.png"), 10);
		m_pCheckboxFGImage->Transform(Math::CTransformSharedPtr(new Math::CTransform()));
		m_pCheckboxFGImage->Transform()->Update();

		if((retCode = m_pCheckboxFGImage->Initialize()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}

	// Method for updating the game UI elements.
	int CGameUI::Update()
	{
		int retCode = Util::Err::NONE;

		if((retCode = m_pFontRenderer->Update()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}
	
	// Method for rendering the game UI elements.
	int CGameUI::Render()
	{
		int retCode = Util::Err::NONE;
		
		if((retCode = m_pCheckboxFGImage->Render()) != Util::Err::NONE) return retCode;
		if((retCode = m_pCheckboxBGImage->Render()) != Util::Err::NONE) return retCode;

		if((retCode = m_pFontRenderer->Render()) != Util::Err::NONE)
			return retCode;

		return retCode;
	}
	
	// Method for releasing the game UI elements.
	void CGameUI::Release()
	{
		SAFE_RELEASE_DELETE(m_pCheckboxFGImage);
		SAFE_RELEASE_DELETE(m_pCheckboxBGImage);

		SAFE_RELEASE_DELETE(m_pFontRenderer);
		SAFE_RELEASE_DELETE(m_pFreeType);
	}
};