//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Project II
//
// File: UI\CPlayerHUD.cpp
//
//-------------------------------------------------------------------------------------------------

#include "CPlayerHUD.h"

namespace UI
{
	CPlayerHUD::CPlayerHUD()
	{
		m_bDebugToggle						= false;
		m_bActivateHUD						= false;
		
		m_bPlayerLooked						= false;
		m_bPlayerMoved						= false;

		m_hudFade							= 1.0f;
		m_fadeSpeed							= 1.0f;

		m_virtualMemUsage					= 0;
		m_physicalMemUsage					= 0;

		m_meshCount							= 0;
		m_polyCount							= 0;
		m_vertCount							= 0;

		m_pLabel							= NULL;

		m_pSystemPanelCheckbox				= NULL;
		m_pRenderPanelCheckbox				= NULL;
		m_pDebugCheckbox					= NULL;

		m_pGameUI							= NULL;

		m_pRenderData						= NULL;
		m_pInput							= NULL;
		m_pTimer							= NULL;
	}

	CPlayerHUD::CPlayerHUD(const CPlayerHUD& other)
	{
	}

	CPlayerHUD::~CPlayerHUD()
	{
	}

	// Method for initialize the player HUD. Primarily for initializing UI object for the HUD.
	int CPlayerHUD::Initialize()
	{
		int retCode = Util::Err::NONE;

		// Allocate memory.
		m_pLabel							= new UI::CLabel();
		m_pSystemPanelCheckbox				= new UI::CCheckbox();
		m_pRenderPanelCheckbox				= new UI::CCheckbox();
		m_pDebugCheckbox					= new UI::CCheckbox();
		
		// Setup UI elements.

		// Setup the label.
		m_pLabel->DirectXData(m_pDX11Data);
		m_pLabel->FontRenderer(m_pGameUI->FontRenderer());
		
		// Setup the system panel checkbox.
		if((retCode = m_pSystemPanelCheckbox->Initialize()) != Util::Err::NONE)
			return retCode;

		m_pSystemPanelCheckbox->Background(m_pGameUI->CheckboxBGImage());
		m_pSystemPanelCheckbox->CheckIcon(m_pGameUI->CheckboxFGImage());
		m_pSystemPanelCheckbox->Checked(false);
		m_pSystemPanelCheckbox->FontRenderer(m_pGameUI->FontRenderer());
		m_pSystemPanelCheckbox->DirectXData(m_pDX11Data);
		m_pSystemPanelCheckbox->SetupCheckbox(m_pInput);
		m_pSystemPanelCheckbox->SetupCheckboxText("System panel ", 3.5f, Math::CColor(1), true);
		m_pSystemPanelCheckbox->SetupCheckIcon(Math::CColor(1));

		// Setup the render panel checkbox.
		if((retCode = m_pRenderPanelCheckbox->Initialize()) != Util::Err::NONE)
			return retCode;

		m_pRenderPanelCheckbox->Background(m_pGameUI->CheckboxBGImage());
		m_pRenderPanelCheckbox->CheckIcon(m_pGameUI->CheckboxFGImage());
		m_pRenderPanelCheckbox->Checked(false);
		m_pRenderPanelCheckbox->FontRenderer(m_pGameUI->FontRenderer());
		m_pRenderPanelCheckbox->DirectXData(m_pDX11Data);
		m_pRenderPanelCheckbox->SetupCheckbox(m_pInput);
		m_pRenderPanelCheckbox->SetupCheckboxText("Render panel ", 3.5f, Math::CColor(1), true);
		m_pRenderPanelCheckbox->SetupCheckIcon(Math::CColor(1));

		// Setup the debug checkbox.
		if((retCode = m_pDebugCheckbox->Initialize()) != Util::Err::NONE)
			return retCode;

		m_pDebugCheckbox->Background(m_pGameUI->CheckboxBGImage());
		m_pDebugCheckbox->CheckIcon(m_pGameUI->CheckboxFGImage());
		m_pDebugCheckbox->Checked(false);
		m_pDebugCheckbox->FontRenderer(m_pGameUI->FontRenderer());
		m_pDebugCheckbox->DirectXData(m_pDX11Data);
		m_pDebugCheckbox->SetupCheckbox(m_pInput);
		m_pDebugCheckbox->SetupCheckboxText("Debug elements ", 3.5f, Math::CColor(1), true);
		m_pDebugCheckbox->SetupCheckIcon(Math::CColor(1));

		return retCode;
	}
	
	// Method for updating the player HUD.
	int CPlayerHUD::Update()
	{
		int retCode = Util::Err::NONE;

		m_bDebugToggle = m_pDebugCheckbox->Checked();

		m_hudFade = Math::MoveTowards(m_hudFade, m_bActivateHUD ? 1.0f : 0.0f, m_pTimer->Delta());

		m_meshCount		= m_pRenderData->geoRenderCount;
		m_polyCount		= m_pRenderData->polyRenderCount;
		m_vertCount		= m_pRenderData->vertRenderCount;

		return retCode;
	}
	
	// Method for rendering the player HUD.
	int CPlayerHUD::Render()
	{
		int retCode = Util::Err::NONE;
		
		m_pLabel->Fade(1.0f);
		float colorValue = fabsf(sinf(m_pTimer->Time() * 3) * 0.25f) + 0.75f;
		
		if(!m_bPlayerLooked)
		{
			if((retCode = m_pLabel->PrintLabel("Hold right mouse to look around.", Math::CVector2(0.0f, 10.0f), 4.0f, Math::CColor(colorValue), UI::UI_HALIGN::Center, UI::UI_VALIGN::Bottom)) != Util::Err::NONE)
				return retCode;
		}

		if(!m_bPlayerMoved)
		{
			if((retCode = m_pLabel->PrintLabel("Use 'WASD' or 'Up Left Down Right' to move.", Math::CVector2(0.0f, 5.0f), 4.0f, Math::CColor((1.0f - colorValue) + 0.75f), UI::UI_HALIGN::Center, UI::UI_VALIGN::Bottom)) != Util::Err::NONE)
				return retCode;
		}

		float yIncrement = 4.0f;
		float offsetY = 0.0f;
		
		m_pLabel->Fade(m_hudFade);

		if((retCode = m_pLabel->PrintLabel("Development Display", Math::CVector2(5.0f, offsetY += yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Left, UI::UI_VALIGN::Top)) != Util::Err::NONE)
			return retCode;
			
		// System panel checkbox.
		m_pSystemPanelCheckbox->Fade(m_hudFade);
		if((retCode = m_pSystemPanelCheckbox->DisplayCheckbox(Math::CVector2(5.0f, offsetY += yIncrement), Math::CVector2(2.5f, 2.5f), Math::CColor(1), UI::UI_HALIGN::Left, UI::UI_VALIGN::Top)) != Util::Err::NONE)
			return retCode;

		// Render panel checkbox.
		m_pRenderPanelCheckbox->Fade(m_hudFade);
		if((retCode = m_pRenderPanelCheckbox->DisplayCheckbox(Math::CVector2(5.0f, offsetY += yIncrement), Math::CVector2(2.5f, 2.5f), Math::CColor(1), UI::UI_HALIGN::Left, UI::UI_VALIGN::Top)) != Util::Err::NONE)
			return retCode;

		// Debug checkbox.
		m_pDebugCheckbox->Fade(m_hudFade);
		if((retCode = m_pDebugCheckbox->DisplayCheckbox(Math::CVector2(5.0f, offsetY += yIncrement), Math::CVector2(2.5f, 2.5f), Math::CColor(1), UI::UI_HALIGN::Left, UI::UI_VALIGN::Top)) != Util::Err::NONE)
			return retCode;

		offsetY = 0.0f;
		m_pLabel->Fade(1.0f);
		
		// System panel.
		if(m_pSystemPanelCheckbox->Checked())
		{
			Util::CSysInfo::Instance().CalculateCPUUsage();
			Util::CSysInfo::Instance().CalculateMemoryUsage(&m_virtualMemUsage, &m_physicalMemUsage);
			
			if((retCode = m_pLabel->PrintLabel((char*)std::string("System panel").c_str(), Math::CVector2(15.0f, offsetY += yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Top)) != Util::Err::NONE)
				return retCode;
				
			//
			// Descriptions.
			//

			offsetY += yIncrement * 0.5f;
			
			// Memory totals.
			if((retCode = m_pLabel->PrintLabel("Total vRAM: ", Math::CVector2(15.0f, offsetY += yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Top)) != Util::Err::NONE)
				return retCode;
			if((retCode = m_pLabel->PrintLabel("Total RAM: ", Math::CVector2(15.0f, offsetY += yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Top)) != Util::Err::NONE)
				return retCode;
			
			// Memory usage totals.
			if((retCode = m_pLabel->PrintLabel("Total vRAM usage: ", Math::CVector2(15.0f, offsetY += yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Top)) != Util::Err::NONE)
				return retCode;
			if((retCode = m_pLabel->PrintLabel("Total RAM usage: ", Math::CVector2(15.0f, offsetY += yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Top)) != Util::Err::NONE)
				return retCode;
			
			// Process memory usage totals.
			if((retCode = m_pLabel->PrintLabel("Process vRAM usage: ", Math::CVector2(15.0f, offsetY += yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Top)) != Util::Err::NONE)
				return retCode;
			if((retCode = m_pLabel->PrintLabel("Process RAM usage: ", Math::CVector2(15.0f, offsetY += yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Top)) != Util::Err::NONE)
				return retCode;

			offsetY += yIncrement * 0.5f;
			
			// CPU usage.
			if((retCode = m_pLabel->PrintLabel("Total CPU usage: ", Math::CVector2(15.0f, offsetY += yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Top)) != Util::Err::NONE)
				return retCode;
			if((retCode = m_pLabel->PrintLabel("Process CPU usage: ", Math::CVector2(15.0f, offsetY += yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Top)) != Util::Err::NONE)
				return retCode;

			offsetY += yIncrement * 0.5f;
			
			// Video card info.
			if((retCode = m_pLabel->PrintLabel(m_pDX11Data->videoCardDesc, Math::CVector2(15.0f, offsetY += yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Top)) != Util::Err::NONE)
				return retCode;

			if((retCode = m_pLabel->PrintLabel("Total GPU memory: ", Math::CVector2(15.0f, offsetY += yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Top)) != Util::Err::NONE)
				return retCode;

			//
			// Values.
			//

			offsetY = yIncrement * 1.5f;

			// Memory totals.
			if((retCode = m_pLabel->PrintLabel((char*)(std::to_string(Util::CSysInfo::Instance().TotalVMemory()) + "MB").c_str(), Math::CVector2(5.0f, offsetY += yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Top)) != Util::Err::NONE)
				return retCode;
			if((retCode = m_pLabel->PrintLabel((char*)(std::to_string(Util::CSysInfo::Instance().TotalPMemory()) + "MB").c_str(), Math::CVector2(5.0f, offsetY += yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Top)) != Util::Err::NONE)
				return retCode;
			
			// Memory usage totals.
			if((retCode = m_pLabel->PrintLabel((char*)(std::to_string(Util::CSysInfo::Instance().TotalVMemoryUsed()) + "MB").c_str(), Math::CVector2(5.0f, offsetY += yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Top)) != Util::Err::NONE)
				return retCode;
			if((retCode = m_pLabel->PrintLabel((char*)(std::to_string(Util::CSysInfo::Instance().TotalPMemoryUsed()) + "MB").c_str(), Math::CVector2(5.0f, offsetY += yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Top)) != Util::Err::NONE)
				return retCode;
			
			// Process memory usage totals.
			if((retCode = m_pLabel->PrintLabel((char*)(std::to_string(m_virtualMemUsage) + "MB").c_str(), Math::CVector2(5.0f, offsetY += yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Top)) != Util::Err::NONE)
				return retCode;
			if((retCode = m_pLabel->PrintLabel((char*)(std::to_string(m_physicalMemUsage) + "MB").c_str(), Math::CVector2(5.0f, offsetY += yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Top)) != Util::Err::NONE)
				return retCode;

			offsetY += yIncrement * 0.5f;

			// CPU usage.
			if((retCode = m_pLabel->PrintLabel((char*)(std::to_string((int)Util::CSysInfo::Instance().TotalCPUUsage()) + "%").c_str(), Math::CVector2(5.0f, offsetY += yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Top)) != Util::Err::NONE)
				return retCode;

			if((retCode = m_pLabel->PrintLabel((char*)(std::to_string((int)Util::CSysInfo::Instance().ProcessCPUUsage()) + "%").c_str(), Math::CVector2(5.0f, offsetY += yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Top)) != Util::Err::NONE)
				return retCode;

			offsetY += yIncrement * 1.5f;

			// Video card info.
			
			if((retCode = m_pLabel->PrintLabel((char*)(std::to_string((int)m_pDX11Data->videoCardMemory) + "MB").c_str(), Math::CVector2(5.0f, offsetY += yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Top)) != Util::Err::NONE)
				return retCode;
		}

		// Render panel.
		if(m_pRenderPanelCheckbox->Checked())
		{
			offsetY = yIncrement * 6;
			
			if((retCode = m_pLabel->PrintLabel((char*)std::string("Render panel").c_str(), Math::CVector2(15.0f, offsetY -= yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Bottom)) != Util::Err::NONE)
				return retCode;
			
			//
			// Descriptions.
			//

			offsetY -= yIncrement * 0.5f;
			
			if((retCode = m_pLabel->PrintLabel((char*)std::string("Meshes: ").c_str(), Math::CVector2(15.0f, offsetY -= yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Bottom)) != Util::Err::NONE)
				return retCode;
			
			if((retCode = m_pLabel->PrintLabel((char*)std::string("Polygons: ").c_str(), Math::CVector2(15.0f, offsetY -= yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Bottom)) != Util::Err::NONE)
				return retCode;
			
			if((retCode = m_pLabel->PrintLabel((char*)std::string("Vertices: ").c_str(), Math::CVector2(15.0f, offsetY -= yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Bottom)) != Util::Err::NONE)
				return retCode;
			
			//
			// Values.
			//
			
			offsetY = yIncrement * 4.5f;
			
			if((retCode = m_pLabel->PrintLabel((char*)std::to_string(m_meshCount).c_str(), Math::CVector2(5.0f, offsetY -= yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Bottom)) != Util::Err::NONE)
				return retCode;
			
			if((retCode = m_pLabel->PrintLabel((char*)std::to_string(m_polyCount).c_str(), Math::CVector2(5.0f, offsetY -= yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Bottom)) != Util::Err::NONE)
				return retCode;
			
			if((retCode = m_pLabel->PrintLabel((char*)std::to_string(m_vertCount).c_str(), Math::CVector2(5.0f, offsetY -= yIncrement), 3.5f, Math::CColor(1.0f), UI::UI_HALIGN::Right, UI::UI_VALIGN::Bottom)) != Util::Err::NONE)
				return retCode;
		}

		return retCode;
	}
	
	// Method for releasing the object in the player HUD.
	void CPlayerHUD::Release()
	{
		SAFE_RELEASE_DELETE(m_pDebugCheckbox);
		SAFE_RELEASE_DELETE(m_pRenderPanelCheckbox);
		SAFE_RELEASE_DELETE(m_pSystemPanelCheckbox);
		SAFE_RELEASE_DELETE(m_pLabel);
	}
};