//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Project II
//
// File: UI\CPlayerHUD.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CPLAYERHUD_H
#define CPLAYERHUD_H

#include <Application\CDX11Object.h>
#include <Application\CInputControls.h>
#include <Utilities\CDebugError.h>
#include <Utilities\CSystemInfo.h>
#include <UI\CUILabel.h>
#include <UI\CUICheckbox.h>
#include <UI\CUISlider.h>
#include "..\UI\CGameUI.h"

namespace UI
{
	// CPlayerHUD class.
	// Class for creating the player HUD. In this case it's primarily for debugging infomation.
	class CPlayerHUD : public App::CDX11Object
	{
	public:
		CPlayerHUD();
		CPlayerHUD(const CPlayerHUD& other);
		virtual ~CPlayerHUD();

		int									Initialize();
		int									Update();
		int									Render();
		void								Release();

		inline bool							DebugToggle() const;
		inline void							DebugToggle(bool bDebugToggle);
		
		inline bool							ActivateHUD() const;
		inline void							ActivateHUD(bool bActivateHUD);

		inline bool							PlayerLooked() const;
		inline void							PlayerLooked(bool bPlayerLooked);

		inline bool							PlayerMoved() const;
		inline void							PlayerMoved(bool bPlayerMoved);

		inline float						FadeSpeed() const;
		inline void							FadeSpeed(float fadeSpeed);

		inline UI::CGameUI*					GameUI() const;
		inline void							GameUI(UI::CGameUI* pGameUI);

		inline GFX::RENDERDATA*				RenderData() const;
		inline void							RenderData(GFX::RENDERDATA* pRenderData);
		inline App::CInput*					Input() const;
		inline void							Input(App::CInput* pInput);
		inline Util::CTimer*				Timer() const;
		inline void							Timer(Util::CTimer* pTimer);

	private:
		bool								m_bDebugToggle;
		bool								m_bActivateHUD;
		
		bool								m_bPlayerLooked;
		bool								m_bPlayerMoved;

		float								m_hudFade;
		float								m_fadeSpeed;

		u32									m_virtualMemUsage;
		u32									m_physicalMemUsage;

		u32									m_meshCount;
		u32									m_polyCount;
		u32									m_vertCount;

		UI::CLabel*							m_pLabel;
		UI::CCheckbox*						m_pSystemPanelCheckbox;
		UI::CCheckbox*						m_pRenderPanelCheckbox;
		UI::CCheckbox*						m_pDebugCheckbox;
		UI::CSlider*						m_pSlider;

		UI::CGameUI*						m_pGameUI;

		GFX::RENDERDATA*					m_pRenderData;
		App::CInput*						m_pInput;
		Util::CTimer*						m_pTimer;
	};

	// Debug toggle getter and setter.

	inline bool CPlayerHUD::DebugToggle() const
	{
		return m_bDebugToggle;
	}

	inline void CPlayerHUD::DebugToggle(bool bDebugToggle)
	{
		m_bDebugToggle = bDebugToggle;
	}

	// HUD activated getter and setter.
		
	inline bool CPlayerHUD::ActivateHUD() const
	{
		return m_bActivateHUD;
	}

	inline void CPlayerHUD::ActivateHUD(bool bActivateHUD)
	{
		m_bActivateHUD = bActivateHUD;
	}

	// Player rotate camera getter and setter.

	inline bool CPlayerHUD::PlayerLooked() const
	{
		return m_bPlayerLooked;
	}

	inline void CPlayerHUD::PlayerLooked(bool bPlayerLooked)
	{
		m_bPlayerLooked = bPlayerLooked;
	}

	// Player translate camera getter and setter.

	inline bool CPlayerHUD::PlayerMoved() const
	{
		return m_bPlayerMoved;
	}

	inline void CPlayerHUD::PlayerMoved(bool bPlayerMoved)
	{
		m_bPlayerMoved = bPlayerMoved;
	}

	// HUD fade speed getter and setter.

	inline float CPlayerHUD::FadeSpeed() const
	{
		return m_fadeSpeed;
	}

	inline void CPlayerHUD::FadeSpeed(float fadeSpeed)
	{
		m_fadeSpeed = fadeSpeed;
	}

	// Game UI getter and setter.

	inline UI::CGameUI* CPlayerHUD::GameUI() const
	{
		return m_pGameUI;
	}

	inline void CPlayerHUD::GameUI(UI::CGameUI* pGameUI)
	{
		m_pGameUI = pGameUI;
	}

	// Render data getter and setter.

	inline GFX::RENDERDATA* CPlayerHUD::RenderData() const
	{
		return m_pRenderData;
	}

	inline void CPlayerHUD::RenderData(GFX::RENDERDATA* pRenderData)
	{
		m_pRenderData = pRenderData;
	}

	// Input getter and setter.

	inline App::CInput* CPlayerHUD::Input() const
	{
		return m_pInput;
	}

	inline void CPlayerHUD::Input(App::CInput* pInput)
	{
		m_pInput = pInput;
	}

	// Timer getter and setter.

	inline Util::CTimer* CPlayerHUD::Timer() const
	{
		return m_pTimer;
	}

	inline void CPlayerHUD::Timer(Util::CTimer* pTimer)
	{
		m_pTimer = pTimer;
	}
};

#endif