//-------------------------------------------------------------------------------------------------
//
// Copyright (c) 2015-2016 Ryan Alasandro
//
// Portfolio Project: Portfolio Project II
//
// File: Graphics\CFireObject.h
//
//-------------------------------------------------------------------------------------------------

#ifndef CFIREOBJECT_H
#define CFIREOBJECT_H

#include <Audio\CXAudio2.h>
#include <Audio\CAudioList.h>
#include <Graphics\CParticleSystem.h>
#include <Graphics\CLighting.h>
#include "..\Graphics\CParticleFire.h"
#include "..\Graphics\CMatFireEffect.h"
#include "CWorldObject.h"

namespace World
{
	// CFireObject class
	// object for creating a small fire effect in at a location.
	class CFireObject : public CWorldObject
	{
	public:
		CFireObject();
		CFireObject(const CFireObject& other);
		virtual ~CFireObject();

		int									Initialize() override;
		int									Update() override;
		void								Release() override;
		friend Math::CColor					ParticleFade(void* game, float lifetime);
		friend Math::CVector3				ParticleScale(void* game, float lifetime);

		inline Math::CVector3				PositionOffset();
		inline void							PositionOffset(const Math::CVector3& offset);

		inline void							FireSoundFilename(const std::string& fireSound);

		inline GFX::CParticleFire*			ParticleFire() const;
		inline void							ParticleFire(GFX::CParticleFire* pParticleFire);
		inline GFX::CParticleSystem*		ParticleSystem() const;
		inline void							ParticleSystem(GFX::CParticleSystem* pParticleSystem);

		inline GFX::POINT_LIGHT*			PointLight() const;
		inline GFX::POINT_LIGHT**			ppPointLight();
		inline void							PointLight(GFX::POINT_LIGHT* pPointLight);

	private:
		Math::CVector3						m_positionOffset;

		bool								m_bSoundStarted;
		std::string							m_fireSoundFilename;

		GFX::CParticleFire*					m_pParticleFire;
		GFX::CParticleSystem*				m_pParticleSystem;

		Audio::CXAudio2Source*				m_pFireSound;
		Audio::CXAudio2Source::AUDIO_VOICE*	m_pFireVoice;
		GFX::POINT_LIGHT*					m_pPointLight;
	};

	// Position offset getter and setter.

	inline Math::CVector3 CFireObject::PositionOffset()
	{
		return m_positionOffset;
	}

	inline void CFireObject::PositionOffset(const Math::CVector3& offset)
	{
		m_positionOffset = offset;
	}

	// Fire audio filename setter.
	inline void CFireObject::FireSoundFilename(const std::string& fireSound)
	{
		m_fireSoundFilename = fireSound;
	}

	// Fire particle effect getter and setter.

	inline GFX::CParticleFire* CFireObject::ParticleFire() const
	{
		return m_pParticleFire;
	}

	inline void CFireObject::ParticleFire(GFX::CParticleFire* pParticleFire)
	{
		m_pParticleFire = pParticleFire;
	}

	// Particle system getter and setter.

	inline GFX::CParticleSystem* CFireObject::ParticleSystem() const
	{
		return m_pParticleSystem;
	}

	inline void CFireObject::ParticleSystem(GFX::CParticleSystem* pParticleSystem)
	{
		m_pParticleSystem = pParticleSystem;
	}

	// Point light getter and setter.

	inline GFX::POINT_LIGHT* CFireObject::PointLight() const
	{
		return m_pPointLight;
	}

	inline GFX::POINT_LIGHT** CFireObject::ppPointLight()
	{
		return &m_pPointLight;
	}

	inline void CFireObject::PointLight(GFX::POINT_LIGHT* pPointLight)
	{
		m_pPointLight = pPointLight;
	}
};

#endif